@@ -1,13 +1,9 @@
 #pragma once
 #include "nlohmann/json_fwd.hpp"
-#include "pool/unit.hpp"
 #include "util/uuid.hpp"
 #include "util/uuid_ptr.hpp"
-#include <fstream>
-#include <map>
-#include <vector>
-
-#include "net_class.hpp"
+#include "util/uuid_vec.hpp"
+#include "pool/unit.hpp"
 
 namespace horizon {
 using json = nlohmann::json;
@@ -26,15 +22,21 @@ public:
     PowerSymbolStyle power_symbol_style = PowerSymbolStyle::GND;
     bool power_symbol_name_visible = true;
 
-    uuid_ptr<NetClass> net_class;
+    uuid_ptr<class NetClass> net_class;
     uuid_ptr<Net> diffpair;
     bool diffpair_master = false;
 
+    bool is_port = false;
+    Pin::Direction port_direction = Pin::Direction::BIDIRECTIONAL;
+
+
     // not saved
     bool is_power_forced = false;
     bool is_bussed = false;
     unsigned int n_pins_connected = 0;
     bool has_bus_rippers = false;
+    std::vector<UUIDVec> hrefs;
+    bool is_nc = false;
     json serialize() const;
     bool is_named() const;
 };
