@@ -18,6 +18,7 @@ BoardRules::BoardRules(const BoardRules &other)
       rule_clearance_copper_other(other.rule_clearance_copper_other), rule_plane(other.rule_plane),
       rule_diffpair(other.rule_diffpair), rule_clearance_copper_keepout(other.rule_clearance_copper_keepout),
       rule_layer_pair(other.rule_layer_pair), rule_clearance_same_net(other.rule_clearance_same_net),
+      rule_shorted_pads(other.rule_shorted_pads),
       rule_clearance_silkscreen_exposed_copper(other.rule_clearance_silkscreen_exposed_copper),
       rule_parameters(other.rule_parameters), rule_preflight_checks(other.rule_preflight_checks)
 {
@@ -36,6 +37,7 @@ void BoardRules::operator=(const BoardRules &other)
     rule_clearance_copper_keepout = other.rule_clearance_copper_keepout;
     rule_layer_pair = other.rule_layer_pair;
     rule_clearance_same_net = other.rule_clearance_same_net;
+    rule_shorted_pads = other.rule_shorted_pads;
     rule_clearance_silkscreen_exposed_copper = other.rule_clearance_silkscreen_exposed_copper;
     rule_parameters = other.rule_parameters;
     rule_preflight_checks = other.rule_preflight_checks;
@@ -150,6 +152,14 @@ void BoardRules::import_rules(const json &j, const RuleImportMap &import_map)
         }
         fix_order(RuleID::CLEARANCE_SAME_NET);
     }
+    if (j.count("shorted_pads")) {
+        for (const auto &[key, value] : j.at("shorted_pads").items()) {
+            UUID u = key;
+            rule_shorted_pads.emplace(std::piecewise_construct, std::forward_as_tuple(u),
+                                      std::forward_as_tuple(u, value));
+        }
+        fix_order(RuleID::SHORTED_PADS);
+    }
     if (j.count("clearance_silkscreen_exposed_copper")) {
         const json &o = j["clearance_silkscreen_exposed_copper"];
         rule_clearance_silkscreen_exposed_copper = RuleClearanceSilkscreenExposedCopper(o, import_map);
@@ -191,6 +201,9 @@ void BoardRules::cleanup(const Block *block)
     for (auto &it : rule_clearance_same_net) {
         it.second.match.cleanup(block);
     }
+    for (auto &it : rule_shorted_pads) {
+        it.second.match.cleanup(block);
+    }
     for (auto &it : rule_diffpair) {
         if (!block->net_classes.count(it.second.net_class))
             it.second.net_class = block->net_class_default->uuid;
@@ -199,6 +212,7 @@ void BoardRules::cleanup(const Block *block)
 
 void BoardRules::apply(RuleID id, Board &brd, IPool &pool) const
 {
+    brd.rules = *this;
     if (id == RuleID::TRACK_WIDTH) {
         for (auto &it : brd.tracks) {
             auto &track = it.second;
@@ -234,6 +248,9 @@ void BoardRules::apply(RuleID id, Board &brd, IPool &pool) const
             }
         }
     }
+    else if (id == RuleID::SHORTED_PADS) {
+        brd.expand_flags |= Board::EXPAND_ALL_AIRWIRES;
+    }
 }
 
 json BoardRules::serialize_or_export(Rule::SerializeMode mode) const
@@ -256,6 +273,7 @@ json BoardRules::serialize_or_export(Rule::SerializeMode mode) const
     SERIALIZE(via);
     SERIALIZE(plane);
     SERIALIZE(diffpair);
+    SERIALIZE(shorted_pads);
     SERIALIZE(clearance_copper_other);
     SERIALIZE(clearance_copper_keepout);
     SERIALIZE(clearance_same_net);
@@ -273,64 +291,70 @@ json BoardRules::serialize() const
     return serialize_or_export(Rule::SerializeMode::SERIALIZE);
 }
 
-std::set<RuleID> BoardRules::get_rule_ids() const
+std::vector<RuleID> BoardRules::get_rule_ids() const
 {
     return {
-            RuleID::HOLE_SIZE,
-            RuleID::CLEARANCE_SILKSCREEN_EXPOSED_COPPER,
-            RuleID::TRACK_WIDTH,
             RuleID::CLEARANCE_COPPER,
-            RuleID::PARAMETERS,
-            RuleID::VIA,
             RuleID::CLEARANCE_COPPER_OTHER,
+            RuleID::CLEARANCE_COPPER_KEEPOUT,
+            RuleID::CLEARANCE_SAME_NET,
+            RuleID::CLEARANCE_SILKSCREEN_EXPOSED_COPPER,
+
+            RuleID::TRACK_WIDTH,
+            RuleID::HOLE_SIZE,
+
+            RuleID::VIA,
             RuleID::PLANE,
             RuleID::DIFFPAIR,
-            RuleID::PREFLIGHT_CHECKS,
-            RuleID::CLEARANCE_COPPER_KEEPOUT,
+            RuleID::PARAMETERS,
+
+            RuleID::SHORTED_PADS,
             RuleID::LAYER_PAIR,
-            RuleID::CLEARANCE_SAME_NET,
+            RuleID::PREFLIGHT_CHECKS,
     };
 }
 
-const Rule *BoardRules::get_rule(RuleID id) const
+const Rule &BoardRules::get_rule(RuleID id) const
 {
     if (id == RuleID::CLEARANCE_SILKSCREEN_EXPOSED_COPPER) {
-        return &rule_clearance_silkscreen_exposed_copper;
+        return rule_clearance_silkscreen_exposed_copper;
     }
     else if (id == RuleID::PARAMETERS) {
-        return &rule_parameters;
+        return rule_parameters;
     }
     else if (id == RuleID::PREFLIGHT_CHECKS) {
-        return &rule_preflight_checks;
+        return rule_preflight_checks;
     }
-    return nullptr;
+    throw std::runtime_error("rule does not exist");
 }
 
-const Rule *BoardRules::get_rule(RuleID id, const UUID &uu) const
+const Rule &BoardRules::get_rule(RuleID id, const UUID &uu) const
 {
     switch (id) {
     case RuleID::HOLE_SIZE:
-        return &rule_hole_size.at(uu);
+        return rule_hole_size.at(uu);
     case RuleID::TRACK_WIDTH:
-        return &rule_track_width.at(uu);
+        return rule_track_width.at(uu);
     case RuleID::CLEARANCE_COPPER:
-        return &rule_clearance_copper.at(uu);
+        return rule_clearance_copper.at(uu);
     case RuleID::VIA:
-        return &rule_via.at(uu);
+        return rule_via.at(uu);
     case RuleID::CLEARANCE_COPPER_OTHER:
-        return &rule_clearance_copper_other.at(uu);
+        return rule_clearance_copper_other.at(uu);
     case RuleID::PLANE:
-        return &rule_plane.at(uu);
+        return rule_plane.at(uu);
     case RuleID::DIFFPAIR:
-        return &rule_diffpair.at(uu);
+        return rule_diffpair.at(uu);
     case RuleID::CLEARANCE_COPPER_KEEPOUT:
-        return &rule_clearance_copper_keepout.at(uu);
+        return rule_clearance_copper_keepout.at(uu);
     case RuleID::LAYER_PAIR:
-        return &rule_layer_pair.at(uu);
+        return rule_layer_pair.at(uu);
     case RuleID::CLEARANCE_SAME_NET:
-        return &rule_clearance_same_net.at(uu);
+        return rule_clearance_same_net.at(uu);
+    case RuleID::SHORTED_PADS:
+        return rule_shorted_pads.at(uu);
     default:
-        return nullptr;
+        throw std::runtime_error("rule does not exist");
     }
 }
 
@@ -396,6 +420,12 @@ std::map<UUID, const Rule *> BoardRules::get_rules(RuleID id) const
         }
         break;
 
+    case RuleID::SHORTED_PADS:
+        for (auto &it : rule_shorted_pads) {
+            r.emplace(it.first, &it.second);
+        }
+        break;
+
     default:;
     }
     return r;
@@ -444,13 +474,17 @@ void BoardRules::remove_rule(RuleID id, const UUID &uu)
         rule_clearance_same_net.erase(uu);
         break;
 
+    case RuleID::SHORTED_PADS:
+        rule_shorted_pads.erase(uu);
+        break;
+
     default:;
     }
     fix_order(id);
     update_sorted();
 }
 
-Rule *BoardRules::add_rule(RuleID id)
+Rule &BoardRules::add_rule(RuleID id)
 {
     auto uu = UUID::random();
     Rule *r = nullptr;
@@ -495,22 +529,26 @@ Rule *BoardRules::add_rule(RuleID id)
         r = &rule_clearance_same_net.emplace(uu, uu).first->second;
         break;
 
+    case RuleID::SHORTED_PADS:
+        r = &rule_shorted_pads.emplace(uu, uu).first->second;
+        break;
+
     default:
-        return nullptr;
+        throw std::runtime_error("unsupported rule");
     }
     fix_order(id);
     update_sorted();
-    return r;
+    return *r;
 }
 
 void BoardRules::update_sorted()
 {
-    rule_sorted_clearance_copper = get_rules_sorted<const RuleClearanceCopper>(RuleID::CLEARANCE_COPPER);
+    rule_sorted_clearance_copper = get_rules_sorted<const RuleClearanceCopper>();
 }
 
 uint64_t BoardRules::get_default_track_width(const Net *net, int layer) const
 {
-    auto rules = get_rules_sorted<RuleTrackWidth>(RuleID::TRACK_WIDTH);
+    auto rules = get_rules_sorted<RuleTrackWidth>();
     for (auto rule : rules) {
         if (rule->enabled && rule->match.match(net)) {
             if (rule->widths.count(layer)) {
@@ -540,7 +578,7 @@ static const RuleClearanceCopperOther fallback_clearance_copper_non_copper = UUI
 
 const RuleClearanceCopperOther *BoardRules::get_clearance_copper_other(const Net *net, int layer) const
 {
-    auto rules = get_rules_sorted<RuleClearanceCopperOther>(RuleID::CLEARANCE_COPPER_OTHER);
+    auto rules = get_rules_sorted<RuleClearanceCopperOther>();
     for (auto ru : rules) {
         if (ru->enabled && (ru->match.match(net) && (ru->layer == layer || ru->layer == 10000))) {
             return ru;
@@ -553,7 +591,7 @@ static const RuleDiffpair diffpair_fallback = UUID();
 
 const RuleDiffpair *BoardRules::get_diffpair(const NetClass *net_class, int layer) const
 {
-    auto rules = get_rules_sorted<RuleDiffpair>(RuleID::DIFFPAIR);
+    auto rules = get_rules_sorted<RuleDiffpair>();
     for (auto ru : rules) {
         if (ru->enabled && (ru->net_class == net_class->uuid && (ru->layer == layer || ru->layer == 10000))) {
             return ru;
@@ -567,7 +605,7 @@ static const RuleClearanceCopperKeepout keepout_fallback = UUID();
 const RuleClearanceCopperKeepout *BoardRules::get_clearance_copper_keepout(const Net *net,
                                                                            const KeepoutContour *contour) const
 {
-    auto rules = get_rules_sorted<RuleClearanceCopperKeepout>(RuleID::CLEARANCE_COPPER_KEEPOUT);
+    auto rules = get_rules_sorted<RuleClearanceCopperKeepout>();
     for (auto ru : rules) {
         if (ru->enabled && ru->match.match(net) && ru->match_keepout.match(contour)) {
             return ru;
@@ -579,7 +617,7 @@ static const RuleClearanceSameNet same_net_fallback = UUID();
 
 const RuleClearanceSameNet *BoardRules::get_clearance_same_net(const Net *net, int layer) const
 {
-    auto rules = get_rules_sorted<RuleClearanceSameNet>(RuleID::CLEARANCE_SAME_NET);
+    auto rules = get_rules_sorted<RuleClearanceSameNet>();
     for (auto ru : rules) {
         if (ru->enabled && ru->match.match(net) && (ru->layer == layer || ru->layer == 10000)) {
             return ru;
@@ -593,7 +631,7 @@ uint64_t BoardRules::get_max_clearance() const
 {
     uint64_t max_clearance = 0;
     {
-        auto rules = get_rules_sorted<RuleClearanceCopper>(RuleID::CLEARANCE_COPPER);
+        auto rules = get_rules_sorted<RuleClearanceCopper>();
         for (auto ru : rules) {
             if (ru->enabled) {
                 max_clearance = std::max(max_clearance, ru->get_max_clearance());
@@ -601,7 +639,7 @@ uint64_t BoardRules::get_max_clearance() const
         }
     }
     {
-        auto rules = get_rules_sorted<RuleClearanceCopperOther>(RuleID::CLEARANCE_COPPER_OTHER);
+        auto rules = get_rules_sorted<RuleClearanceCopperOther>();
         for (auto ru : rules) {
             if (ru->enabled) {
                 max_clearance = std::max(max_clearance, ru->get_max_clearance());
@@ -609,7 +647,7 @@ uint64_t BoardRules::get_max_clearance() const
         }
     }
     {
-        auto rules = get_rules_sorted<RuleClearanceCopperKeepout>(RuleID::CLEARANCE_COPPER_KEEPOUT);
+        auto rules = get_rules_sorted<RuleClearanceCopperKeepout>();
         for (auto ru : rules) {
             if (ru->enabled) {
                 max_clearance = std::max(max_clearance, ru->get_max_clearance());
@@ -626,7 +664,7 @@ const RuleParameters *BoardRules::get_parameters() const
 
 UUID BoardRules::get_via_padstack_uuid(const Net *net) const
 {
-    auto rules = get_rules_sorted<RuleVia>(RuleID::VIA);
+    auto rules = get_rules_sorted<RuleVia>();
     for (auto rule : rules) {
         if (rule->enabled && rule->match.match(net)) {
             return rule->padstack;
@@ -639,7 +677,7 @@ static const ParameterSet ps_empty;
 
 const ParameterSet &BoardRules::get_via_parameter_set(const Net *net) const
 {
-    auto rules = get_rules_sorted<RuleVia>(RuleID::VIA);
+    auto rules = get_rules_sorted<RuleVia>();
     for (auto rule : rules) {
         if (rule->enabled && rule->match.match(net)) {
             return rule->parameter_set;
@@ -652,7 +690,7 @@ static const PlaneSettings plane_settings_default;
 
 const PlaneSettings &BoardRules::get_plane_settings(const Net *net, int layer) const
 {
-    auto rules = get_rules_sorted<RulePlane>(RuleID::PLANE);
+    auto rules = get_rules_sorted<RulePlane>();
     for (auto rule : rules) {
         if (rule->enabled && rule->match.match(net) && (rule->layer == layer || rule->layer == 10000)) {
             return rule->settings;
@@ -663,7 +701,7 @@ const PlaneSettings &BoardRules::get_plane_settings(const Net *net, int layer) c
 
 int BoardRules::get_layer_pair(const Net *net, int layer) const
 {
-    auto rules = get_rules_sorted<RuleLayerPair>(RuleID::LAYER_PAIR);
+    auto rules = get_rules_sorted<RuleLayerPair>();
     for (auto rule : rules) {
         if (rule->enabled && rule->match.match(net)) {
             if (rule->layers.first == layer)
