@@ -13,6 +13,7 @@
 #include "rule_clearance_copper_keepout.hpp"
 #include "rule_layer_pair.hpp"
 #include "rule_clearance_same_net.hpp"
+#include "rule_shorted_pads.hpp"
 #include "rules/rules.hpp"
 #include "util/uuid.hpp"
 
@@ -30,12 +31,12 @@ public:
                            check_status_cb_t status_cb) const;
     void apply(RuleID id, class Board &b, class IPool &pool) const;
     json serialize() const override;
-    std::set<RuleID> get_rule_ids() const override;
-    const Rule *get_rule(RuleID id) const override;
-    const Rule *get_rule(RuleID id, const UUID &uu) const override;
+    std::vector<RuleID> get_rule_ids() const override;
+    const Rule &get_rule(RuleID id) const override;
+    const Rule &get_rule(RuleID id, const UUID &uu) const override;
     std::map<UUID, const Rule *> get_rules(RuleID id) const override;
     void remove_rule(RuleID id, const UUID &uu) override;
-    Rule *add_rule(RuleID id) override;
+    Rule &add_rule(RuleID id) override;
     void cleanup(const class Block *block);
 
     uint64_t get_default_track_width(const class Net *net, int layer) const;
@@ -74,6 +75,7 @@ private:
     std::map<UUID, RuleClearanceCopperKeepout> rule_clearance_copper_keepout;
     std::map<UUID, RuleLayerPair> rule_layer_pair;
     std::map<UUID, RuleClearanceSameNet> rule_clearance_same_net;
+    std::map<UUID, RuleShortedPads> rule_shorted_pads;
 
     std::vector<const RuleClearanceCopper *> rule_sorted_clearance_copper;
     void update_sorted();
@@ -88,6 +90,8 @@ private:
                                             check_status_cb_t status_cb) const;
     RulesCheckResult check_clearance_copper_non_copper(const class Board &b, class RulesCheckCache &cache,
                                                        check_status_cb_t status_cb) const;
+    RulesCheckResult check_clearance_silkscreen_exposed_copper(const class Board &b, class RulesCheckCache &cache,
+                                                               check_status_cb_t status_cb) const;
     RulesCheckResult check_preflight(const class Board &b) const;
     RulesCheckResult check_clearance_copper_keepout(const class Board &b, class RulesCheckCache &cache,
                                                     check_status_cb_t status_cb) const;
