@@ -4,7 +4,6 @@
 #include "common/junction.hpp"
 #include "nlohmann/json_fwd.hpp"
 #include "util/uuid.hpp"
-#include "util/uuid_provider.hpp"
 #include "util/uuid_ptr.hpp"
 #include <fstream>
 #include <map>
@@ -13,13 +12,13 @@
 namespace horizon {
 using json = nlohmann::json;
 
-class ConnectionLine : public UUIDProvider {
+class ConnectionLine {
 public:
     ConnectionLine(const UUID &uu, const json &j, class Board *brd = nullptr);
     ConnectionLine(const UUID &uu);
 
     void update_refs(class Board &brd);
-    virtual UUID get_uuid() const;
+    UUID get_uuid() const;
 
     UUID uuid;
 
