@@ -8,22 +8,16 @@ namespace horizon {
 
 RuleClearanceCopperOther::RuleClearanceCopperOther(const UUID &uu) : Rule(uu)
 {
-    id = RuleID::CLEARANCE_COPPER_OTHER;
 }
 
 RuleClearanceCopperOther::RuleClearanceCopperOther(const UUID &uu, const json &j, const RuleImportMap &import_map)
     : Rule(uu, j, import_map), match(j.at("match"), import_map), layer(j.at("layer")),
       routing_offset(j.value("routing_offset", 0.05_mm))
 {
-    id = RuleID::CLEARANCE_COPPER_OTHER;
-    {
-        const json &o = j["clearances"];
-        for (auto it = o.cbegin(); it != o.cend(); ++it) {
-            const json &va = it.value();
-            PatchType a = patch_type_lut.lookup(va.at("types").at(0));
-            PatchType b = patch_type_lut.lookup(va.at("types").at(1));
-            set_clearance(a, b, va.at("clearance"));
-        }
+    for (const auto &va : j.at("clearances")) {
+        PatchType a = patch_type_lut.lookup(va.at("types").at(0));
+        PatchType b = patch_type_lut.lookup(va.at("types").at(1));
+        set_clearance(a, b, va.at("clearance").get<uint64_t>());
     }
 }
 
@@ -70,11 +64,11 @@ json RuleClearanceCopperOther::serialize() const
     return j;
 }
 
-std::string RuleClearanceCopperOther::get_brief(const class Block *block) const
+std::string RuleClearanceCopperOther::get_brief(const class Block *block, class IPool *pool) const
 {
     std::stringstream ss;
     ss << "Match " << match.get_brief(block) << "\n";
-    ss << "Layer " << layer;
+    ss << layer_to_string(layer);
     return ss.str();
 }
 
