@@ -6,12 +6,10 @@
 namespace horizon {
 RulePreflightChecks::RulePreflightChecks() : Rule()
 {
-    id = RuleID::PREFLIGHT_CHECKS;
 }
 
 RulePreflightChecks::RulePreflightChecks(const json &j) : Rule(j)
 {
-    id = RuleID::PREFLIGHT_CHECKS;
 }
 
 json RulePreflightChecks::serialize() const
@@ -20,7 +18,7 @@ json RulePreflightChecks::serialize() const
     return j;
 }
 
-std::string RulePreflightChecks::get_brief(const class Block *block) const
+std::string RulePreflightChecks::get_brief(const class Block *block, class IPool *pool) const
 {
     return "";
 }
