@@ -7,11 +7,17 @@
 namespace horizon {
 class RuleVia : public Rule {
 public:
+    static const auto id = RuleID::VIA;
+    RuleID get_id() const override
+    {
+        return id;
+    }
+
     RuleVia(const UUID &uu);
     RuleVia(const UUID &uu, const json &j, const RuleImportMap &import_map);
     json serialize() const override;
 
-    std::string get_brief(const class Block *block = nullptr) const override;
+    std::string get_brief(const class Block *block = nullptr, class IPool *pool = nullptr) const override;
     bool is_match_all() const override;
     bool can_export() const override;
 
