@@ -15,7 +15,7 @@ using json = nlohmann::json;
 
 class Via {
 public:
-    Via(const UUID &uu, const json &j, class Board *brd = nullptr, class ViaPadstackProvider *vpp = nullptr);
+    Via(const UUID &uu, const json &j, class IPool &pool, class Board *brd = nullptr);
     Via(const UUID &uu, const Padstack *ps);
     Via(shallow_copy_t sh, const Via &other);
 
@@ -23,7 +23,7 @@ public:
 
     uuid_ptr<Net> net_set = nullptr;
     uuid_ptr<BoardJunction> junction = nullptr;
-    uuid_ptr<const Padstack> vpp_padstack;
+    uuid_ptr<const Padstack> pool_padstack;
     Padstack padstack;
     void expand(const class Board &brd);
 
