@@ -7,6 +7,7 @@
 #include "util/util.hpp"
 #include <algorithm>
 #include "util/bbox_accumulator.hpp"
+#include "block_symbol/block_symbol.hpp"
 
 namespace horizon {
 
@@ -62,7 +63,6 @@ void Canvas::clear()
             it.second.clear();
     }
     targets.clear();
-    sheet_current_uuid = UUID();
     for (auto &it : object_refs) {
         it.second.clear();
     }
@@ -211,7 +211,6 @@ void Canvas::update(const Sheet &sheet)
 {
     clear();
     layer_provider = &sheet;
-    sheet_current_uuid = sheet.uuid;
     update_markers();
     render(sheet);
     request_push();
@@ -256,6 +255,14 @@ void Canvas::update(const class Decal &dec, bool edit)
     request_push();
 }
 
+void Canvas::update(const class BlockSymbol &sym, bool edit)
+{
+    clear();
+    layer_provider = &sym;
+    render(sym, !edit);
+    request_push();
+}
+
 void Canvas::transform_save()
 {
     transforms.push_back(transform);
