@@ -27,6 +27,7 @@
 #include "util/geom_util.hpp"
 #include "board/board_panel.hpp"
 #include "common/picture.hpp"
+#include "block_symbol/block_symbol.hpp"
 #include "util/polygon_arc_removal_proxy.hpp"
 #include <algorithm>
 #include <ctime>
@@ -466,44 +467,7 @@ void Canvas::render(const SymbolPin &pin, bool interactive, ColorP co)
         draw_line(Coordf(-(int64_t)pin.length, .375_mm), Coordf(-(int64_t)pin.length - .75_mm, 0), c_main, 0, true, 0);
         draw_line(Coordf(-(int64_t)pin.length, -.375_mm), Coordf(-(int64_t)pin.length - .75_mm, 0), c_main, 0, true, 0);
     }
-    {
-        auto dl = [this, c_pin](float ax, float ay, float bx, float by) {
-            draw_line(Coordf(ax * 1_mm, ay * 1_mm), Coordf(bx * 1_mm, by * 1_mm), c_pin, 0, true, 0);
-        };
-        switch (pin.direction) {
-        case Pin::Direction::OUTPUT:
-            dl(0, -.6, -1, -.2);
-            dl(0, -.6, -1, -1);
-            break;
-        case Pin::Direction::INPUT:
-            dl(-1, -.6, 0, -.2);
-            dl(-1, -.6, 0, -1);
-            break;
-        case Pin::Direction::POWER_INPUT:
-            dl(-1, -.6, 0, -.2);
-            dl(-1, -.6, 0, -1);
-            dl(-1.4, -.6, -.4, -.2);
-            dl(-1.4, -.6, -.4, -1);
-            break;
-        case Pin::Direction::POWER_OUTPUT:
-            dl(0, -.6, -1, -.2);
-            dl(0, -.6, -1, -1);
-            dl(-.4, -.6, -1.4, -.2);
-            dl(-.4, -.6, -1.4, -1);
-            break;
-        case Pin::Direction::BIDIRECTIONAL:
-            dl(0, -.6, -1, -.2);
-            dl(0, -.6, -1, -1);
-            dl(-2, -.6, -1, -.2);
-            dl(-2, -.6, -1, -1);
-            break;
-        case Pin::Direction::NOT_CONNECTED:
-            dl(-.4, -1, -1, -.2);
-            dl(-.4, -.2, -1, -1);
-            break;
-        default:;
-        }
-    }
+    draw_direction(pin.direction, c_pin);
     transform_restore();
 
     if (pin.decoration.schmitt) {
@@ -581,7 +545,7 @@ void Canvas::render(const SymbolPin &pin, bool interactive, ColorP co)
     case SymbolPin::ConnectorStyle::NONE:
         break;
     }
-    if (pin.connected_net_lines.size() > 1) {
+    if (pin.connection_count > 1) {
         draw_line(p0, p0 + Coordi(0, 10), c_main, 0, false, 0.75_mm);
     }
     draw_line(p0, p1, c_main, 0, false);
@@ -628,7 +592,8 @@ void Canvas::render(const SchematicSymbol &sym)
         render(*it, true, ColorP::FROM_LAYER);
     }
 
-    if (sym.component->nopopulate) {
+    if (sym.component->nopopulate
+        || sym.component->nopopulate_from_instance == Component::NopopulateFromInstance::SET) {
         transform = sym.placement;
         img_auto_line = img_mode;
         draw_line(bb.first - Coordi(0.2_mm, 0.2_mm), bb.second + Coordi(0.2_mm, 0.2_mm), ColorP::NOPOPULATE_X, 0, true,
@@ -644,11 +609,12 @@ void Canvas::render(const Text &text, bool interactive, ColorP co)
 {
     bool rev = layer_provider->get_layers().at(text.layer).reverse;
     transform_save();
+    const auto mirror_orig = transform.mirror;
+    const auto angle_orig = transform.get_angle();
     transform.accumulate(text.placement);
-    auto angle = transform.get_angle();
-    if (transform.mirror ^ rev) {
-        angle = 32768 - angle;
-    }
+    const auto text_angle = text.placement.get_angle();
+    const auto angle =
+            (transform.mirror ^ rev ? 32768 - text_angle : text_angle) + (mirror_orig ^ rev ? -1 : 1) * angle_orig;
 
     img_patch_type(PatchType::TEXT);
     triangle_type_current = TriangleInfo::Type::TEXT;
@@ -691,11 +657,42 @@ void Canvas::render(const NetLabel &label)
 {
     std::string txt = "<no net>";
     auto c = ColorP::NET;
+    const Net *net = label.junction->net;
     if (label.junction->net) {
         txt = label.junction->net->name;
         if (label.junction->net->diffpair)
             c = ColorP::DIFFPAIR;
     }
+    if (label.show_port && net && net->is_port) {
+        std::string port_txt;
+        switch (net->port_direction) {
+        case Pin::Direction::BIDIRECTIONAL:
+            port_txt = "BIDI";
+            break;
+        case Pin::Direction::INPUT:
+            port_txt = "IN";
+            break;
+        case Pin::Direction::OUTPUT:
+            port_txt = "OUT";
+            break;
+        case Pin::Direction::OPEN_COLLECTOR:
+            port_txt = "OC";
+            break;
+        case Pin::Direction::POWER_INPUT:
+            port_txt = "PIN";
+            break;
+        case Pin::Direction::POWER_OUTPUT:
+            port_txt = "POUT";
+            break;
+        case Pin::Direction::PASSIVE:
+            port_txt = "PASV";
+            break;
+        case Pin::Direction::NOT_CONNECTED:
+            port_txt = "NC";
+            break;
+        }
+        txt = port_txt + ": " + txt;
+    }
     if (txt == "") {
         txt = "? plz fix";
     }
@@ -814,7 +811,8 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
             add_triangle(poly.layer, transform.transform(tri[0]), transform.transform(tri[1]),
                          transform.transform(tri[2]), co);
         }
-        triangle_type_current = TriangleInfo::Type::PLANE_OUTLINE;
+        triangle_type_current = TriangleInfo::Type::NONE;
+
         for (const auto &frag : plane->fragments) {
             ColorP co_orphan = co;
             if (frag.orphan == true)
@@ -835,14 +833,12 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
             }
         }
         end_group();
-        triangle_type_current = TriangleInfo::Type::NONE;
         object_ref_pop();
     }
     else { // normal polygon
-        if (triangle_type_current == TriangleInfo::Type::NONE)
-            triangle_type_current = TriangleInfo::Type::POLYGON;
+        const bool is_keepout = dynamic_cast<Keepout *>(poly.usage.ptr);
         begin_group(poly.layer);
-        if (poly_is_rect(poly)) {
+        if (poly_is_rect(poly) && !is_keepout) {
             const Coordf p0 = (poly.get_vertex(0).position + poly.get_vertex(1).position) / 2;
             const Coordf p1 = (poly.get_vertex(2).position + poly.get_vertex(3).position) / 2;
             const float width = (poly.get_vertex(0).position - poly.get_vertex(1).position).magd();
@@ -865,6 +861,10 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
             TPPLPartition part;
             po.SetOrientation(TPPL_ORIENTATION_CCW);
             part.Triangulate_EC(&po, &outpolys);
+            if (is_keepout) {
+                assert(triangle_type_current == TriangleInfo::Type::NONE);
+                triangle_type_current = TriangleInfo::Type::KEEPOUT_FILL;
+            }
             for (auto &tri : outpolys) {
                 assert(tri.GetNumPoints() == 3);
                 Coordf p0 = transform.transform(coordf_from_pt(tri[0]));
@@ -872,14 +872,15 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
                 Coordf p2 = transform.transform(coordf_from_pt(tri[2]));
                 add_triangle(poly.layer, p0, p1, p2, co);
             }
+            if (is_keepout) {
+                triangle_type_current = TriangleInfo::Type::NONE;
+            }
             for (size_t i = 0; i < poly.vertices.size(); i++) {
                 draw_line(poly.vertices[i].position, poly.vertices[(i + 1) % poly.vertices.size()].position, co,
                           poly.layer);
             }
         }
         end_group();
-        if (triangle_type_current == TriangleInfo::Type::POLYGON)
-            triangle_type_current = TriangleInfo::Type::NONE;
     }
 
     if (interactive) {
@@ -1097,6 +1098,9 @@ void Canvas::render(const Sheet &sheet)
     for (const auto &it : sheet.pictures) {
         render(it.second);
     }
+    for (const auto &it : sheet.block_symbols) {
+        render(it.second);
+    }
 }
 
 void Canvas::render(const Padstack &padstack, bool interactive)
@@ -1609,7 +1613,7 @@ void Canvas::render(const Decal &dec, bool interactive)
     }
 }
 
-void Canvas::render(const Picture &pic)
+void Canvas::render(const Picture &pic, bool interactive)
 {
     if (!pic.data && !img_mode) {
         draw_error(pic.placement.shift, 2e5, "Image " + (std::string)pic.data_uuid + " not found");
@@ -1621,17 +1625,21 @@ void Canvas::render(const Picture &pic)
         return;
     pictures.emplace_back();
     auto &x = pictures.back();
-    x.angle = pic.placement.get_angle_rad();
-    x.x = pic.placement.shift.x;
-    x.y = pic.placement.shift.y;
+    auto tr = transform;
+    tr.accumulate(pic.placement);
+    x.angle = tr.get_angle_rad();
+    x.x = tr.shift.x;
+    x.y = tr.shift.y;
     x.px_size = pic.px_size;
     x.data = pic.data;
     x.on_top = pic.on_top;
     x.opacity = pic.opacity;
-    float w = pic.data->width * pic.px_size;
-    float h = pic.data->height * pic.px_size;
-    selectables.append_angled(pic.uuid, ObjectType::PICTURE, pic.placement.shift, pic.placement.shift, {w, h},
-                              pic.placement.get_angle_rad());
+    if (interactive) {
+        float w = pic.data->width * pic.px_size;
+        float h = pic.data->height * pic.px_size;
+        selectables.append_angled(pic.uuid, ObjectType::PICTURE, pic.placement.shift, pic.placement.shift, {w, h},
+                                  pic.placement.get_angle_rad());
+    }
 }
 
 void Canvas::render(const BoardDecal &decal)
@@ -1647,4 +1655,213 @@ void Canvas::render(const BoardDecal &decal)
     transform_restore();
 }
 
+void Canvas::render(const BlockSymbol &sym, bool on_sheet)
+{
+    if (!on_sheet) {
+        for (const auto &it : sym.junctions) {
+            auto &junc = it.second;
+            selectables.append(junc.uuid, ObjectType::JUNCTION, junc.position, 0, 10000, true);
+            targets.emplace_back(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position));
+        }
+    }
+    for (const auto &it : sym.lines) {
+        render(it.second, !on_sheet);
+    }
+
+    if (object_refs_current.size() && object_refs_current.back().type == ObjectType::SCHEMATIC_BLOCK_SYMBOL) {
+        auto sym_uuid = object_refs_current.back().uuid;
+        for (const auto &it : sym.ports) {
+            object_ref_push(ObjectType::BLOCK_SYMBOL_PORT, it.second.uuid, sym_uuid);
+            render(it.second, !on_sheet);
+            object_ref_pop();
+        }
+    }
+    else {
+        for (const auto &it : sym.ports) {
+            render(it.second, !on_sheet);
+        }
+    }
+
+
+    for (const auto &it : sym.arcs) {
+        render(it.second, !on_sheet);
+    }
+    for (const auto &it : sym.texts) {
+        render(it.second, !on_sheet);
+    }
+    for (const auto &it : sym.pictures) {
+        render(it.second, !on_sheet);
+    }
+}
+
+void Canvas::draw_direction(Pin::Direction dir, ColorP color)
+{
+    auto dl = [this, color](float ax, float ay, float bx, float by) {
+        draw_line(Coordf(ax * 1_mm, ay * 1_mm), Coordf(bx * 1_mm, by * 1_mm), color, 0, true, 0);
+    };
+    switch (dir) {
+    case Pin::Direction::OUTPUT:
+        dl(0, -.6, -1, -.2);
+        dl(0, -.6, -1, -1);
+        break;
+    case Pin::Direction::INPUT:
+        dl(-1, -.6, 0, -.2);
+        dl(-1, -.6, 0, -1);
+        break;
+    case Pin::Direction::POWER_INPUT:
+        dl(-1, -.6, 0, -.2);
+        dl(-1, -.6, 0, -1);
+        dl(-1.4, -.6, -.4, -.2);
+        dl(-1.4, -.6, -.4, -1);
+        break;
+    case Pin::Direction::POWER_OUTPUT:
+        dl(0, -.6, -1, -.2);
+        dl(0, -.6, -1, -1);
+        dl(-.4, -.6, -1.4, -.2);
+        dl(-.4, -.6, -1.4, -1);
+        break;
+    case Pin::Direction::BIDIRECTIONAL:
+        dl(0, -.6, -1, -.2);
+        dl(0, -.6, -1, -1);
+        dl(-2, -.6, -1, -.2);
+        dl(-2, -.6, -1, -1);
+        break;
+    case Pin::Direction::NOT_CONNECTED:
+        dl(-.4, -1, -1, -.2);
+        dl(-.4, -.2, -1, -1);
+        break;
+    default:;
+    }
+}
+
+void Canvas::render(const class BlockSymbolPort &port, bool interactive)
+{
+    Coordi p0 = transform.transform(port.position);
+    Coordi p1 = p0;
+
+    Coordi p_name = p0;
+    Coordi p_nc = p0;
+
+    Orientation port_orientation = port.get_orientation_for_placement(transform);
+
+    Orientation name_orientation = Orientation::LEFT;
+    int64_t text_shift = 0.5_mm;
+    int64_t text_shift_name = text_shift;
+    int64_t nc_shift = 0.25_mm;
+    int64_t length = port.length;
+    const auto nc_orientation = port_orientation;
+
+    switch (port_orientation) {
+    case Orientation::LEFT:
+        p1.x += length;
+        p_name.x += port.length + text_shift_name;
+        p_nc.x -= nc_shift;
+        name_orientation = Orientation::RIGHT;
+        break;
+
+    case Orientation::RIGHT:
+        p1.x -= length;
+        p_name.x -= port.length + text_shift_name;
+        p_nc.x += nc_shift;
+        name_orientation = Orientation::LEFT;
+        break;
+
+    case Orientation::UP:
+        p1.y -= length;
+        p_name.y -= port.length + text_shift_name;
+        p_nc.y += nc_shift;
+        name_orientation = Orientation::DOWN;
+        break;
+
+    case Orientation::DOWN:
+        p1.y += length;
+        p_name.y += port.length + text_shift_name;
+        p_nc.y -= nc_shift;
+        name_orientation = Orientation::UP;
+        break;
+    }
+    ColorP c_main = ColorP::FROM_LAYER;
+    ColorP c_name = ColorP::PIN;
+    ColorP c_port = ColorP::PIN;
+
+    img_auto_line = img_mode;
+
+    const bool draw_in_line = port.name_orientation == PinNameOrientation::IN_LINE
+                              || (port.name_orientation == PinNameOrientation::HORIZONTAL
+                                  && (port_orientation == Orientation::LEFT || port_orientation == Orientation::RIGHT));
+
+    if (draw_in_line) {
+        draw_text(p_name, 1.5_mm, port.name, orientation_to_angle(name_orientation), TextOrigin::CENTER, c_name, 0, {});
+    }
+    else { // draw perp
+        Placement tr;
+        tr.set_angle(orientation_to_angle(port_orientation));
+        auto shift = tr.transform(Coordi(-1_mm, 0));
+        TextOptions opts;
+        opts.center = true;
+        draw_text(p_name + shift, 1.5_mm, port.name, orientation_to_angle(name_orientation) + 16384, TextOrigin::CENTER,
+                  c_name, 0, opts);
+    }
+
+
+    transform_save();
+    transform.accumulate(port.position);
+    transform.set_angle(0);
+    transform.mirror = false;
+    switch (port_orientation) {
+    case Orientation::RIGHT:
+        break;
+    case Orientation::LEFT:
+        transform.mirror ^= true;
+        break;
+    case Orientation::UP:
+        transform.inc_angle_deg(90);
+        break;
+    case Orientation::DOWN:
+        transform.inc_angle_deg(-90);
+        transform.mirror = true;
+        break;
+    }
+
+    draw_direction(port.direction, c_port);
+    transform_restore();
+
+    switch (port.connector_style) {
+    case BlockSymbolPort::ConnectorStyle::BOX:
+        draw_box(p0, 0.25_mm, c_main, 0, false);
+        break;
+
+    case BlockSymbolPort::ConnectorStyle::NC:
+        draw_cross(p0, 0.25_mm, c_main, 0, false);
+        draw_text(p_nc, 1.5_mm, "NC", orientation_to_angle(nc_orientation), TextOrigin::CENTER, c_port, 0, {});
+        break;
+
+    case BlockSymbolPort::ConnectorStyle::NONE:
+        break;
+    }
+    /* if (port.connected_net_lines.size() > 1) {
+         draw_line(p0, p0 + Coordi(0, 10), c_main, 0, false, 0.75_mm);
+     }*/
+    draw_line(p0, p1, c_main, 0, false);
+    if (interactive)
+        selectables.append_line(port.uuid, ObjectType::BLOCK_SYMBOL_PORT, p0, p1, 0);
+    img_auto_line = false;
+}
+
+void Canvas::render(const SchematicBlockSymbol &sym)
+{
+    transform = sym.placement;
+    object_ref_push(ObjectType::SCHEMATIC_BLOCK_SYMBOL, sym.uuid);
+    render(sym.symbol, true);
+    object_ref_pop();
+    for (const auto &it : sym.symbol.ports) {
+        targets.emplace_back(UUIDPath<2>(sym.uuid, it.second.uuid), ObjectType::BLOCK_SYMBOL_PORT,
+                             transform.transform(it.second.position));
+    }
+    auto bb = sym.symbol.get_bbox();
+    selectables.append(sym.uuid, ObjectType::SCHEMATIC_BLOCK_SYMBOL, {0, 0}, bb.first, bb.second);
+    transform.reset();
+}
+
+
 } // namespace horizon
