@@ -5,7 +5,7 @@
 namespace horizon {
 bool SelectionFilter::can_select(const SelectableRef &sel) const
 {
-    if (!ca.layer_is_visible(sel.layer) && sel.layer != 10000 && sel.type != ObjectType::BOARD_PACKAGE)
+    if (!ca.layer_is_visible(sel.layer) && sel.layer.start() != 10000 && sel.type != ObjectType::BOARD_PACKAGE)
         return false;
     ObjectType type = sel.type;
     if (type == ObjectType::POLYGON_ARC_CENTER || sel.type == ObjectType::POLYGON_EDGE
@@ -14,12 +14,11 @@ bool SelectionFilter::can_select(const SelectableRef &sel) const
 
     if (object_filter.count(type)) {
         const auto &filter = object_filter.at(type);
-        if (filter.layers.count(sel.layer)) {
-            return filter.layers.at(sel.layer);
-        }
-        else {
-            return filter.other_layers;
+        for (const auto [layer, enabled] : filter.layers) {
+            if (sel.layer.overlaps(layer))
+                return enabled;
         }
+        return filter.other_layers;
     }
     else {
         return true;
