@@ -143,8 +143,9 @@ void TriangleRenderer::realize()
 
 class UBOBuffer {
 public:
-    static_assert(static_cast<int>(ColorP::N_COLORS) == 19, "ubo size mismatch");
-    std::array<std::array<float, 4>, 19> colors;   // 18==ColorP::N_COLORS, keep in sync with shaders
+    static constexpr size_t ubo_size = 20; //  keep in sync with ubo.glsl
+    static_assert(static_cast<int>(ColorP::N_COLORS) == ubo_size, "ubo size mismatch");
+    std::array<std::array<float, 4>, ubo_size> colors;
     std::array<std::array<float, 4>, 256> colors2; // keep in sync with shader
     std::array<float, 12> screenmat;
     std::array<float, 12> viewmat;
@@ -152,18 +153,10 @@ public:
     float scale;
     std::array<float, 2> offset;
     float min_line_width;
-    int layer_flags;
+    unsigned int layer_mode;
+    unsigned int stencil_mode;
 };
 
-static void mat3_to_array(std::array<float, 12> &dest, const glm::mat3 &src)
-{
-    for (int r = 0; r < 3; r++) {
-        for (int c = 0; c < 3; c++) {
-            dest[r * 4 + c] = src[r][c];
-        }
-    }
-}
-
 static std::array<float, 4> operator+(const std::array<float, 4> &a, float b)
 {
     return {a.at(0) + b, a.at(1) + b, a.at(2) + b, a.at(3)};
@@ -179,14 +172,9 @@ static std::array<float, 4> operator*(const std::array<float, 4> &a, float b)
     return {a.at(0) * b, a.at(1) * b, a.at(2) * b, a.at(3)};
 }
 
-static std::array<float, 4> array_from_color(const Color &c)
-{
-    return {c.r, c.g, c.b, 1};
-}
-
 std::array<float, 4> TriangleRenderer::apply_highlight(const Color &icolor, HighlightMode mode, int layer) const
 {
-    const std::array<float, 4> color = array_from_color(icolor);
+    const std::array<float, 4> color = gl_array_from_color(icolor);
     if (layer >= 20000 && layer < 30000) { // is annotation
         if (!ca.annotations.at(layer).use_highlight)
             return color;
@@ -199,7 +187,7 @@ std::array<float, 4> TriangleRenderer::apply_highlight(const Color &icolor, High
             if (mode == HighlightMode::ONLY)
                 return color;
             else
-                return array_from_color(ca.appearance.colors.at(ColorP::SHADOW));
+                return gl_array_from_color(ca.appearance.colors.at(ColorP::SHADOW));
         }
     }
     if (!ca.highlight_enabled)
@@ -216,117 +204,142 @@ std::array<float, 4> TriangleRenderer::apply_highlight(const Color &icolor, High
             return color;
         else
             return (color * ca.appearance.highlight_dim)
-                   + (array_from_color(ca.appearance.colors.at(ColorP::BACKGROUND))
+                   + (gl_array_from_color(ca.appearance.colors.at(ColorP::BACKGROUND))
                       * (1 - ca.appearance.highlight_dim));
 
     case CanvasGL::HighlightMode::SHADOW:
         if (mode == HighlightMode::ONLY)
             return color;
         else
-            return array_from_color(ca.appearance.colors.at(ColorP::SHADOW));
+            return gl_array_from_color(ca.appearance.colors.at(ColorP::SHADOW));
     }
     return color;
 }
 
-
-void TriangleRenderer::render_layer(int layer, HighlightMode highlight_mode, bool ignore_flip)
+void TriangleRenderer::render_layer_batch(int layer, HighlightMode highlight_mode, bool ignore_flip, const Batch &batch,
+                                          bool use_stencil, bool stencil_mode)
 {
     const auto &ld = ca.get_layer_display(layer);
-
-    GL_CHECK_ERROR
     UBOBuffer buf;
 
     buf.alpha = ca.property_layer_opacity() / 100;
-    mat3_to_array(buf.screenmat, ca.screenmat);
+    gl_mat3_to_array(buf.screenmat, ca.screenmat);
     if (ignore_flip)
-        mat3_to_array(buf.viewmat, ca.viewmat_noflip);
+        gl_mat3_to_array(buf.viewmat, ca.viewmat_noflip);
     else
-        mat3_to_array(buf.viewmat, ca.viewmat);
+        gl_mat3_to_array(buf.viewmat, ca.viewmat);
 
-    buf.layer_flags = static_cast<int>(ld.mode);
+    buf.layer_mode = static_cast<unsigned int>(ld.mode);
     buf.scale = ca.scale;
     buf.offset[0] = ca.offset.x;
     buf.offset[1] = ca.offset.y;
     buf.min_line_width = ca.appearance.min_line_width;
+    buf.stencil_mode = stencil_mode;
 
-    if (ld.mode == LayerDisplay::Mode::FILL_ONLY)
-        glStencilFunc(GL_GREATER, stencil, 0xff);
-    else
-        glStencilFunc(GL_ALWAYS, stencil, 0xff);
+    for (const auto &[key, span] : batch) {
+        bool skip = false;
+        switch (key.type) {
+        case Type::TRIANGLE:
+            glUseProgram(program_triangle);
+            if (ld.mode == LayerDisplay::Mode::OUTLINE)
+                skip = true;
+            break;
 
+        case Type::LINE0:
+            glUseProgram(program_line0);
+            break;
+
+        case Type::LINE_BUTT:
+            glUseProgram(program_line_butt);
+            break;
+
+        case Type::LINE:
+            glUseProgram(program_line);
+            break;
+
+        case Type::GLYPH:
+            glUseProgram(program_glyph);
+            break;
+
+        case Type::CIRCLE:
+            glUseProgram(program_circle);
+            break;
+
+        case Type::ARC0:
+            glUseProgram(program_arc0);
+            break;
+
+        case Type::ARC:
+            glUseProgram(program_arc);
+            break;
+        }
+        switch (highlight_mode) {
+        case HighlightMode::ONLY: // only highlighted, skip not highlighted
+            if (!key.highlight) {
+                skip = true;
+            }
+            break;
+        case HighlightMode::SKIP: // only not highlighted, skip highlighted
+            if (key.highlight) {
+                skip = true;
+            }
+            break;
+        }
+        if (!skip) {
+            for (size_t i = 0; i < buf.colors.size(); i++) {
+                auto k = static_cast<ColorP>(i);
+                if (ca.appearance.colors.count(k))
+                    buf.colors[i] = apply_highlight(ca.appearance.colors.at(k), highlight_mode, layer);
+            }
+            auto lc = ca.get_layer_color(layer);
+            buf.colors[static_cast<int>(ColorP::AIRWIRE_ROUTER)] =
+                    gl_array_from_color(ca.appearance.colors.at(ColorP::AIRWIRE_ROUTER));
+            buf.colors[static_cast<int>(ColorP::FROM_LAYER)] = apply_highlight(lc, highlight_mode, layer);
+            buf.colors[static_cast<int>(ColorP::LAYER_HIGHLIGHT)] =
+                    gl_array_from_color(lc) + ca.appearance.highlight_lighten;
+            for (size_t i = 0; i < std::min(buf.colors2.size(), ca.colors2.size()); i++) {
+                buf.colors2[i] = apply_highlight(ca.colors2[i].to_color(), highlight_mode, layer);
+            }
+
+            if (ld.mode == LayerDisplay::Mode::FILL_ONLY || (key.stencil && use_stencil))
+                glStencilFunc(GL_GREATER, stencil, 0xff);
+            else
+                glStencilFunc(GL_ALWAYS, stencil, 0xff);
+
+
+            glBindBuffer(GL_UNIFORM_BUFFER, ubo);
+            glBufferData(GL_UNIFORM_BUFFER, sizeof(buf), &buf, GL_DYNAMIC_DRAW);
+            glBindBuffer(GL_UNIFORM_BUFFER, 0);
+            GL_CHECK_ERROR
+            glDrawElements(GL_POINTS, span.count, GL_UNSIGNED_INT, (void *)(span.offset * sizeof(unsigned int)));
+        }
+    }
+}
+
+void TriangleRenderer::render_layer(int layer, HighlightMode highlight_mode, bool ignore_flip)
+{
+    GL_CHECK_ERROR
     if (layer_offsets.count(layer)) {
-        for (const auto &it : layer_offsets.at(layer)) {
-            bool skip = false;
-            switch (it.first.first) {
-            case Type::TRIANGLE:
-                glUseProgram(program_triangle);
-                if (ld.mode == LayerDisplay::Mode::OUTLINE)
-                    skip = true;
-                break;
+        const auto &ld = ca.get_layer_display(layer);
 
-            case Type::LINE0:
-                glUseProgram(program_line0);
-                break;
+        const auto &batches = layer_offsets.at(layer);
+        Batch batch_stencil, batch_no_stencil;
+        for (const auto &[key, span] : batches) {
+            if (key.stencil)
+                batch_stencil.emplace_back(key, span);
+            else
+                batch_no_stencil.emplace_back(key, span);
+        }
+        if (ld.mode == LayerDisplay::Mode::FILL_ONLY) {
+            render_layer_batch(layer, highlight_mode, ignore_flip, batch_stencil, false, false);
+            render_layer_batch(layer, highlight_mode, ignore_flip, batch_no_stencil, false, false);
+        }
+        else {
+            // draw stencil first
+            render_layer_batch(layer, highlight_mode, ignore_flip, batch_stencil, true, true);
+            render_layer_batch(layer, highlight_mode, ignore_flip, batch_stencil, true, false);
 
-            case Type::LINE_BUTT:
-                glUseProgram(program_line_butt);
-                break;
-
-            case Type::LINE:
-                glUseProgram(program_line);
-                break;
-
-            case Type::GLYPH:
-                glUseProgram(program_glyph);
-                break;
-
-            case Type::CIRCLE:
-                glUseProgram(program_circle);
-                break;
-
-            case Type::ARC0:
-                glUseProgram(program_arc0);
-                break;
-
-            case Type::ARC:
-                glUseProgram(program_arc);
-                break;
-            }
-            switch (highlight_mode) {
-            case HighlightMode::ONLY: // only highlighted, skip not highlighted
-                if (!it.first.second) {
-                    skip = true;
-                }
-                break;
-            case HighlightMode::SKIP: // only not highlighted, skip highlighted
-                if (it.first.second) {
-                    skip = true;
-                }
-                break;
-            }
-            if (!skip) {
-                for (size_t i = 0; i < buf.colors.size(); i++) {
-                    auto k = static_cast<ColorP>(i);
-                    if (ca.appearance.colors.count(k))
-                        buf.colors[i] = apply_highlight(ca.appearance.colors.at(k), highlight_mode, layer);
-                }
-                auto lc = ca.get_layer_color(layer);
-                buf.colors[static_cast<int>(ColorP::AIRWIRE_ROUTER)] =
-                        array_from_color(ca.appearance.colors.at(ColorP::AIRWIRE_ROUTER));
-                buf.colors[static_cast<int>(ColorP::FROM_LAYER)] = apply_highlight(lc, highlight_mode, layer);
-                buf.colors[static_cast<int>(ColorP::LAYER_HIGHLIGHT)] =
-                        array_from_color(lc) + ca.appearance.highlight_lighten;
-                for (size_t i = 0; i < std::min(buf.colors2.size(), ca.colors2.size()); i++) {
-                    buf.colors2[i] = apply_highlight(ca.colors2[i].to_color(), highlight_mode, layer);
-                }
-
-                glBindBuffer(GL_UNIFORM_BUFFER, ubo);
-                glBufferData(GL_UNIFORM_BUFFER, sizeof(buf), &buf, GL_DYNAMIC_DRAW);
-                glBindBuffer(GL_UNIFORM_BUFFER, 0);
-                GL_CHECK_ERROR
-                glDrawElements(GL_POINTS, it.second.second, GL_UNSIGNED_INT,
-                               (void *)(it.second.first * sizeof(unsigned int)));
-            }
+            render_layer_batch(layer, highlight_mode, ignore_flip, batch_no_stencil, false, false);
         }
     }
     // glDrawArrays(GL_POINTS, layer_offsets[layer], triangles[layer].size());
@@ -440,7 +453,7 @@ void TriangleRenderer::push()
     for (const auto &[layer, tris] : triangles) {
         const auto &ld = ca.get_layer_display(layer);
         glBufferSubData(GL_ARRAY_BUFFER, ofs * sizeof(Triangle), tris.size() * sizeof(Triangle), tris.first.data());
-        std::map<std::pair<Type, bool>, std::vector<unsigned int>> type_indices;
+        std::map<BatchKey, std::vector<unsigned int>> type_indices;
         unsigned int i = 0;
         for (const auto &[tri, tri_info] : tris) {
             const bool hidden = tri_info.flags & TriangleInfo::FLAG_HIDDEN;
@@ -475,16 +488,18 @@ void TriangleRenderer::push()
                 else {
                     throw std::runtime_error("unknown triangle type");
                 }
-                type_indices[std::make_pair(ty, (tri_info.flags & TriangleInfo::FLAG_HIGHLIGHT)
-                                                        || (tri.color == static_cast<int>(ColorP::LAYER_HIGHLIGHT)))]
-                        .push_back(i + ofs);
+                const bool highlight = (tri_info.flags & TriangleInfo::FLAG_HIGHLIGHT)
+                                       || (tri.color == static_cast<int>(ColorP::LAYER_HIGHLIGHT));
+                const bool do_stencil = tri_info.type == TriangleInfo::Type::PAD;
+                const BatchKey key{ty, highlight, do_stencil};
+                type_indices[key].push_back(i + ofs);
             }
             i++;
         }
-        for (const auto &it2 : type_indices) {
+        for (const auto &[key, elems] : type_indices) {
             auto el_offset = elements.size();
-            elements.insert(elements.end(), it2.second.begin(), it2.second.end());
-            layer_offsets[layer][it2.first] = std::make_pair(el_offset, it2.second.size());
+            elements.insert(elements.end(), elems.begin(), elems.end());
+            layer_offsets[layer][key] = {el_offset, elems.size()};
         }
         ofs += tris.size();
     }
