@@ -3,6 +3,7 @@
 #include "util/gl_inc.h"
 #include <map>
 #include <vector>
+#include <tuple>
 #include "util/vector_pair.hpp"
 
 namespace horizon {
@@ -19,7 +20,35 @@ private:
     const CanvasGL &ca;
     enum class Type { TRIANGLE, LINE, LINE0, LINE_BUTT, GLYPH, CIRCLE, ARC, ARC0 };
     const std::map<int, vector_pair<Triangle, TriangleInfo>> &triangles;
-    std::map<int, std::map<std::pair<Type, bool>, std::pair<size_t, size_t>>> layer_offsets;
+
+    struct BatchKey {
+        Type type;
+        bool highlight;
+        bool stencil;
+
+    private:
+        auto tie() const
+        {
+            return std::tie(type, highlight, stencil);
+        }
+
+    public:
+        bool operator<(const BatchKey &other) const
+        {
+            return tie() < other.tie();
+        }
+        bool operator==(const BatchKey &other) const
+        {
+            return tie() == other.tie();
+        }
+    };
+
+    struct Span {
+        size_t offset;
+        size_t count;
+    };
+
+    std::map<int, std::map<BatchKey, Span>> layer_offsets;
     size_t n_tris = 0;
 
     GLuint program_line0;
@@ -38,6 +67,9 @@ private:
 
     enum class HighlightMode { SKIP, ONLY };
     void render_layer(int layer, HighlightMode highlight_mode, bool ignore_flip = false);
+    using Batch = std::vector<decltype(layer_offsets)::mapped_type::value_type>;
+    void render_layer_batch(int layer, HighlightMode highlight_mode, bool ignore_flip, const Batch &batch,
+                            bool use_stencil, bool stencil_mode);
     void render_annotations(bool top);
     std::array<float, 4> apply_highlight(const class Color &color, HighlightMode mode, int layer) const;
     int stencil = 0;
