@@ -16,6 +16,7 @@
 #include <glm/gtx/rotate_vector.hpp>
 #include <iostream>
 #include <thread>
+#include "util/warp_cursor.hpp"
 
 namespace horizon {
 
@@ -27,6 +28,7 @@ Canvas3D::Canvas3D() : i_model_loading(0), stop_model_load_thread(false)
     models_loading_dispatcher.connect([this] {
         update_max_package_height();
         request_push();
+        invalidate_pick();
         s_signal_models_loading.emit(i_model_loading, n_models_loading);
         if ((i_model_loading >= n_models_loading) && model_load_thread.joinable()) {
             model_load_thread.join();
@@ -48,6 +50,18 @@ Canvas3D::Canvas3D() : i_model_loading(0), stop_model_load_thread(false)
     gesture_rotate->signal_begin().connect(sigc::mem_fun(*this, &Canvas3D::rotate_gesture_begin_cb));
     gesture_rotate->signal_update().connect(sigc::mem_fun(*this, &Canvas3D::rotate_gesture_update_cb));
     gesture_rotate->set_propagation_phase(Gtk::PHASE_CAPTURE);
+
+    signal_pick_ready().connect([this] {
+        const auto pick_result = pick_package_or_point(pick_x, pick_y);
+        if (auto uu = std::get_if<UUID>(&pick_result)) {
+            s_signal_package_select.emit(*uu);
+        }
+        else if (auto pt = std::get_if<glm::dvec3>(&pick_result)) {
+            s_signal_point_select.emit(*pt);
+        }
+    });
+
+    set_can_focus(true);
 }
 
 glm::vec2 Canvas3D::get_center_shift(const glm::vec2 &shift) const
@@ -79,6 +93,8 @@ void Canvas3D::on_size_allocate(Gtk::Allocation &alloc)
 
 bool Canvas3D::on_button_press_event(GdkEventButton *button_event)
 {
+    grab_focus();
+
     if (button_event->button == 2 || (button_event->button == 1 && (button_event->state & Gdk::SHIFT_MASK))) {
         pan_mode = PanMode::MOVE;
         pointer_pos_orig = {button_event->x, button_event->y};
@@ -93,34 +109,32 @@ bool Canvas3D::on_button_press_event(GdkEventButton *button_event)
     return Gtk::GLArea::on_button_press_event(button_event);
 }
 
-void Canvas3D::fix_cam_elevation()
-{
-    while (cam_elevation >= 360)
-        cam_elevation -= 360;
-    while (cam_elevation < 0)
-        cam_elevation += 360;
-    if (cam_elevation > 180)
-        cam_elevation -= 360;
-}
-
 bool Canvas3D::on_motion_notify_event(GdkEventMotion *motion_event)
 {
-    auto delta = glm::mat2(1, 0, 0, -1) * (glm::vec2(motion_event->x, motion_event->y) - pointer_pos_orig);
+    grab_focus();
+
+    const auto delta = glm::mat2(1, 0, 0, -1) * (glm::vec2(motion_event->x, motion_event->y) - pointer_pos_orig);
+    const auto warp_distance = warp_cursor((GdkEvent *)motion_event, *this);
     if (pan_mode == PanMode::ROTATE) {
-        cam_azimuth = cam_azimuth_orig - (delta.x / width) * 360;
-        cam_elevation = cam_elevation_orig - (delta.y / height) * 90;
-        fix_cam_elevation();
-        queue_draw();
+        set_cam_azimuth(cam_azimuth_orig - (delta.x / width) * 360);
+        set_cam_elevation(cam_elevation_orig - (delta.y / height) * 90);
     }
     else if (pan_mode == PanMode::MOVE) {
-        center = center_orig + get_center_shift(delta);
-        queue_draw();
+        set_center(center_orig + get_center_shift(delta));
     }
+    pointer_pos_orig += glm::vec2(warp_distance.x, warp_distance.y);
     return Gtk::GLArea::on_motion_notify_event(motion_event);
 }
 
 bool Canvas3D::on_button_release_event(GdkEventButton *button_event)
 {
+    if (pan_mode == PanMode::ROTATE) {
+        if (glm::length((glm::vec2(button_event->x, button_event->y) - pointer_pos_orig)) < 50) {
+            pick_x = button_event->x;
+            pick_y = button_event->y;
+            queue_pick();
+        }
+    }
     pan_mode = PanMode::NONE;
     return Gtk::GLArea::on_button_release_event(button_event);
 }
@@ -139,8 +153,7 @@ void Canvas3D::drag_gesture_update_cb(GdkEventSequence *seq)
 {
     double x, y;
     if (gesture_drag->get_offset(x, y)) {
-        center = gesture_drag_center_orig + get_center_shift({x, -y});
-        queue_draw();
+        set_center(gesture_drag_center_orig + get_center_shift({x, -y}));
     }
 }
 
@@ -157,8 +170,7 @@ void Canvas3D::zoom_gesture_begin_cb(GdkEventSequence *seq)
 void Canvas3D::zoom_gesture_update_cb(GdkEventSequence *seq)
 {
     auto delta = gesture_zoom->get_scale_delta();
-    cam_distance = gesture_zoom_cam_dist_orig / delta;
-    queue_draw();
+    set_cam_distance(gesture_zoom_cam_dist_orig / delta);
 }
 
 void Canvas3D::rotate_gesture_begin_cb(GdkEventSequence *seq)
@@ -185,12 +197,10 @@ void Canvas3D::rotate_gesture_update_cb(GdkEventSequence *seq)
     double cx, cy;
     gesture_rotate->get_bounding_box_center(cx, cy);
     auto dy = cy - gesture_rotate_pos_orig.y;
-    cam_elevation = gesture_rotate_cam_elevation_orig + (dy / height) * 180;
-    fix_cam_elevation();
-    queue_draw();
+    set_cam_elevation(gesture_rotate_cam_elevation_orig + (dy / height) * 180);
 }
 
-int Canvas3D::_animate_step(GdkFrameClock *frame_clock)
+int Canvas3D::zoom_animate_step(GdkFrameClock *frame_clock)
 {
     auto r = zoom_animator.step(gdk_frame_clock_get_frame_time(frame_clock) / 1e6);
     if (!r) { // should stop
@@ -198,12 +208,11 @@ int Canvas3D::_animate_step(GdkFrameClock *frame_clock)
     }
     auto s = zoom_animator.get_s();
 
-    cam_distance = zoom_animation_cam_dist_orig * pow(1.5, s);
-    queue_draw();
+    set_cam_distance(zoom_animation_cam_dist_orig * pow(1.5, s));
+
 
     if (std::abs((s - zoom_animator.target) / std::max(std::abs(zoom_animator.target), 1.f)) < .005) {
-        cam_distance = zoom_animation_cam_dist_orig * pow(1.5, zoom_animator.target);
-        queue_draw();
+        set_cam_distance(zoom_animation_cam_dist_orig * pow(1.5, zoom_animator.target));
         zoom_animator.stop();
         return G_SOURCE_REMOVE;
     }
@@ -211,11 +220,11 @@ int Canvas3D::_animate_step(GdkFrameClock *frame_clock)
     return G_SOURCE_CONTINUE;
 }
 
-static int tick_cb(GtkWidget *cwidget, GdkFrameClock *frame_clock, gpointer user_data)
+int Canvas3D::zoom_tick_cb(GtkWidget *cwidget, GdkFrameClock *frame_clock, gpointer user_data)
 {
     Gtk::Widget *widget = Glib::wrap(cwidget);
     auto canvas = dynamic_cast<Canvas3D *>(widget);
-    return canvas->_animate_step(frame_clock);
+    return canvas->zoom_animate_step(frame_clock);
 }
 
 
@@ -244,13 +253,12 @@ bool Canvas3D::on_scroll_event(GdkEventScroll *scroll_event)
         if (!zoom_animator.is_running()) {
             zoom_animator.start();
             zoom_animation_cam_dist_orig = cam_distance;
-            gtk_widget_add_tick_callback(GTK_WIDGET(gobj()), &tick_cb, nullptr, nullptr);
+            gtk_widget_add_tick_callback(GTK_WIDGET(gobj()), &zoom_tick_cb, nullptr, nullptr);
         }
         zoom_animator.target += inc;
     }
     else {
-        cam_distance *= pow(1.5, inc);
-        queue_draw();
+        set_cam_distance(cam_distance * pow(1.5, inc));
     }
 
 
@@ -259,13 +267,7 @@ bool Canvas3D::on_scroll_event(GdkEventScroll *scroll_event)
 
 void Canvas3D::inc_cam_azimuth(float v)
 {
-    cam_azimuth += v;
-    while (cam_azimuth < 0)
-        cam_azimuth += 360;
-
-    while (cam_azimuth > 360)
-        cam_azimuth -= 360;
-    queue_draw();
+    set_cam_azimuth(get_cam_azimuth() + v);
 }
 
 void Canvas3D::view_all()
@@ -385,6 +387,7 @@ bool Canvas3D::on_render(const Glib::RefPtr<Gdk::GLContext> &context)
     }
     if (needs_resize) {
         resize_buffers();
+        invalidate_pick();
         needs_resize = false;
     }
 
@@ -401,4 +404,9 @@ Canvas3D::~Canvas3D()
     }
 }
 
+void Canvas3D::redraw()
+{
+    queue_draw();
+}
+
 } // namespace horizon
