@@ -40,13 +40,25 @@ public:
 
     void view_all();
 
+    glm::vec2 get_center_shift(const glm::vec2 &shift) const;
+
     typedef sigc::signal<void, unsigned int, unsigned int> type_signal_models_loading;
     type_signal_models_loading signal_models_loading()
     {
         return s_signal_models_loading;
     }
 
-    int _animate_step(GdkFrameClock *frame_clock);
+    typedef sigc::signal<void, UUID> type_signal_package_select;
+    type_signal_package_select signal_package_select()
+    {
+        return s_signal_package_select;
+    }
+
+    typedef sigc::signal<void, glm::dvec3> type_signal_point_select;
+    type_signal_point_select signal_point_select()
+    {
+        return s_signal_point_select;
+    }
 
     ~Canvas3D();
 
@@ -54,6 +66,7 @@ private:
     bool needs_push = false;
     bool needs_view_all = false;
     int a_get_scale_factor() const override;
+    void redraw() override;
 
     void on_size_allocate(Gtk::Allocation &alloc) override;
     void on_realize() override;
@@ -65,7 +78,6 @@ private:
 
     Glib::RefPtr<Gtk::GestureDrag> gesture_drag;
     glm::vec2 gesture_drag_center_orig;
-    glm::vec2 get_center_shift(const glm::vec2 &shift) const;
     void drag_gesture_begin_cb(GdkEventSequence *seq);
     void drag_gesture_update_cb(GdkEventSequence *seq);
 
@@ -80,7 +92,6 @@ private:
     glm::vec2 gesture_rotate_pos_orig;
     void rotate_gesture_begin_cb(GdkEventSequence *seq);
     void rotate_gesture_update_cb(GdkEventSequence *seq);
-    void fix_cam_elevation();
 
     glm::vec2 pointer_pos_orig;
     float cam_azimuth_orig;
@@ -94,6 +105,9 @@ private:
     MSDAnimator zoom_animator;
     float zoom_animation_cam_dist_orig = 1;
 
+    static int zoom_tick_cb(GtkWidget *cwidget, GdkFrameClock *frame_clock, gpointer user_data);
+    int zoom_animate_step(GdkFrameClock *frame_clock);
+
     bool needs_resize = false;
 
 
@@ -110,5 +124,9 @@ private:
 
     std::thread model_load_thread;
     std::atomic<bool> stop_model_load_thread;
+
+    type_signal_package_select s_signal_package_select;
+    type_signal_point_select s_signal_point_select;
+    int pick_x, pick_y;
 };
 } // namespace horizon
