@@ -177,6 +177,9 @@ protected:
 
     virtual STEPImporter::Faces import_step(const std::string &filename_rel, const std::string &filename_abs);
 
+    virtual std::optional<std::pair<std::string, std::string>> get_model_filename(const class BoardPackage &pkg,
+                                                                                  IPool &pool);
+
 private:
     class FaceVertex {
     public:
