@@ -15,6 +15,12 @@ bool check_tag(const std::string &s)
     return re->match(us);
 }
 
+bool check_prefix(const std::string &s)
+{
+    const Glib::ustring up(s);
+    return Glib::Regex::match_simple("^[A-Z]+$", up);
+}
+
 void accumulate_level(RulesCheckErrorLevel &r, RulesCheckErrorLevel l)
 {
     r = static_cast<RulesCheckErrorLevel>(std::max(static_cast<int>(r), static_cast<int>(l)));
