@@ -60,6 +60,13 @@ enum class ObjectType {
     DECAL,
     BOARD_DECAL,
     PROJECT,
+    BLOCK,
+    BLOCKS,
+    BLOCK_INSTANCE,
+    BLOCK_SYMBOL,
+    BLOCK_SYMBOL_PORT,
+    SCHEMATIC_BLOCK_SYMBOL,
+    POOL,
 };
 enum class PatchType { OTHER, TRACK, PAD, PAD_TH, VIA, PLANE, HOLE_PTH, HOLE_NPTH, BOARD_EDGE, TEXT, N_TYPES };
 
