@@ -41,7 +41,7 @@ enum class ToolID;
  */
 class Core : public virtual Document {
 public:
-    class Block *get_block() override
+    class Block *get_top_block() override
     {
         return nullptr;
     }
@@ -70,7 +70,7 @@ public:
      * Expands the non-working document.
      * And copies the non-working document to the working document.
      */
-    virtual void rebuild(bool from_undo = false);
+    void rebuild(const std::string &comment);
     ToolResponse tool_begin(ToolID tool_id, const ToolArgs &args, class ImpInterface *imp, bool transient = false);
     ToolResponse tool_update(const ToolArgs &args);
     std::pair<bool, bool> tool_can_begin(ToolID tool_id, const std::set<SelectableRef> &selection);
@@ -84,6 +84,11 @@ public:
     bool can_undo() const;
     bool can_redo() const;
 
+    const std::string &get_undo_comment() const;
+    const std::string &get_redo_comment() const;
+
+    void set_history_max(unsigned int m);
+
     inline bool tool_is_active()
     {
         return tool != nullptr;
@@ -179,6 +184,7 @@ protected:
     class IPool &m_pool;
     class IPool &m_pool_caching;
 
+    ToolID tool_id_current;
     std::unique_ptr<ToolBase> tool = nullptr;
     type_signal_tool_changed s_signal_tool_changed;
     type_signal_rebuilt s_signal_rebuilt;
@@ -191,18 +197,22 @@ protected:
     bool needs_save = false;
     void set_needs_save(bool v);
 
+    void rebuild_finish(bool from_undo, const std::string &comment);
+
     class HistoryItem {
     public:
-        // Symbol sym;
-        // HistoryItem(const Symbol &s): sym(s) {}
-        std::string comment;
+        HistoryItem(const std::string &c) : comment(c)
+        {
+        }
+        const std::string comment;
         virtual ~HistoryItem()
         {
         }
     };
     std::deque<std::unique_ptr<HistoryItem>> history;
     int history_current = -1;
-    virtual void history_push() = 0;
+    size_t history_max = 50;
+    virtual void history_push(const std::string &comment) = 0;
     virtual void history_load(unsigned int i) = 0;
     void history_clear();
     void history_trim();
@@ -221,5 +231,6 @@ private:
     std::unique_ptr<ToolBase> create_tool(ToolID tool_id);
     std::set<SelectableRef> tool_selection;
     void maybe_end_tool(const ToolResponse &r);
+    virtual void rebuild_internal(bool from_undo, const std::string &comment) = 0;
 };
 } // namespace horizon
