@@ -8,24 +8,57 @@
 #include "pool/ipool.hpp"
 #include "pool/part.hpp"
 #include "util/picture_load.hpp"
+#include "blocks/iblock_provider.hpp"
+#include "blocks/blocks.hpp"
 
 namespace horizon {
-CoreBoard::CoreBoard(const std::string &board_filename, const std::string &block_filename,
+
+class NoneBlockProvider : public IBlockProvider {
+public:
+    Block &get_block(const UUID &uu) override
+    {
+        throw std::runtime_error("no blocks");
+    }
+
+    Block &get_top_block() override
+    {
+        throw std::runtime_error("no blocks");
+    }
+
+    std::map<UUID, Block *> get_blocks() override
+    {
+        throw std::runtime_error("no blocks");
+    }
+
+    static auto &get()
+    {
+        static NoneBlockProvider inst;
+        return inst;
+    }
+};
+
+static Block get_flattend_block(const std::string &blocks_filename, IPool &pool)
+{
+    auto blocks = Blocks::new_from_file(blocks_filename, pool);
+    return blocks.get_top_block_item().block.flatten();
+}
+
+CoreBoard::CoreBoard(const std::string &board_filename, const std::string &blocks_filename,
                      const std::string &pictures_dir, IPool &pool, IPool &pool_caching)
-    : Core(pool, &pool_caching), block(Block::new_from_file(block_filename, pool_caching)),
+    : Core(pool, &pool_caching), block(get_flattend_block(blocks_filename, pool_caching)),
       brd(Board::new_from_file(board_filename, *block, pool_caching)), rules(brd->rules),
       fab_output_settings(brd->fab_output_settings), pdf_export_settings(brd->pdf_export_settings),
       step_export_settings(brd->step_export_settings), pnp_export_settings(brd->pnp_export_settings),
       grid_settings(brd->grid_settings), colors(brd->colors), m_board_filename(board_filename),
-      m_block_filename(block_filename), m_pictures_dir(pictures_dir)
+      m_blocks_filename(blocks_filename), m_pictures_dir(pictures_dir)
 {
     brd->load_pictures(pictures_dir);
-    rebuild();
+    rebuild("init");
 }
 
 void CoreBoard::reload_netlist()
 {
-    block = Block::new_from_file(m_block_filename, m_pool_caching);
+    block = get_flattend_block(m_blocks_filename, m_pool_caching);
     brd->block = &*block;
     brd->update_refs();
     for (auto it = brd->packages.begin(); it != brd->packages.end();) {
@@ -39,7 +72,20 @@ void CoreBoard::reload_netlist()
             it++;
         }
     }
+
+    // delete planes with deleted nets
+    for (auto it = brd->planes.begin(); it != brd->planes.end();) {
+        const auto &plane = it->second;
+        if (!block->nets.count(plane.net.uuid)) { // net is gone
+            plane.polygon->usage = nullptr;
+            brd->planes.erase(it++);
+        }
+        else {
+            it++;
+        }
+    }
     brd->update_refs();
+
     for (auto it = brd->tracks.begin(); it != brd->tracks.end();) {
         bool del = false;
         for (auto &it_ft : {it->second.from, it->second.to}) {
@@ -65,7 +111,7 @@ void CoreBoard::reload_netlist()
     brd->update_refs();
     rules.cleanup(&*block);
     brd->expand_flags = Board::EXPAND_PROPAGATE_NETS | Board::EXPAND_ALL_AIRWIRES | Board::EXPAND_PACKAGES;
-    rebuild();
+    rebuild("reload netlist");
 }
 
 bool CoreBoard::get_property(ObjectType type, const UUID &uu, ObjectProperty::ID property, PropertyValue &value)
@@ -230,7 +276,8 @@ bool CoreBoard::get_property(ObjectType type, const UUID &uu, ObjectProperty::ID
             return true;
 
         case ObjectProperty::ID::NAME:
-            dynamic_cast<PropertyValueString &>(value).value = plane->net ? (plane->net->name) : "<no net>";
+            dynamic_cast<PropertyValueString &>(value).value =
+                    plane->net ? brd->block->get_net_name(plane->net->uuid) : "<no net>";
             return true;
 
         case ObjectProperty::ID::PRIORITY:
@@ -519,7 +566,7 @@ bool CoreBoard::set_property(ObjectType type, const UUID &uu, ObjectProperty::ID
         return false;
     }
     if (!property_transaction) {
-        rebuild(false);
+        rebuild_internal(false, "edit properties");
         set_needs_save(true);
     }
     return true;
@@ -588,11 +635,11 @@ bool CoreBoard::get_property_meta(ObjectType type, const UUID &uu, ObjectPropert
     }
 }
 
-void CoreBoard::rebuild(bool from_undo)
+void CoreBoard::rebuild_internal(bool from_undo, const std::string &comment)
 {
     clock_t begin = clock();
     brd->expand_some();
-    Core::rebuild(from_undo);
+    rebuild_finish(from_undo, comment);
     clock_t end = clock();
     double elapsed_secs = double(end - begin) / CLOCKS_PER_SEC;
     std::cout << "rebuild took " << elapsed_secs << std::endl;
@@ -618,7 +665,7 @@ const Board *CoreBoard::get_board() const
     return &*brd;
 }
 
-Block *CoreBoard::get_block()
+Block *CoreBoard::get_top_block()
 {
     return get_board()->block;
 }
@@ -633,13 +680,14 @@ void CoreBoard::update_rules()
     brd->rules = rules;
 }
 
-CoreBoard::HistoryItem::HistoryItem(const Block &b, const Board &r) : block(b), brd(shallow_copy, r)
+CoreBoard::HistoryItem::HistoryItem(const Block &b, const Board &r, const std::string &cm)
+    : Core::HistoryItem(cm), block(b), brd(shallow_copy, r)
 {
 }
 
-void CoreBoard::history_push()
+void CoreBoard::history_push(const std::string &comment)
 {
-    history.push_back(std::make_unique<CoreBoard::HistoryItem>(*block, *brd));
+    history.push_back(std::make_unique<CoreBoard::HistoryItem>(*block, *brd, comment));
 }
 
 void CoreBoard::history_load(unsigned int i)
@@ -661,11 +709,12 @@ void CoreBoard::reload_pool()
     const auto block_j = block->serialize();
     m_pool.clear();
     m_pool_caching.clear();
-    block.emplace(block->uuid, block_j, m_pool_caching);
+    block.emplace(block->uuid, block_j, m_pool_caching, NoneBlockProvider::get());
     brd.emplace(brd->uuid, brd_j, *block, m_pool_caching);
     keeper.restore(brd->pictures);
     history_clear();
-    rebuild();
+    rebuild("reload pool");
+    s_signal_can_undo_redo.emit();
 }
 
 json CoreBoard::get_meta()
