@@ -9,10 +9,10 @@
 namespace horizon {
 class CoreBoard : public Core, public DocumentBoard {
 public:
-    CoreBoard(const std::string &board_filename, const std::string &block_filename, const std::string &pictures_dir,
+    CoreBoard(const std::string &board_filename, const std::string &blocks_filename, const std::string &pictures_dir,
               IPool &pool, IPool &pool_caching);
 
-    class Block *get_block() override;
+    class Block *get_top_block() override;
     class LayerProvider &get_layer_provider() override;
 
     bool set_property(ObjectType type, const UUID &uu, ObjectProperty::ID property,
@@ -22,7 +22,6 @@ public:
     bool get_property_meta(ObjectType type, const UUID &uu, ObjectProperty::ID property,
                            class PropertyMeta &meta) override;
 
-    void rebuild(bool from_undo = false) override;
     void reload_netlist();
 
     const Board &get_canvas_data();
@@ -89,16 +88,17 @@ private:
     BoardColors colors;
 
     std::string m_board_filename;
-    std::string m_block_filename;
+    std::string m_blocks_filename;
     std::string m_pictures_dir;
 
     class HistoryItem : public Core::HistoryItem {
     public:
-        HistoryItem(const Block &b, const Board &r);
+        HistoryItem(const Block &b, const Board &r, const std::string &comment);
         Block block;
         Board brd;
     };
-    void history_push() override;
+    void rebuild_internal(bool from_undo, const std::string &comment) override;
+    void history_push(const std::string &comment) override;
     void history_load(unsigned int i) override;
     void save(const std::string &suffix) override;
     void delete_autosave() override;
