@@ -14,8 +14,6 @@ public:
     const Decal &get_canvas_data() const;
     class LayerProvider &get_layer_provider() override;
 
-    void rebuild(bool from_undo = false) override;
-
     std::pair<Coordi, Coordi> get_bbox() override;
 
     const std::string &get_filename() const override;
@@ -43,10 +41,11 @@ private:
 
     class HistoryItem : public Core::HistoryItem {
     public:
-        HistoryItem(const Decal &r);
+        HistoryItem(const Decal &r, const std::string &cm);
         Decal decal;
     };
-    void history_push() override;
+    void rebuild_internal(bool from_undo, const std::string &comment) override;
+    void history_push(const std::string &comment) override;
     void history_load(unsigned int i) override;
     void save(const std::string &suffix) override;
     void delete_autosave() override;
