@@ -33,8 +33,6 @@ public:
     std::vector<Line *> get_lines() override;
     std::vector<Arc *> get_arcs() override;
 
-    void rebuild(bool from_undo = false) override;
-
     bool set_property(ObjectType type, const UUID &uu, ObjectProperty::ID property,
                       const class PropertyValue &value) override;
     bool get_property(ObjectType type, const UUID &uu, ObjectProperty::ID property,
@@ -78,12 +76,13 @@ private:
 
     class HistoryItem : public Core::HistoryItem {
     public:
-        HistoryItem(const Symbol &s) : sym(s)
+        HistoryItem(const Symbol &s, const std::string &cm) : Core::HistoryItem(cm), sym(s)
         {
         }
         Symbol sym;
     };
-    void history_push() override;
+    void rebuild_internal(bool from_undo, const std::string &comment) override;
+    void history_push(const std::string &comment) override;
     void history_load(unsigned int i) override;
     void save(const std::string &suffix) override;
     void delete_autosave() override;
