@@ -1,5 +1,6 @@
 #include "tool.hpp"
 #include "tool_id.hpp"
+#include "imp/in_tool_action.hpp"
 
 namespace horizon {
 
@@ -19,11 +20,6 @@ void ToolBase::set_transient()
     is_transient = true;
 }
 
-ToolSettingsProxy::~ToolSettingsProxy()
-{
-    tool->apply_settings();
-}
-
 ToolResponse::ToolResponse() : next_tool(ToolID::NONE)
 {
 }
@@ -32,4 +28,8 @@ ToolResponse::ToolResponse(ToolResponse::Result r) : next_tool(ToolID::NONE), re
 {
 }
 
+ToolArgs::ToolArgs() : action(InToolActionID::NONE)
+{
+}
+
 } // namespace horizon
