@@ -1,12 +1,11 @@
 #pragma once
 #include "core/tool.hpp"
-#include <forward_list>
 
 namespace horizon {
 
 class ToolBendLineNet : public ToolBase {
 public:
-    ToolBendLineNet(IDocument *c, ToolID tid);
+    using ToolBase::ToolBase;
     ToolResponse begin(const ToolArgs &args) override;
     ToolResponse update(const ToolArgs &args) override;
     bool can_begin() override;
@@ -25,7 +24,6 @@ public:
     }
 
 private:
-    class Junction *temp = 0;
-    std::forward_list<Junction *> junctions_placed;
+    class SchematicJunction *temp = 0;
 };
 } // namespace horizon
