@@ -9,6 +9,8 @@
 #include "schematic/schematic.hpp"
 #include "document/idocument_symbol.hpp"
 #include "pool/symbol.hpp"
+#include "document/idocument_block_symbol.hpp"
+#include "block_symbol/block_symbol.hpp"
 #include "imp/imp_interface.hpp"
 #include "canvas/canvas_gl.hpp"
 #include "pool/entity.hpp"
@@ -35,14 +37,14 @@ ToolResponse ToolDelete::begin(const ToolArgs &args)
                 }
             }
             if (doc.c) {
-                for (const auto line : doc.c->get_net_lines()) {
-                    if (line->to.junc.uuid == it.uuid || line->from.junc.uuid == it.uuid) {
-                        delete_extra.emplace(line->uuid, ObjectType::LINE_NET);
+                for (const auto &[uu, line] : doc.c->get_sheet()->net_lines) {
+                    if (line.to.junc.uuid == it.uuid || line.from.junc.uuid == it.uuid) {
+                        delete_extra.emplace(line.uuid, ObjectType::LINE_NET);
                     }
                 }
-                for (const auto label : doc.c->get_net_labels()) {
-                    if (label->junction.uuid == it.uuid) {
-                        delete_extra.emplace(label->uuid, ObjectType::NET_LABEL);
+                for (const auto &[uu, label] : doc.c->get_sheet()->net_labels) {
+                    if (label.junction.uuid == it.uuid) {
+                        delete_extra.emplace(label.uuid, ObjectType::NET_LABEL);
                     }
                 }
                 for (const auto &it_rip : doc.c->get_sheet()->bus_rippers) {
@@ -65,12 +67,16 @@ ToolResponse ToolDelete::begin(const ToolArgs &args)
             }
         }
         else if (it.type == ObjectType::SCHEMATIC_SYMBOL) {
-            auto sym = doc.c->get_schematic_symbol(it.uuid);
-            doc.c->get_schematic()->disconnect_symbol(doc.c->get_sheet(), sym);
-            for (const auto &it_text : sym->texts) {
+            auto &sym = doc.c->get_sheet()->symbols.at(it.uuid);
+            doc.c->get_current_schematic()->disconnect_symbol(doc.c->get_sheet(), &sym);
+            for (const auto &it_text : sym.texts) {
                 delete_extra.emplace(it_text->uuid, ObjectType::TEXT);
             }
         }
+        else if (it.type == ObjectType::SCHEMATIC_BLOCK_SYMBOL) {
+            auto &sym = doc.c->get_sheet()->block_symbols.at(it.uuid);
+            doc.c->get_current_schematic()->disconnect_block_symbol(doc.c->get_sheet(), &sym);
+        }
         else if (it.type == ObjectType::BOARD_PACKAGE) {
             auto pkg = &doc.b->get_board()->packages.at(it.uuid);
             doc.b->get_board()->disconnect_package(&doc.b->get_board()->packages.at(it.uuid));
@@ -95,7 +101,7 @@ ToolResponse ToolDelete::begin(const ToolArgs &args)
     for (const auto &it : delete_extra) {
         if (it.type == ObjectType::LINE_NET) { // need to erase net lines before symbols
             LineNet *line = &doc.c->get_sheet()->net_lines.at(it.uuid);
-            doc.c->get_schematic()->delete_net_line(doc.c->get_sheet(), line);
+            doc.c->get_current_schematic()->delete_net_line(doc.c->get_sheet(), line);
         }
         else if (it.type == ObjectType::POWER_SYMBOL) { // need to erase power
                                                         // symbols before
@@ -113,7 +119,7 @@ ToolResponse ToolDelete::begin(const ToolArgs &args)
             auto &nsinfo = ns.at(j->net_segment);
             if (!nsinfo.has_power_sym) {
                 auto pins = sheet->get_pins_connected_to_net_segment(j->net_segment);
-                doc.c->get_schematic()->block->extract_pins(pins);
+                doc.c->get_current_schematic()->block->extract_pins(pins);
             }
         }
         else if (it.type == ObjectType::BUS_RIPPER) { // need to erase bus
@@ -133,7 +139,7 @@ ToolResponse ToolDelete::begin(const ToolArgs &args)
                 auto &nsinfo = ns.at(j.net_segment);
                 if (!nsinfo.has_label) {
                     auto pins = sheet.get_pins_connected_to_net_segment(j.net_segment);
-                    doc.c->get_schematic()->block->extract_pins(pins);
+                    doc.c->get_current_schematic()->block->extract_pins(pins);
                 }
             }
 
@@ -249,6 +255,9 @@ ToolResponse ToolDelete::begin(const ToolArgs &args)
         case ObjectType::SYMBOL_PIN:
             doc.y->delete_symbol_pin(it.uuid);
             break;
+        case ObjectType::BLOCK_SYMBOL_PORT:
+            doc.o->get_block_symbol().ports.erase(it.uuid);
+            break;
         case ObjectType::BOARD_PACKAGE:
             doc.b->get_board()->packages.erase(it.uuid);
             break;
@@ -262,10 +271,10 @@ ToolResponse ToolDelete::begin(const ToolArgs &args)
             doc.r->delete_picture(it.uuid);
             break;
         case ObjectType::SCHEMATIC_SYMBOL: {
-            SchematicSymbol *schsym = doc.c->get_schematic_symbol(it.uuid);
-            Component *comp = schsym->component.ptr;
-            doc.c->delete_schematic_symbol(it.uuid);
-            Schematic *sch = doc.c->get_schematic();
+            auto &schsym = doc.c->get_sheet()->symbols.at(it.uuid);
+            Component *comp = schsym.component.ptr;
+            doc.c->get_sheet()->symbols.erase(it.uuid);
+            Schematic *sch = doc.c->get_current_schematic();
             bool found = false;
             for (auto &it_sheet : sch->sheets) {
                 for (auto &it_sym : it_sheet.second.symbols) {
@@ -286,6 +295,11 @@ ToolResponse ToolDelete::begin(const ToolArgs &args)
                 comp = nullptr;
             }
         } break;
+        case ObjectType::SCHEMATIC_BLOCK_SYMBOL: {
+            auto &schsym = doc.c->get_sheet()->block_symbols.at(it.uuid);
+            doc.c->get_current_block()->block_instances.erase(schsym.block_instance->uuid);
+            doc.c->get_sheet()->block_symbols.erase(it.uuid);
+        } break;
         case ObjectType::NET_LABEL: {
             doc.c->get_sheet()->net_labels.erase(it.uuid);
         } break;
