@@ -1,11 +1,11 @@
 #pragma once
-#include "core/tool.hpp"
+#include "tool_helper_collect_nets.hpp"
 
 namespace horizon {
 
-class ToolDelete : public ToolBase {
+class ToolDelete : public virtual ToolBase, public ToolHelperCollectNets {
 public:
-    ToolDelete(IDocument *c, ToolID tid);
+    using ToolBase::ToolBase;
     ToolResponse begin(const ToolArgs &args) override;
     ToolResponse update(const ToolArgs &args) override;
     bool can_begin() override;
