@@ -43,7 +43,7 @@ void ToolDrawLineNet::set_snap_filter()
 
 void ToolDrawLineNet::restart(const Coordi &c)
 {
-    doc.c->get_schematic()->expand(); // gets rid of warning=s...
+    doc.c->get_current_schematic()->expand_connectivity(); // gets rid of warning=s...
     const auto uu = UUID::random();
     temp_junc_head = &doc.c->get_sheet()->junctions.emplace(uu, uu).first->second;
     temp_junc_head->position = c;
@@ -51,6 +51,7 @@ void ToolDrawLineNet::restart(const Coordi &c)
     temp_line_head = nullptr;
     temp_line_mid = nullptr;
     component_floating = nullptr;
+    block_instance_floating = nullptr;
     net_label = nullptr;
     set_snap_filter();
     update_tip();
@@ -131,6 +132,14 @@ void ToolDrawLineNet::apply_settings()
     }
 }
 
+LineNet *ToolDrawLineNet::insert_line_net()
+{
+    auto uu = UUID::random();
+    return &doc.c->get_sheet()
+                    ->net_lines.emplace(std::piecewise_construct, std::forward_as_tuple(uu), std::forward_as_tuple(uu))
+                    .first->second;
+}
+
 ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
 {
     if (args.type == ToolEventType::MOVE) {
@@ -142,6 +151,8 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
             SchematicJunction *ju = nullptr;
             SchematicSymbol *sym = nullptr;
             uuid_ptr<SymbolPin> pin = nullptr;
+            SchematicBlockSymbol *block_sym = nullptr;
+            uuid_ptr<BlockSymbolPort> port = nullptr;
             BusRipper *rip = nullptr;
             Net *net = nullptr;
             Bus *bus = nullptr;
@@ -152,7 +163,7 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                     bus = ju->bus;
                 }
                 else if (args.target.type == ObjectType::SYMBOL_PIN) {
-                    sym = doc.c->get_schematic_symbol(args.target.path.at(0));
+                    sym = &doc.c->get_sheet()->symbols.at(args.target.path.at(0));
                     pin = &sym->symbol.pins.at(args.target.path.at(1));
                     pin_start = pin;
                     UUIDPath<2> connpath(sym->gate->uuid, args.target.path.at(1));
@@ -179,23 +190,57 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                         net = conn.net;
                     }
                     else { // pin needs net
-                        net = doc.c->get_schematic()->block->insert_net();
+                        net = doc.c->get_current_schematic()->block->insert_net();
                         sym->component->connections.emplace(connpath, net);
                         component_floating = sym->component;
                         connpath_floating = connpath;
                     }
                 }
+                else if (args.target.type == ObjectType::BLOCK_SYMBOL_PORT) {
+                    block_sym = &doc.c->get_sheet()->block_symbols.at(args.target.path.at(0));
+                    port = &block_sym->symbol.ports.at(args.target.path.at(1));
+                    port_start = port;
+                    const UUID net_port = port->net;
+
+                    const auto orientation = port->get_orientation_for_placement(block_sym->placement);
+                    switch (orientation) {
+                    case Orientation::UP:
+                    case Orientation::DOWN:
+                        bend_mode = BendMode::YX;
+                        break;
+
+                    case Orientation::LEFT:
+                    case Orientation::RIGHT:
+                        bend_mode = BendMode::XY;
+                        break;
+                    }
+
+                    if (block_sym->block_instance->connections.count(net_port)) { // port is connected
+                        Connection &conn = block_sym->block_instance->connections.at(net_port);
+                        if (conn.net == nullptr) {
+                            imp->tool_bar_flash("can't connect to NC port");
+                            return ToolResponse();
+                        }
+                        net = conn.net;
+                    }
+                    else { // pin needs net
+                        net = doc.c->get_current_schematic()->block->insert_net();
+                        block_sym->block_instance->connections.emplace(net_port, net);
+                        block_instance_floating = block_sym->block_instance;
+                        net_port_floating = net_port;
+                    }
+                }
                 else if (args.target.type == ObjectType::BUS_RIPPER) {
                     rip = &doc.c->get_sheet()->bus_rippers.at(args.target.path.at(0));
                     net = rip->bus_member->net;
                 }
                 else { // unknown or no target
                     ju = make_temp_junc(args.coords);
-                    for (auto it : doc.c->get_net_lines()) {
-                        if (it->coord_on_line(temp_junc_head->position)) {
-                            if (it != temp_line_head && it != temp_line_mid) {
+                    for (auto &[uu, it] : doc.c->get_sheet()->net_lines) {
+                        if (it.coord_on_line(temp_junc_head->position)) {
+                            if (&it != temp_line_head && &it != temp_line_mid) {
                                 imp->tool_bar_flash("split net line");
-                                auto li = doc.c->get_sheet()->split_line_net(it, ju);
+                                auto li = doc.c->get_sheet()->split_line_net(&it, ju);
                                 net = li->net;
                                 bus = li->bus;
                                 ju->net = li->net;
@@ -215,6 +260,7 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
             }
             else { // already have net line
                 component_floating = nullptr;
+                block_instance_floating = nullptr;
                 if (args.target.type == ObjectType::JUNCTION) {
                     ju = &doc.c->get_sheet()->junctions.at(args.target.path.at(0));
                     auto do_merge = merge_bus_net(temp_line_head->net, temp_line_head->bus, ju->net, ju->bus);
@@ -229,7 +275,7 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                     }
                 }
                 else if (args.target.type == ObjectType::SYMBOL_PIN) {
-                    sym = doc.c->get_schematic_symbol(args.target.path.at(0));
+                    sym = &doc.c->get_sheet()->symbols.at(args.target.path.at(0));
                     pin = &sym->symbol.pins.at(args.target.path.at(1));
                     if (pin == pin_start) // do noting
                         return ToolResponse();
@@ -256,7 +302,7 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                             net = temp_junc_head->net;
                         }
                         else {
-                            net = doc.c->get_schematic()->block->insert_net();
+                            net = doc.c->get_current_schematic()->block->insert_net();
                         }
                         sym->component->connections.emplace(connpath, net);
                     }
@@ -265,6 +311,43 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                     restart(args.coords);
                     return ToolResponse();
                 }
+                else if (args.target.type == ObjectType::BLOCK_SYMBOL_PORT) {
+                    block_sym = &doc.c->get_sheet()->block_symbols.at(args.target.path.at(0));
+                    port = &block_sym->symbol.ports.at(args.target.path.at(1));
+                    if (port == port_start) // do noting
+                        return ToolResponse();
+                    const UUID net_port = port->net;
+                    if (temp_line_head->bus) { // can't connect bus to port
+                        return ToolResponse();
+                    }
+                    if (block_sym->block_instance->connections.count(net_port)) { // port is connected
+                        Connection &conn = block_sym->block_instance->connections.at(net_port);
+                        if (conn.net == nullptr) {
+                            imp->tool_bar_flash("can't connect to NC pin");
+                            return ToolResponse();
+                        }
+                        if (temp_line_head->net && (temp_line_head->net->uuid != conn.net->uuid)) { // has net
+                            if (merge_nets(conn.net, temp_line_head->net)) {
+                                return ToolResponse();
+                            }
+                            port = &block_sym->symbol.ports.at(args.target.path.at(1)); // update port after merge,
+                                                                                        // since merge replaces ports
+                        }
+                    }
+                    else { // port needs net
+                        if (temp_junc_head->net) {
+                            net = temp_junc_head->net;
+                        }
+                        else {
+                            net = doc.c->get_current_schematic()->block->insert_net();
+                        }
+                        block_sym->block_instance->connections.emplace(net_port, net);
+                    }
+                    temp_line_head->to.connect(block_sym, port);
+                    doc.r->delete_junction(temp_junc_head->uuid);
+                    restart(args.coords);
+                    return ToolResponse();
+                }
                 else if (args.target.type == ObjectType::BUS_RIPPER) {
                     rip = &doc.c->get_sheet()->bus_rippers.at(args.target.path.at(0));
                     net = rip->bus_member->net;
@@ -294,17 +377,16 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                     }
 
                     ju = temp_junc_head;
-                    for (auto it : doc.c->get_net_lines()) {
-                        if (it->coord_on_line(temp_junc_head->position)) {
-                            if (it != temp_line_head && it != temp_line_mid) {
+                    for (auto &[uu, it] : doc.c->get_sheet()->net_lines) {
+                        if (it.coord_on_line(temp_junc_head->position)) {
+                            if (&it != temp_line_head && &it != temp_line_mid) {
                                 imp->tool_bar_flash("split net line");
-                                net = it->net;
-                                bus = it->bus;
+                                net = it.net;
+                                bus = it.bus;
 
-                                auto do_merge =
-                                        merge_bus_net(temp_line_head->net, temp_line_head->bus, it->net, it->bus);
+                                auto do_merge = merge_bus_net(temp_line_head->net, temp_line_head->bus, it.net, it.bus);
                                 if (do_merge) {
-                                    doc.c->get_sheet()->split_line_net(it, ju);
+                                    doc.c->get_sheet()->split_line_net(&it, ju);
                                     restart(args.coords);
                                     return ToolResponse();
                                 }
@@ -328,7 +410,9 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
                     set_snap_filter();
                 }
             }
-            temp_line_mid = doc.c->insert_line_net(UUID::random());
+
+
+            temp_line_mid = insert_line_net();
             temp_line_mid->net = net;
             temp_line_mid->bus = bus;
             if (ju) {
@@ -337,6 +421,9 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
             else if (sym) {
                 temp_line_mid->from.connect(sym, pin);
             }
+            else if (block_sym) {
+                temp_line_mid->from.connect(block_sym, port);
+            }
             else if (rip) {
                 temp_line_mid->from.connect(rip);
             }
@@ -345,7 +432,7 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
             }
             temp_line_mid->to.connect(temp_junc_mid);
 
-            temp_line_head = doc.c->insert_line_net(UUID::random());
+            temp_line_head = insert_line_net();
             temp_line_head->net = net;
             temp_line_head->bus = bus;
             temp_line_head->from.connect(temp_junc_mid);
@@ -354,11 +441,10 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
 
         case InToolActionID::RMB:
             if (temp_line_head) {
-                doc.c->delete_line_net(temp_line_head->uuid);
-                doc.c->delete_line_net(temp_line_mid->uuid);
+                doc.c->get_sheet()->net_lines.erase(temp_line_head->uuid);
+                doc.c->get_sheet()->net_lines.erase(temp_line_mid->uuid);
                 doc.r->delete_junction(temp_junc_mid->uuid);
-                if (component_floating)
-                    component_floating->connections.erase(connpath_floating);
+                cleanup_floating();
                 restart(args.coords);
                 return ToolResponse();
             }
@@ -370,11 +456,10 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
 
         case InToolActionID::CANCEL:
             if (temp_line_head) {
-                doc.c->delete_line_net(temp_line_head->uuid);
-                doc.c->delete_line_net(temp_line_mid->uuid);
+                doc.c->get_sheet()->net_lines.erase(temp_line_head->uuid);
+                doc.c->get_sheet()->net_lines.erase(temp_line_mid->uuid);
                 doc.r->delete_junction(temp_junc_mid->uuid);
-                if (component_floating)
-                    component_floating->connections.erase(connpath_floating);
+                cleanup_floating();
             }
             doc.r->delete_junction(temp_junc_head->uuid);
             return ToolResponse::commit();
@@ -389,13 +474,13 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
             temp_junc_head->net = ju->net;
             temp_junc_head->bus = ju->bus;
 
-            temp_line_mid = doc.c->insert_line_net(UUID::random());
+            temp_line_mid = insert_line_net();
             temp_line_mid->net = ju->net;
             temp_line_mid->bus = ju->bus;
             temp_line_mid->from.connect(ju);
             temp_line_mid->to.connect(temp_junc_mid);
 
-            temp_line_head = doc.c->insert_line_net(UUID::random());
+            temp_line_head = insert_line_net();
             temp_line_head->net = ju->net;
             temp_line_head->bus = ju->bus;
             temp_line_head->from.connect(temp_junc_mid);
@@ -467,4 +552,13 @@ ToolResponse ToolDrawLineNet::update(const ToolArgs &args)
     update_tip();
     return ToolResponse();
 }
+
+void ToolDrawLineNet::cleanup_floating()
+{
+    if (component_floating)
+        component_floating->connections.erase(connpath_floating);
+    if (block_instance_floating)
+        block_instance_floating->connections.erase(net_port_floating);
+}
+
 } // namespace horizon
