@@ -14,9 +14,7 @@ public:
     std::set<InToolActionID> get_actions() const override
     {
         using I = InToolActionID;
-        return {
-                I::LMB, I::CANCEL, I::RMB, I::FLIP_ARC, I::TOGGLE_ARC, I::RESTRICT,
-        };
+        return {I::LMB, I::CANCEL, I::RMB, I::FLIP_ARC, I::TOGGLE_ARC, I::RESTRICT, I::TOGGLE_DEG45_RESTRICT};
     }
 
 protected:
