@@ -1,11 +1,10 @@
 #pragma once
 #include "core/tool.hpp"
-#include <forward_list>
 
 namespace horizon {
 class ToolEditPlane : public ToolBase {
 public:
-    ToolEditPlane(IDocument *c, ToolID tid);
+    using ToolBase::ToolBase;
     ToolResponse begin(const ToolArgs &args) override;
     ToolResponse update(const ToolArgs &args) override;
     bool can_begin() override;
