@@ -13,18 +13,9 @@ public:
         uint64_t net_label_size = 1.5_mm;
     };
 
-    const ToolSettings *get_settings_const() const override
-    {
-        return &settings;
-    }
-
-    ToolID get_tool_id_for_settings() const override;
+    std::map<ToolID, ToolSettings *> get_all_settings() override;
 
 protected:
-    ToolSettings *get_settings() override
-    {
-        return &settings;
-    }
     void step_net_label_size(bool up);
     void ask_net_label_size();
 
