@@ -9,7 +9,7 @@ public:
 protected:
     bool merge_bus_net(class Net *net, class Bus *bus, class Net *net_other, class Bus *bus_other);
     int merge_nets(Net *net, Net *into);
-    void merge_selected_junctions();
+    void merge_and_connect();
 
 private:
 };
