@@ -8,10 +8,15 @@ namespace horizon {
 
 bool ToolNoPopulate::can_begin()
 {
+    if (!doc.c)
+        return false;
+    if (!doc.c->in_hierarchy())
+        return false;
     for (const auto &it : selection) {
         if (it.type == ObjectType::SCHEMATIC_SYMBOL) {
-            auto sym = doc.c->get_schematic_symbol(it.uuid);
-            if (sym->component->nopopulate == (tool_id == ToolID::POPULATE))
+            const auto &comp = *doc.c->get_sheet()->symbols.at(it.uuid).component;
+            const auto comp_info = doc.c->get_top_block()->get_component_info(comp, doc.c->get_instance_path());
+            if (comp_info.nopopulate == (tool_id == ToolID::POPULATE))
                 return true;
         }
     }
@@ -22,8 +27,8 @@ ToolResponse ToolNoPopulate::begin(const ToolArgs &args)
 {
     for (const auto &it : args.selection) {
         if (it.type == ObjectType::SCHEMATIC_SYMBOL) {
-            auto sym = doc.c->get_schematic_symbol(it.uuid);
-            sym->component->nopopulate = (tool_id == ToolID::NOPOPULATE);
+            auto &comp = *doc.c->get_sheet()->symbols.at(it.uuid).component;
+            doc.c->get_top_block()->set_nopopulate(comp, doc.c->get_instance_path(), (tool_id == ToolID::NOPOPULATE));
         }
     }
     return ToolResponse::commit();
