@@ -1,5 +1,4 @@
 #pragma once
-#include "common/hole.hpp"
 #include "core/tool.hpp"
 #include <forward_list>
 
@@ -7,7 +6,7 @@ namespace horizon {
 
 class ToolPlaceHole : public ToolBase {
 public:
-    ToolPlaceHole(IDocument *c, ToolID tid);
+    using ToolBase::ToolBase;
     ToolResponse begin(const ToolArgs &args) override;
     ToolResponse update(const ToolArgs &args) override;
     bool can_begin() override;
@@ -22,7 +21,7 @@ public:
     }
 
 protected:
-    Hole *temp = 0;
+    class Hole *temp = 0;
     std::forward_list<Hole *> holes_placed;
 
     void create_hole(const Coordi &c);
