@@ -5,13 +5,13 @@
 namespace horizon {
 bool ToolPlaceJunctionSchematic::junction_placed()
 {
-    for (auto it : doc.c->get_net_lines()) {
-        if (it->coord_on_line(get_junction()->position)) {
+    for (auto &[uu, it] : doc.c->get_sheet()->net_lines) {
+        if (it.coord_on_line(get_junction()->position)) {
             std::cout << "on line" << std::endl;
-            if (!check_line(it))
+            if (!check_line(&it))
                 return true;
 
-            doc.c->get_sheet()->split_line_net(it, temp);
+            doc.c->get_sheet()->split_line_net(&it, temp);
             break;
         }
     }
