@@ -1,13 +1,11 @@
 #pragma once
 #include "core/tool.hpp"
-#include <forward_list>
-#include <map>
 
 namespace horizon {
 
 class ToolPlaceRefdesAndValue : public ToolBase {
 public:
-    ToolPlaceRefdesAndValue(IDocument *c, ToolID tid);
+    using ToolBase::ToolBase;
     ToolResponse begin(const ToolArgs &args) override;
     ToolResponse update(const ToolArgs &args) override;
     bool can_begin() override;
