@@ -32,7 +32,7 @@ public:
         std::map<int, LayerSettings> layers;
     };
 
-    const ToolSettings *get_settings_const() const override
+    ToolSettings *get_settings() override
     {
         return &settings;
     }
@@ -47,12 +47,6 @@ public:
         };
     }
 
-protected:
-    ToolSettings *get_settings() override
-    {
-        return &settings;
-    }
-
 private:
     class Text *temp = 0;
     class BoardPackage *pkg = nullptr;
