@@ -10,6 +10,7 @@
 #include "router/router/pns_router.h"
 #include "router/router/pns_meander_placer_base.h"
 #include "util/util.hpp"
+#include "util/geom_util.hpp"
 #include "core/tool_id.hpp"
 #include "util/selection_util.hpp"
 #include "dialogs/router_settings_window.hpp"
@@ -169,7 +170,7 @@ ToolResponse ToolRouteTrackInteractive::begin(const ToolArgs &args)
     iface->create_debug_decorator(imp->get_canvas());
     iface->SetCanvas(imp->get_canvas());
     iface->SetRules(rules);
-    iface->SetViaPadstackProvider(&doc.b->get_via_padstack_provider());
+    iface->SetPool(&doc.b->get_pool_caching());
     // m_iface->SetHostFrame( m_frame );
 
     router = new PNS::ROUTER;
@@ -514,7 +515,13 @@ bool ToolWrapper::prepareInteractive()
     sizes.SetTrackWidth(track_width);
     if (tool->tool_id == ToolID::ROUTE_DIFFPAIR_INTERACTIVE && m_startItem) {
         auto netcode = m_startItem->Net();
-        auto net = tool->iface->get_net_for_code(netcode);
+        auto *net = tool->iface->get_net_for_code(netcode);
+
+        if (!net) {
+            tool->imp->tool_bar_flash("No net");
+            return false;
+        }
+
         auto rules_dp =
                 tool->rules->get_diffpair(net->net_class, PNS::PNS_HORIZON_IFACE::layer_from_router(routingLayer));
         sizes.SetDiffPairGap(rules_dp->track_gap);
