@@ -38,7 +38,7 @@ public:
         bool drc = true;
     };
 
-    const ToolSettings *get_settings_const() const override
+    ToolSettings *get_settings() override
     {
         return &settings;
     }
@@ -68,12 +68,6 @@ public:
         };
     }
 
-protected:
-    ToolSettings *get_settings() override
-    {
-        return &settings;
-    }
-
 private:
     Settings settings;
 
