@@ -1,6 +1,5 @@
 #include "dialogs.hpp"
-#include "map_pin.hpp"
-#include "map_symbol.hpp"
+#include "map_uuid_path.hpp"
 #include "map_package.hpp"
 #include "pool_browser_dialog.hpp"
 #include "widgets/pool_browser_part.hpp"
@@ -41,6 +40,9 @@
 #include <glibmm.h>
 #include "pool/ipool.hpp"
 #include "widgets/net_selector.hpp"
+#include "manage_ports.hpp"
+#include "select_block.hpp"
+#include "align_and_distribute_window.hpp"
 
 namespace horizon {
 void Dialogs::set_parent(Gtk::Window *w)
@@ -53,20 +55,11 @@ void Dialogs::set_interface(ImpInterface *intf)
     interface = intf;
 }
 
-std::optional<UUID> Dialogs::map_pin(const std::vector<std::pair<const Pin *, bool>> &pins)
+std::optional<UUIDPath<2>> Dialogs::map_uuid_path(const std::string &title,
+                                                  const std::map<UUIDPath<2>, std::string> &items)
 {
-    MapPinDialog dia(parent, pins);
-    auto r = dia.run();
-    if (r == Gtk::RESPONSE_OK && dia.selection_valid) {
-        return dia.selected_uuid;
-    }
-    else {
-        return {};
-    }
-}
-std::optional<UUIDPath<2>> Dialogs::map_symbol(const std::map<UUIDPath<2>, std::string> &gates)
-{
-    MapSymbolDialog dia(parent, gates);
+    MapUUIDPathDialog dia(parent, items);
+    dia.set_title(title);
     auto r = dia.run();
     if (r == Gtk::RESPONSE_OK && dia.selection_valid) {
         return dia.selected_uuid_path;
@@ -205,6 +198,19 @@ std::optional<UUID> Dialogs::select_included_board(const Board &brd)
     }
 }
 
+std::optional<UUID> Dialogs::select_block(const BlocksSchematic &blocks)
+{
+    SelectBlockDialog dia(parent, blocks);
+    auto r = dia.run();
+    if (r == Gtk::RESPONSE_OK && dia.valid) {
+        return dia.selected_uuid;
+    }
+    else {
+        return {};
+    }
+}
+
+
 bool Dialogs::edit_shapes(std::set<Shape *> shapes)
 {
     ShapeDialog dia(parent, shapes);
@@ -235,13 +241,13 @@ bool Dialogs::manage_buses(Block &b)
     return dia.run() == Gtk::RESPONSE_OK;
 }
 
-bool Dialogs::manage_net_classes(Block &b)
+bool Dialogs::manage_net_classes(IBlockProvider &b)
 {
     ManageNetClassesDialog dia(parent, b);
     return dia.run() == Gtk::RESPONSE_OK;
 }
 
-bool Dialogs::manage_power_nets(Block &b)
+bool Dialogs::manage_power_nets(IBlockProvider &b)
 {
     ManagePowerNetsDialog dia(parent, b);
     return dia.run() == Gtk::RESPONSE_OK;
@@ -265,9 +271,9 @@ bool Dialogs::edit_board_hole(std::set<class BoardHole *> &holes, IPool &pool, I
     return dia.run() == Gtk::RESPONSE_OK;
 }
 
-bool Dialogs::edit_schematic_properties(class Schematic &sch, class IPool &pool, class IPool &pool_caching)
+bool Dialogs::edit_schematic_properties(class IDocumentSchematicBlockSymbol &doc)
 {
-    SchematicPropertiesDialog dia(parent, sch, pool, pool_caching);
+    SchematicPropertiesDialog dia(parent, doc);
     return dia.run() == Gtk::RESPONSE_OK;
 }
 
@@ -494,6 +500,12 @@ std::optional<std::string> Dialogs::ask_picture_filename()
     }
 }
 
+bool Dialogs::manage_ports(Block &b)
+{
+    ManagePortsDialog dia(parent, b);
+    return dia.run() == Gtk::RESPONSE_OK;
+}
+
 class SymbolPinNamesWindow *Dialogs::show_symbol_pin_names_window(class SchematicSymbol &symbol)
 {
     if (window_nonmodal)
@@ -590,6 +602,20 @@ PadParameterSetWindow *Dialogs::show_pad_parameter_set_window(std::set<class Pad
     return win;
 }
 
+AlignAndDistributeWindow *Dialogs::show_align_and_distribute_window()
+{
+    if (auto win = dynamic_cast<AlignAndDistributeWindow *>(window_nonmodal)) {
+        win->present();
+        return win;
+    }
+    auto win = new AlignAndDistributeWindow(parent, interface);
+    window_nonmodal = win;
+    win->signal_hide().connect([this] { close_nonmodal(); });
+    win->present();
+    return win;
+}
+
+
 void Dialogs::close_nonmodal()
 {
     delete window_nonmodal;
