@@ -19,8 +19,7 @@ public:
     void set_parent(Gtk::Window *w);
     void set_interface(class ImpInterface *intf);
 
-    std::optional<UUID> map_pin(const std::vector<std::pair<const class Pin *, bool>> &pins);
-    std::optional<UUIDPath<2>> map_symbol(const std::map<UUIDPath<2>, std::string> &gates);
+    std::optional<UUIDPath<2>> map_uuid_path(const std::string &title, const std::map<UUIDPath<2>, std::string> &gates);
     std::optional<UUID> map_package(const std::vector<std::pair<class Component *, bool>> &components);
     std::optional<UUID> select_symbol(class IPool &p, const UUID &unit_uuid, const UUID &sym_default = UUID());
     std::optional<UUID> select_part(class IPool &p, const UUID &entity_uuid, const UUID &part_uuid,
@@ -36,11 +35,12 @@ public:
     std::optional<UUID> select_group_tag(const class Block &block, bool tag_mode, const UUID &current);
     std::optional<UUID> select_included_board(const class Board &brd);
     std::optional<UUID> select_decal(class IPool &pool);
+    std::optional<UUID> select_block(const class BlocksSchematic &blocks);
 
     unsigned int ask_net_merge(class Net &net, class Net &into);
     bool manage_buses(class Block &b);
-    bool manage_net_classes(class Block &b);
-    bool manage_power_nets(class Block &b);
+    bool manage_net_classes(class IBlockProvider &b);
+    bool manage_power_nets(class IBlockProvider &b);
     bool manage_included_boards(class Board &b);
     bool edit_board_hole(std::set<class BoardHole *> &holes, class IPool &pool, class IPool &pool_caching,
                          class Block &block);
@@ -48,7 +48,7 @@ public:
     bool edit_plane(class Plane &plane, class Board &brd);
     bool edit_keepout(class Keepout &keepout, class IDocument &c, bool add_mode);
     bool edit_stackup(class IDocumentBoard &brd);
-    bool edit_schematic_properties(class Schematic &s, class IPool &pool, class IPool &pool_caching);
+    bool edit_schematic_properties(class IDocumentSchematicBlockSymbol &s);
     bool edit_frame_properties(class Frame &fr);
     std::optional<int64_t> ask_datum(const std::string &label, int64_t def = 0);
     std::optional<Coordi> ask_datum_coord(const std::string &label, Coordi def = Coordi());
@@ -63,6 +63,7 @@ public:
     std::optional<std::string> ask_dxf_filename();
     std::optional<std::string> ask_kicad_package_filename();
     std::optional<std::string> ask_picture_filename();
+    bool manage_ports(class Block &block);
 
     class SymbolPinNamesWindow *show_symbol_pin_names_window(class SchematicSymbol &symbol);
     class RenumberPadsWindow *show_renumber_pads_window(class Package &pkg, const std::set<UUID> &pads);
@@ -73,6 +74,7 @@ public:
     class EnterDatumScaleWindow *show_enter_datum_scale_window(const std::string &label, double def = 1);
     class PadParameterSetWindow *show_pad_parameter_set_window(std::set<class Pad *> &pads, class IPool &pool,
                                                                class Package &pkg);
+    class AlignAndDistributeWindow *show_align_and_distribute_window();
 
     void close_nonmodal();
     class ToolWindow *get_nonmodal();
