@@ -690,6 +690,10 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
          {"Show in pool manager", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD,
           ActionCatalogItem::FLAGS_SPECIFIC}},
 
+        {{ActionID::SHOW_IN_PROJECT_POOL_MANAGER, ToolID::NONE},
+         {"Show in project pool manager", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD,
+          ActionCatalogItem::FLAGS_SPECIFIC}},
+
         {{ActionID::SELECT_ALL, ToolID::NONE},
          {"Select all", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_EVERYWHERE,
           ActionCatalogItem::FLAGS_DEFAULT}},
@@ -968,6 +972,47 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
         {{ActionID::TOOL, ToolID::SET_TRACK_WIDTH},
          {"Set track width", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::EXCHANGE_GATES},
+         {"Exchange gates", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::MANAGE_PORTS},
+         {"Manage port nets", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::MAP_PORT},
+         {"Place port", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::ADD_BLOCK_INSTANCE},
+         {"Place block", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::PUSH_INTO_BLOCK, ToolID::NONE},
+         {"Push", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_SPECIFIC}},
+
+        {{ActionID::POP_OUT_OF_BLOCK, ToolID::NONE},
+         {"Pop to parent block", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::EDIT_BLOCK_SYMBOL, ToolID::NONE},
+         {"Edit block symbol", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_SPECIFIC}},
+
+        {{ActionID::GO_TO_BLOCK_SYMBOL, ToolID::NONE},
+         {"Go to block symbol", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::ALIGN_AND_DISTRIBUTE},
+         {"Align and distribute", ActionGroup::MOVE, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::GO_TO_PROJECT_MANAGER, ToolID::NONE},
+         {"Go to project manager", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
 };
 
 const std::vector<std::pair<ActionGroup, std::string>> action_group_catalog = {
@@ -1062,6 +1107,7 @@ const LutEnumStr<ActionID> action_lut = {
         ACTION_LUT_ITEM(GO_TO_BOARD),
         ACTION_LUT_ITEM(GO_TO_SCHEMATIC),
         ACTION_LUT_ITEM(SHOW_IN_POOL_MANAGER),
+        ACTION_LUT_ITEM(SHOW_IN_PROJECT_POOL_MANAGER),
         ACTION_LUT_ITEM(SELECT_ALL),
         ACTION_LUT_ITEM(PDF_EXPORT_WINDOW),
         ACTION_LUT_ITEM(EXPORT_PDF),
@@ -1106,6 +1152,11 @@ const LutEnumStr<ActionID> action_lut = {
         ACTION_LUT_ITEM(VIEW_3D_RIGHT),
         ACTION_LUT_ITEM(GRIDS_WINDOW),
         ACTION_LUT_ITEM(SELECT_GRID),
+        ACTION_LUT_ITEM(PUSH_INTO_BLOCK),
+        ACTION_LUT_ITEM(POP_OUT_OF_BLOCK),
+        ACTION_LUT_ITEM(EDIT_BLOCK_SYMBOL),
+        ACTION_LUT_ITEM(GO_TO_BLOCK_SYMBOL),
+        ACTION_LUT_ITEM(GO_TO_PROJECT_MANAGER),
 };
 
 #define TOOL_LUT_ITEM(x)                                                                                               \
@@ -1262,5 +1313,10 @@ const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(EDIT_CUSTOM_VALUE),
         TOOL_LUT_ITEM(PLACE_DOT),
         TOOL_LUT_ITEM(SET_TRACK_WIDTH),
+        TOOL_LUT_ITEM(EXCHANGE_GATES),
+        TOOL_LUT_ITEM(MANAGE_PORTS),
+        TOOL_LUT_ITEM(MAP_PORT),
+        TOOL_LUT_ITEM(ADD_BLOCK_INSTANCE),
+        TOOL_LUT_ITEM(ALIGN_AND_DISTRIBUTE),
 };
 } // namespace horizon
