@@ -98,14 +98,27 @@ ImpBase::ImpBase(const PoolParams &params)
                 mgr_pid = std::stoi(pid_p);
         }
 
+        {
+            auto ipc_cookie_p = Glib::getenv("HORIZON_IPC_COOKIE");
+            if (ipc_cookie_p.size())
+                ipc_cookie = ipc_cookie_p;
+        }
+
         Glib::signal_io().connect(
                 [this](Glib::IOCondition cond) {
                     while (zmq_helper::can_recv(sock_broadcast_rx)) {
                         zmq::message_t msg;
                         if (zmq_helper::recv(sock_broadcast_rx, msg)) {
+                            if (msg.size() < (UUID::size + sizeof(int) + 1))
+                                throw std::runtime_error("received short message");
+
                             int prefix;
-                            memcpy(&prefix, msg.data(), 4);
-                            char *data = ((char *)msg.data()) + 4;
+                            auto m = reinterpret_cast<const char *>(msg.data());
+                            memcpy(&prefix, m, sizeof(int));
+                            if (memcmp(m + sizeof(int), ipc_cookie.get_bytes(), UUID::size) != 0)
+                                throw std::runtime_error("IPC cookie didn't match");
+
+                            auto data = m + sizeof(int) + UUID::size;
                             json j = json::parse(data);
                             if (prefix == 0 || prefix == getpid()) {
                                 handle_broadcast(j);
@@ -143,9 +156,10 @@ json ImpBase::send_json(const json &j)
         return nullptr;
 
     std::string s = j.dump();
-    zmq::message_t msg(s.size() + 1);
-    memcpy(((uint8_t *)msg.data()), s.c_str(), s.size());
+    zmq::message_t msg(UUID::size + s.size() + 1);
     auto m = (char *)msg.data();
+    memcpy(m, ipc_cookie.get_bytes(), UUID::size);
+    memcpy(m + UUID::size, s.c_str(), s.size());
     m[msg.size() - 1] = 0;
     try {
         if (zmq_helper::send(sock_project, msg) == false) {
@@ -304,6 +318,14 @@ void ImpBase::set_flip_view(bool flip)
     g_simple_action_set_state(bottom_view_action->gobj(), g_variant_new_boolean(canvas->get_flip_view()));
 }
 
+static std::string append_if_not_empty(const std::string &prefix, const std::string &suffix)
+{
+    if (suffix.size())
+        return prefix + " " + suffix;
+    else
+        return prefix;
+}
+
 void ImpBase::run(int argc, char *argv[])
 {
     auto app = Gtk::Application::create(argc, argv, "org.horizon_eda.HorizonEDA.imp", Gio::APPLICATION_NON_UNIQUE);
@@ -414,7 +436,7 @@ void ImpBase::run(int argc, char *argv[])
     }
 
     canvas->signal_selection_mode_changed().connect([this](auto mode) {
-        set_action_sensitive(make_action(ActionID::CLICK_SELECT), mode == CanvasGL::SelectionMode::HOVER);
+        set_action_sensitive(ActionID::CLICK_SELECT, mode == CanvasGL::SelectionMode::HOVER);
         if (mode == CanvasGL::SelectionMode::HOVER) {
             main_window->selection_mode_label->set_text("Hover select");
         }
@@ -487,15 +509,26 @@ void ImpBase::run(int argc, char *argv[])
             if (!j.is_null()) {
                 save_json_to_file(core->get_filename() + meta_suffix, j);
             }
-            main_window->set_version_info("");
+
+            if (uses_dynamic_version()) {
+                saved_version = get_required_version();
+                check_version();
+            }
+            else {
+                main_window->set_version_info("");
+            }
         }
     });
-    connect_action(ActionID::UNDO, [this](const auto &a) {
+    connect_action_with_source(ActionID::UNDO, [this](const auto &a, auto src) {
+        if (src == ActionSource::KEY && preferences.undo_redo.show_hints)
+            main_window->set_undo_redo_hint("Undid " + core->get_undo_comment());
         core->undo();
         this->canvas_update_from_pp();
         this->update_property_panels();
     });
-    connect_action(ActionID::REDO, [this](const auto &a) {
+    connect_action_with_source(ActionID::REDO, [this](const auto &a, auto src) {
+        if (src == ActionSource::KEY && preferences.undo_redo.show_hints)
+            main_window->set_undo_redo_hint("Redid " + core->get_redo_comment());
         core->redo();
         this->canvas_update_from_pp();
         this->update_property_panels();
@@ -512,7 +545,6 @@ void ImpBase::run(int argc, char *argv[])
 
     connect_action(ActionID::VIEW_ALL, [this](const auto &a) {
         auto bbox = canvas->get_bbox();
-        std::cout << coord_to_string(bbox.first) << " " << coord_to_string(bbox.second) << std::endl;
         canvas->zoom_to_bbox(bbox.first, bbox.second);
     });
 
@@ -527,12 +559,12 @@ void ImpBase::run(int argc, char *argv[])
         std::map<ActionToolID, bool> can_begin;
         auto sel = canvas->get_selection();
         for (const auto &it : action_catalog) {
-            if (it.first.first == ActionID::TOOL) {
-                bool r = core->tool_can_begin(it.first.second, sel).first;
+            if (it.first.is_tool()) {
+                bool r = core->tool_can_begin(it.first.tool, sel).first;
                 can_begin[it.first] = r;
             }
             else {
-                can_begin[it.first] = this->get_action_sensitive(it.first);
+                can_begin[it.first] = this->get_action_sensitive(it.first.action);
             }
         }
         tool_popover->set_can_begin(can_begin);
@@ -616,12 +648,12 @@ void ImpBase::run(int argc, char *argv[])
         main_window->grid_window_button->signal_clicked().connect([this] { trigger_action(ActionID::GRIDS_WINDOW); });
         grids_window->signal_changed().connect([this] {
             core->set_needs_save();
-            set_action_sensitive(make_action(ActionID::SELECT_GRID), grids_window->has_grids());
+            set_action_sensitive(ActionID::SELECT_GRID, grids_window->has_grids());
         });
-        set_action_sensitive(make_action(ActionID::SELECT_GRID), grids_window->has_grids());
+        set_action_sensitive(ActionID::SELECT_GRID, grids_window->has_grids());
     }
 
-    auto save_button = create_action_button(make_action(ActionID::SAVE));
+    auto save_button = create_action_button(ActionID::SAVE);
     save_button->show();
     main_window->header->pack_start(*save_button);
     core->signal_needs_save().connect([this](bool v) {
@@ -633,19 +665,19 @@ void ImpBase::run(int argc, char *argv[])
         j["needs_save"] = core->get_needs_save();
         send_json(j);
     });
-    set_action_sensitive(make_action(ActionID::SAVE), false);
+    set_action_sensitive(ActionID::SAVE, false);
 
     {
         auto undo_redo_box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 0));
         undo_redo_box->get_style_context()->add_class("linked");
 
-        auto undo_button = create_action_button(make_action(ActionID::UNDO));
+        undo_button = create_action_button(ActionID::UNDO);
         gtk_button_set_label(undo_button->gobj(), NULL);
         undo_button->set_tooltip_text("Undo");
         undo_button->set_image_from_icon_name("edit-undo-symbolic", Gtk::ICON_SIZE_BUTTON);
         undo_redo_box->pack_start(*undo_button, false, false, 0);
 
-        auto redo_button = create_action_button(make_action(ActionID::REDO));
+        redo_button = create_action_button(ActionID::REDO);
         gtk_button_set_label(redo_button->gobj(), NULL);
         redo_button->set_tooltip_text("Redo");
         redo_button->set_image_from_icon_name("edit-redo-symbolic", Gtk::ICON_SIZE_BUTTON);
@@ -655,7 +687,11 @@ void ImpBase::run(int argc, char *argv[])
         main_window->header->pack_start(*undo_redo_box);
     }
 
-    core->signal_can_undo_redo().connect([this] { update_action_sensitivity(); });
+    core->signal_can_undo_redo().connect([this] {
+        undo_button->set_tooltip_text(append_if_not_empty("Undo", core->get_undo_comment()));
+        redo_button->set_tooltip_text(append_if_not_empty("Redo", core->get_redo_comment()));
+        update_action_sensitivity();
+    });
     canvas->signal_selection_changed().connect([this] {
         if (!core->tool_is_active()) {
             update_action_sensitivity();
@@ -688,7 +724,7 @@ void ImpBase::run(int argc, char *argv[])
         connect_action(ActionID::RULES_APPLY, [this](const auto &conn) { rules_window->apply_rules(); });
 
         {
-            auto button = create_action_button(make_action(ActionID::RULES));
+            auto button = create_action_button(ActionID::RULES);
             button->set_label("Rules…");
             main_window->header->pack_start(*button);
             button->show();
@@ -697,8 +733,7 @@ void ImpBase::run(int argc, char *argv[])
 
     tool_popover = Gtk::manage(new ToolPopover(canvas, get_editor_type_for_action()));
     tool_popover->set_position(Gtk::POS_BOTTOM);
-    tool_popover->signal_action_activated().connect(
-            [this](ActionID action_id, ToolID tool_id) { trigger_action(std::make_pair(action_id, tool_id)); });
+    tool_popover->signal_action_activated().connect([this](ActionToolID action_id) { trigger_action(action_id); });
 
 
     log_window = new LogWindow();
@@ -869,7 +904,7 @@ void ImpBase::run(int argc, char *argv[])
     });
     core->signal_modified().connect([this] { needs_autosave = true; });
 
-    if (core->get_block()) {
+    if (core->get_top_block()) {
         core->signal_rebuilt().connect(sigc::mem_fun(*this, &ImpBase::set_window_title_from_block));
         set_window_title_from_block();
     }
@@ -877,7 +912,16 @@ void ImpBase::run(int argc, char *argv[])
 
     reset_tool_hint_label();
 
+    saved_version = core->get_version().get_file();
+    {
+        const auto &version = core->get_version();
+        if (version.get_app() < version.get_file()) {
+            set_read_only(true);
+        }
+    }
     check_version();
+    if (uses_dynamic_version())
+        core->signal_modified().connect(sigc::mem_fun(*this, &ImpBase::check_version));
 
     {
         json j;
@@ -958,7 +1002,7 @@ void ImpBase::handle_drag()
     auto pos = canvas->get_cursor_pos_win();
     auto delta = pos - cursor_pos_drag_begin;
     if (delta.mag_sq() > (50 * 50)) {
-        highlights.clear();
+        clear_highlights();
         update_highlights();
         ToolArgs args;
         args.coords = cursor_pos_grid_drag_begin;
@@ -1029,6 +1073,8 @@ void ImpBase::apply_preferences()
         it->set_keep_primary_action(!preferences.action_bar.remember);
     }
     main_window->set_use_action_bar(preferences.action_bar.enable);
+    core->set_history_max(preferences.undo_redo.max_depth);
+    preferences_apply_appearance(preferences);
 }
 
 void ImpBase::canvas_update_from_pp()
@@ -1052,7 +1098,7 @@ void ImpBase::tool_begin(ToolID id, bool override_selection, const std::set<Sele
         Logger::log_critical("can't begin tool while tool is active", Logger::Domain::IMP);
         return;
     }
-    highlights.clear();
+    clear_highlights();
     update_highlights();
     ToolArgs args;
     args.data = std::move(data);
@@ -1187,11 +1233,11 @@ void ImpBase::create_context_menu(Gtk::Menu *parent, const std::set<SelectableRe
         for (const auto &it : action_catalog) {
             if (it.second.group == it_gr.first && (it.second.availability & get_editor_type_for_action())
                 && !(it.second.flags & ActionCatalogItem::FLAGS_NO_MENU)) {
-                if (it.first.first == ActionID::TOOL) {
-                    auto r = core->tool_can_begin(it.first.second, {sel});
+                if (it.first.is_tool()) {
+                    auto r = core->tool_can_begin(it.first.tool, {sel});
                     if (r.first && r.second) {
                         auto la_sub = create_context_menu_item(it.first);
-                        ToolID tool_id = it.first.second;
+                        ToolID tool_id = it.first.tool;
                         std::set<SelectableRef> sr(sel);
                         la_sub->signal_activate().connect([this, tool_id, sr] {
                             canvas->set_selection(sr, false);
@@ -1203,14 +1249,15 @@ void ImpBase::create_context_menu(Gtk::Menu *parent, const std::set<SelectableRe
                     }
                 }
                 else {
-                    if (get_action_sensitive(it.first) && (it.second.flags & ActionCatalogItem::FLAGS_SPECIFIC)) {
+                    if (get_action_sensitive(it.first.action)
+                        && (it.second.flags & ActionCatalogItem::FLAGS_SPECIFIC)) {
                         auto la_sub = create_context_menu_item(it.first);
-                        ActionID action_id = it.first.first;
+                        ActionID action_id = it.first.action;
                         std::set<SelectableRef> sr(sel);
                         la_sub->signal_activate().connect([this, action_id, sr] {
                             canvas->set_selection(sr, false);
                             fix_cursor_pos();
-                            trigger_action(make_action(action_id));
+                            trigger_action(action_id);
                         });
                         la_sub->show();
                         menu_items.push_back(la_sub);
@@ -1246,7 +1293,7 @@ void ImpBase::create_context_menu(Gtk::Menu *parent, const std::set<SelectableRe
 
 void ImpBase::reset_tool_hint_label()
 {
-    const auto act = make_action(ActionID::POPOVER);
+    const auto act = ActionID::POPOVER;
     if (action_connections.count(act)) {
         if (action_connections.at(act).key_sequences.size()) {
             const auto keys = key_sequence_to_string(action_connections.at(act).key_sequences.front());
@@ -1288,7 +1335,7 @@ bool ImpBase::handle_click(const GdkEventButton *button_event)
         auto sel = canvas->get_selection();
         if (sel.size() == 1) {
             auto a = get_doubleclick_action(sel.begin()->type, sel.begin()->uuid);
-            if (a.first != ActionID::NONE) {
+            if (a.is_valid()) {
                 selection_for_drag_move.clear();
                 trigger_action(a);
             }
@@ -1402,7 +1449,7 @@ void ImpBase::tool_process(ToolResponse &resp)
         canvas->set_cursor_external(false);
         canvas->snap_filter.clear();
         no_update = false;
-        highlights.clear();
+        clear_highlights();
         update_highlights();
     }
     if (!no_update) {
@@ -1411,16 +1458,12 @@ void ImpBase::tool_process(ToolResponse &resp)
             canvas->set_selection(core->get_tool_selection());
         }
         else {
-            if (canvas->get_selection_mode() == CanvasGL::SelectionMode::NORMAL) {
-                canvas->set_selection(core->get_tool_selection());
-            }
-            else {
-                canvas->set_selection({});
-            }
+            canvas->set_selection(core->get_tool_selection(),
+                                  canvas->get_selection_mode() == CanvasGL::SelectionMode::NORMAL);
         }
     }
     if (resp.next_tool != ToolID::NONE) {
-        highlights.clear();
+        clear_highlights();
         update_highlights();
         ToolArgs args;
         args.coords = canvas->get_cursor_pos();
@@ -1431,6 +1474,11 @@ void ImpBase::tool_process(ToolResponse &resp)
     }
 }
 
+void ImpBase::clear_highlights()
+{
+    highlights.clear();
+}
+
 void ImpBase::expand_selection_for_property_panel(std::set<SelectableRef> &sel_extra,
                                                   const std::set<SelectableRef> &sel)
 {
@@ -1441,7 +1489,7 @@ void ImpBase::handle_selection_changed(void)
     // std::cout << "Selection changed\n";
     // std::cout << "---" << std::endl;
     if (!core->tool_is_active()) {
-        highlights.clear();
+        clear_highlights();
         update_highlights();
         update_property_panels();
     }
@@ -1509,7 +1557,7 @@ void ImpBase::handle_warning_selected(const Coordi &pos)
 
 bool ImpBase::handle_broadcast(const json &j)
 {
-    const std::string op = j.at("op");
+    const auto op = j.at("op").get<std::string>();
     guint32 timestamp = j.value("time", 0);
     if (op == "present") {
         main_window->present(timestamp);
@@ -1550,7 +1598,7 @@ void ImpBase::set_monitor_items(const ItemSet &items)
 {
     std::set<std::string> filenames;
     std::transform(items.begin(), items.end(), std::inserter(filenames, filenames.begin()),
-                   [this](const auto &it) { return pool->get_filename(it.first, it.second); });
+                   [this](const auto &it) { return pool_caching->get_filename(it.first, it.second); });
     file_monitor_delay_connection.disconnect();
     file_monitor_delay_connection = Glib::signal_timeout().connect_seconds(
             [this, filenames] {
@@ -1591,15 +1639,15 @@ ActionToolID ImpBase::get_doubleclick_action(ObjectType type, const UUID &uu)
 {
     switch (type) {
     case ObjectType::TEXT:
-        return make_action(ToolID::ENTER_DATUM);
-        break;
+        return ToolID::ENTER_DATUM;
+
     case ObjectType::POLYGON_ARC_CENTER:
     case ObjectType::POLYGON_VERTEX:
     case ObjectType::POLYGON_EDGE:
-        return make_action(ActionID::SELECT_POLYGON);
-        break;
+        return ActionID::SELECT_POLYGON;
+
     default:
-        return {ActionID::NONE, ToolID::NONE};
+        return {};
     }
 }
 
@@ -1645,8 +1693,8 @@ void ImpBase::set_window_title(const std::string &s)
 void ImpBase::set_window_title_from_block()
 {
     std::string title;
-    if (core->get_block()->project_meta.count("project_title"))
-        title = core->get_block()->project_meta.at("project_title");
+    if (core->get_top_block()->project_meta.count("project_title"))
+        title = core->get_top_block()->project_meta.at("project_title");
 
     set_window_title(title);
 }
@@ -1708,10 +1756,20 @@ ObjectType ImpBase::get_editor_type() const
 
 void ImpBase::check_version()
 {
-    const auto &version = core->get_version();
-    main_window->set_version_info(version.get_message(core->get_object_type()));
-    if (version.get_app() < version.get_file()) {
-        set_read_only(true);
+    const auto dyn = uses_dynamic_version();
+    if (dyn && !read_only) {
+        if (get_required_version() > saved_version) {
+            const auto &t = object_descriptions.at(core->get_object_type()).name;
+            main_window->set_version_info(
+                    "Saving this " + t + " will update it from version " + std::to_string(saved_version) + " to "
+                    + std::to_string(get_required_version()) + " . " + FileVersion::learn_more_markup);
+        }
+        else {
+            main_window->set_version_info("");
+        }
+    }
+    else {
+        main_window->set_version_info(core->get_version().get_message(core->get_object_type()));
     }
 }
 
@@ -1725,6 +1783,22 @@ void ImpBase::view_options_menu_append_action(const std::string &label, const st
     bu->show();
 }
 
+void ImpBase::show_in_pool_manager(ObjectType type, const UUID &uu, ShowInPoolManagerPool p)
+{
+    json j;
+    j["op"] = "show-in-pool-mgr";
+    j["type"] = object_type_lut.lookup_reverse(type);
+    const auto pool_uuids = pool->get_pool_uuids(type, uu);
+    UUID pool_uuid;
+    if ((p == ShowInPoolManagerPool::LAST) && pool_uuids.last)
+        pool_uuid = pool_uuids.last;
+    else
+        pool_uuid = pool_uuids.pool;
+    j["pool_uuid"] = (std::string)pool_uuid;
+    j["uuid"] = (std::string)uu;
+    send_json(j);
+}
+
 static const char *get_cursor_icon(ToolID id)
 {
     switch (id) {
@@ -1746,7 +1820,7 @@ static const char *get_cursor_icon(ToolID id)
         return nullptr;
 
     default:
-        return get_action_icon(make_action(id));
+        return get_action_icon(id);
     }
 }
 
