@@ -46,9 +46,6 @@ public:
     }
     void set_read_only(bool v);
 
-    std::set<ObjectRef> highlights;
-    virtual void update_highlights(){};
-
     class SelectionFilterInfo {
     public:
         enum class Flag {
@@ -84,7 +81,11 @@ protected:
     class GridsWindow *grids_window = nullptr;
 
     std::unique_ptr<Pool> pool;
+
+private:
     std::unique_ptr<Pool> real_pool_caching;
+
+protected:
     Pool *pool_caching;
     class Core *core = nullptr;
     std::unique_ptr<ClipboardBase> clipboard = nullptr;
@@ -94,9 +95,11 @@ protected:
     Glib::RefPtr<Glib::Binding> grid_spacing_binding;
 
     std::map<ActionToolID, ActionConnection> action_connections;
-    ActionConnection &connect_action(ToolID tool_id, std::function<void(const ActionConnection &)> cb);
+
     ActionConnection &connect_action(ToolID tool_id);
-    ActionConnection &connect_action(ActionID action_id, std::function<void(const ActionConnection &)> cb);
+    ActionConnection &connect_action(ActionToolID id, std::function<void(const ActionConnection &)> cb);
+    ActionConnection &connect_action_with_source(ActionToolID id,
+                                                 std::function<void(const ActionConnection &, ActionSource)> cb);
 
     class RulesWindow *rules_window = nullptr;
 
@@ -105,6 +108,7 @@ protected:
     zmq::socket_t sock_project;
     bool sockets_connected = false;
     int mgr_pid = -1;
+    UUID ipc_cookie;
     bool no_update = false;
     bool distraction_free = false;
 
@@ -132,12 +136,11 @@ protected:
     bool handle_close(const GdkEventAny *ev);
     json send_json(const json &j);
 
-    bool trigger_action(const ActionToolID &action);
-    bool trigger_action(ActionID aid);
-    bool trigger_action(ToolID tid);
+    bool trigger_action(ActionToolID action, ActionSource source = ActionSource::UNKNOWN);
 
-    void add_tool_action(ToolID tid, const std::string &action);
-    void add_tool_action(ActionID aid, const std::string &action);
+    void connect_go_to_project_manager_action();
+
+    void add_tool_action(ActionToolID id, const std::string &action);
     void add_hamburger_menu();
 
     Preferences preferences;
@@ -160,8 +163,8 @@ protected:
 
     Gtk::Button *create_action_button(ActionToolID action);
 
-    void set_action_sensitive(ActionToolID, bool v);
-    bool get_action_sensitive(ActionToolID) const;
+    void set_action_sensitive(ActionID, bool v);
+    bool get_action_sensitive(ActionID) const;
     virtual void update_action_sensitivity();
 
     typedef sigc::signal<void> type_signal_action_sensitive;
@@ -172,6 +175,7 @@ protected:
 
     virtual std::string get_hud_text(std::set<SelectableRef> &sel);
     std::string get_hud_text_for_component(const Component *comp);
+    virtual const Block &get_block_for_group_tag_names();
     std::string get_hud_text_for_net(const Net *net);
 
     void set_monitor_files(const std::set<std::string> &files);
@@ -236,6 +240,26 @@ protected:
     void force_end_tool();
     void reset_tool_hint_label();
 
+    std::set<ObjectRef> highlights;
+
+    virtual void update_highlights()
+    {
+    }
+    virtual void clear_highlights();
+
+    enum class ShowInPoolManagerPool { CURRENT, LAST };
+    void show_in_pool_manager(ObjectType type, const UUID &uu, ShowInPoolManagerPool p);
+
+    virtual bool uses_dynamic_version() const
+    {
+        return false;
+    }
+
+    virtual unsigned int get_required_version() const
+    {
+        throw std::runtime_error("not implemented");
+    }
+
 private:
     void fix_cursor_pos();
     Glib::RefPtr<Gio::FileMonitor> preferences_monitor;
@@ -251,9 +275,6 @@ private:
     void handle_file_changed(const Glib::RefPtr<Gio::File> &file1, const Glib::RefPtr<Gio::File> &file2,
                              Gio::FileMonitorEvent ev);
 
-    ActionConnection &connect_action(ActionID action_id, ToolID tool_id,
-                                     std::function<void(const ActionConnection &)> cb);
-
     void create_context_menu(Gtk::Menu *parent, const std::set<SelectableRef> &sel);
     Gtk::MenuItem *create_context_menu_item(ActionToolID act);
 
@@ -278,7 +299,7 @@ private:
     Coordf cursor_pos_drag_begin;
     Coordi cursor_pos_grid_drag_begin;
 
-    std::map<ActionToolID, bool> action_sensitivity;
+    std::map<ActionID, bool> action_sensitivity;
     type_signal_action_sensitive s_signal_action_sensitive;
 
     bool property_panel_has_focus();
@@ -323,5 +344,10 @@ private:
     void update_cursor(ToolID tool_id);
 
     std::set<SelectableRef> last_canvas_selection;
+
+    Gtk::Button *undo_button = nullptr;
+    Gtk::Button *redo_button = nullptr;
+
+    unsigned int saved_version = 0;
 };
 } // namespace horizon
