@@ -117,32 +117,24 @@ void ImpBoard::update_highlights()
 void ImpBoard::apply_net_colors()
 {
     canvas->reset_color2();
+    std::map<UUID, std::vector<ObjectRef>> net_to_object_ref;
+    for (const auto &[uu, track] : core_board.get_board()->tracks) {
+        net_to_object_ref[track.net.uuid].emplace_back(ObjectType::TRACK, uu);
+    }
+    for (const auto &[uu, via] : core_board.get_board()->vias) {
+        net_to_object_ref[via.junction->net.uuid].emplace_back(ObjectType::VIA, uu);
+    }
+    for (const auto &[uu_pkg, pkg] : core_board.get_board()->packages) {
+        for (const auto &[uu_pad, pad] : pkg.package.pads) {
+            net_to_object_ref[pad.net.uuid].emplace_back(ObjectType::PAD, uu_pad, uu_pkg);
+        }
+    }
+    for (const auto &[uu, plane] : core_board.get_board()->planes) {
+        net_to_object_ref[plane.net.uuid].emplace_back(ObjectType::PLANE, uu);
+    }
     for (const auto &[net, color] : net_color_map) {
-        for (const auto &it_track : core_board.get_board()->tracks) {
-            if (it_track.second.net.uuid == net) {
-                ObjectRef ref(ObjectType::TRACK, it_track.first);
-                canvas->set_color2(ref, color);
-            }
-        }
-        for (const auto &it_via : core_board.get_board()->vias) {
-            if (it_via.second.junction->net.uuid == net) {
-                ObjectRef ref(ObjectType::VIA, it_via.first);
-                canvas->set_color2(ref, color);
-            }
-        }
-        for (const auto &it_pkg : core_board.get_board()->packages) {
-            for (const auto &it_pad : it_pkg.second.package.pads) {
-                if (it_pad.second.net.uuid == net) {
-                    ObjectRef ref(ObjectType::PAD, it_pad.first, it_pkg.first);
-                    canvas->set_color2(ref, color);
-                }
-            }
-        }
-        for (const auto &it_plane : core_board.get_board()->planes) {
-            if (it_plane.second.net.uuid == net) {
-                ObjectRef ref(ObjectType::PLANE, it_plane.first);
-                canvas->set_color2(ref, color);
-            }
+        for (const auto &ref : net_to_object_ref[net]) {
+            canvas->set_color2(ref, color);
         }
     }
 }
@@ -150,15 +142,27 @@ void ImpBoard::apply_net_colors()
 bool ImpBoard::handle_broadcast(const json &j)
 {
     if (!ImpBase::handle_broadcast(j)) {
-        std::string op = j.at("op");
+        const auto op = j.at("op").get<std::string>();
         guint32 timestamp = j.value("time", 0);
         if (op == "highlight" && cross_probing_enabled && !core->tool_is_active()) {
             highlights.clear();
             const json &o = j["objects"];
             for (auto it = o.cbegin(); it != o.cend(); ++it) {
                 auto type = static_cast<ObjectType>(it.value().at("type").get<int>());
-                UUID uu(it.value().at("uuid").get<std::string>());
-                highlights.emplace(type, uu);
+                if (type == ObjectType::NET) {
+                    const auto path = uuid_vec_from_string(it.value().at("uuid").get<std::string>());
+                    for (const auto &[uu, net] : core_board.get_top_block()->nets) {
+                        if (std::any_of(net.hrefs.begin(), net.hrefs.end(),
+                                        [&path](const auto &x) { return x == path; })) {
+                            highlights.emplace(type, uu);
+                            break;
+                        }
+                    }
+                }
+                else {
+                    UUID uu(it.value().at("uuid").get<std::string>());
+                    highlights.emplace(type, uu);
+                }
             }
             update_highlights();
         }
@@ -241,7 +245,17 @@ void ImpBoard::handle_selection_cross_probe()
 
         if (type != ObjectType::INVALID) {
             k["type"] = static_cast<int>(type);
-            k["uuid"] = (std::string)uu;
+            if (type == ObjectType::COMPONENT) {
+                k["uuid"] = (std::string)uu;
+            }
+            else if (type == ObjectType::NET) {
+                const auto &net = board->block->nets.at(uu);
+                std::vector<std::string> hrefs;
+                for (const auto &path : net.hrefs) {
+                    hrefs.push_back(uuid_vec_to_string(path));
+                }
+                k["uuid"] = hrefs;
+            }
             j["selection"].push_back(k);
         }
     }
@@ -265,8 +279,8 @@ void ImpBoard::update_action_sensitivity()
     bool have_tracks = std::any_of(sel.begin(), sel.end(), [](const auto &x) { return x.type == ObjectType::TRACK; });
     int n_pkgs =
             std::count_if(sel.begin(), sel.end(), [](const auto &x) { return x.type == ObjectType::BOARD_PACKAGE; });
-    set_action_sensitive(make_action(ActionID::TUNING_ADD_TRACKS), have_tracks);
-    set_action_sensitive(make_action(ActionID::TUNING_ADD_TRACKS_ALL), have_tracks);
+    set_action_sensitive(ActionID::TUNING_ADD_TRACKS, have_tracks);
+    set_action_sensitive(ActionID::TUNING_ADD_TRACKS_ALL, have_tracks);
 
     bool can_select_more = std::any_of(sel.begin(), sel.end(), [](const auto &x) {
         switch (x.type) {
@@ -280,14 +294,16 @@ void ImpBoard::update_action_sensitivity()
         }
     });
 
-    set_action_sensitive(make_action(ActionID::HIGHLIGHT_NET), can_select_more);
-    set_action_sensitive(make_action(ActionID::HIGHLIGHT_NET_CLASS), can_select_more);
-    set_action_sensitive(make_action(ActionID::SELECT_MORE), can_select_more);
-    set_action_sensitive(make_action(ActionID::SELECT_MORE_NO_VIA), can_select_more);
-    set_action_sensitive(make_action(ActionID::FILTER_AIRWIRES), can_select_more || n_pkgs);
+    set_action_sensitive(ActionID::HIGHLIGHT_NET, can_select_more);
+    set_action_sensitive(ActionID::HIGHLIGHT_NET_CLASS, can_select_more);
+    set_action_sensitive(ActionID::SELECT_MORE, can_select_more);
+    set_action_sensitive(ActionID::SELECT_MORE_NO_VIA, can_select_more);
+    set_action_sensitive(ActionID::FILTER_AIRWIRES, can_select_more || n_pkgs);
 
-    set_action_sensitive(make_action(ActionID::GO_TO_SCHEMATIC), sockets_connected);
-    set_action_sensitive(make_action(ActionID::SHOW_IN_POOL_MANAGER), n_pkgs == 1 && sockets_connected);
+    set_action_sensitive(ActionID::GO_TO_SCHEMATIC, sockets_connected);
+    set_action_sensitive(ActionID::SHOW_IN_POOL_MANAGER, n_pkgs == 1 && sockets_connected);
+    set_action_sensitive(ActionID::SHOW_IN_PROJECT_POOL_MANAGER, n_pkgs == 1 && sockets_connected);
+    set_action_sensitive(ActionID::GO_TO_PROJECT_MANAGER, sockets_connected);
 
     ImpBase::update_action_sensitivity();
 }
@@ -320,7 +336,7 @@ int ImpBoard::get_schematic_pid()
 {
     json j;
     j["op"] = "get-schematic-pid";
-    return this->send_json(j);
+    return this->send_json(j).get<int>();
 }
 
 static json serialize_connector(const Track::Connection &conn)
@@ -351,7 +367,7 @@ static json serialize_connector(const Track::Connection &conn)
 
 void ImpBoard::handle_select_more(const ActionConnection &conn)
 {
-    const bool no_via = conn.action_id == ActionID::SELECT_MORE_NO_VIA;
+    const bool no_via = conn.id.action == ActionID::SELECT_MORE_NO_VIA;
     std::map<const Junction *, std::set<const Track *>> junction_connections;
     const auto brd = core_board.get_board();
     for (const auto &it : brd->tracks) {
@@ -411,6 +427,22 @@ void ImpBoard::handle_select_more(const ActionConnection &conn)
     canvas->set_selection_mode(CanvasGL::SelectionMode::NORMAL);
 }
 
+void ImpBoard::handle_show_in_pool_manager(const ActionConnection &conn)
+{
+    const auto &board = *core_board.get_board();
+    const auto pool_sel = conn.id.action == ActionID::SHOW_IN_PROJECT_POOL_MANAGER ? ShowInPoolManagerPool::CURRENT
+                                                                                   : ShowInPoolManagerPool::LAST;
+    for (const auto &it : canvas->get_selection()) {
+        if (it.type == ObjectType::BOARD_PACKAGE) {
+            const auto &pkg = board.packages.at(it.uuid);
+            if (pkg.component->part) {
+                show_in_pool_manager(ObjectType::PART, pkg.component->part->uuid, pool_sel);
+                break;
+            }
+        }
+    }
+}
+
 void ImpBoard::construct()
 {
     ImpLayer::construct_layer_box(false);
@@ -428,26 +460,26 @@ void ImpBoard::construct()
     hamburger_menu->append("Fabrication output", "win.fab_out");
     main_window->add_action("fab_out", [this] { trigger_action(ActionID::FAB_OUTPUT_WINDOW); });
 
-    hamburger_menu->append("PDF Export", "win.export_pdf");
+    hamburger_menu->append("Export Pick & place…", "win.export_pnp");
+    main_window->add_action("export_pnp", [this] { trigger_action(ActionID::PNP_EXPORT_WINDOW); });
+
+    hamburger_menu->append("Export STEP…", "win.export_step");
+    main_window->add_action("export_step", [this] { trigger_action(ActionID::STEP_EXPORT_WINDOW); });
+
+    hamburger_menu->append("Export PDF…", "win.export_pdf");
     main_window->add_action("export_pdf", [this] { trigger_action(ActionID::PDF_EXPORT_WINDOW); });
 
-    hamburger_menu->append("Stackup…", "win.edit_stackup");
+    hamburger_menu->append("Import DXF…", "win.import_dxf");
+    add_tool_action(ToolID::IMPORT_DXF, "import_dxf");
+
+    hamburger_menu->append("Stackup", "win.edit_stackup");
     add_tool_action(ToolID::EDIT_STACKUP, "edit_stackup");
 
     hamburger_menu->append("Update all planes", "win.update_all_planes");
     add_tool_action(ToolID::UPDATE_ALL_PLANES, "update_all_planes");
 
-    hamburger_menu->append("Clear all planes", "win.clear_all_planes");
-    add_tool_action(ToolID::CLEAR_ALL_PLANES, "clear_all_planes");
-
-    hamburger_menu->append("Import DXF", "win.import_dxf");
-    add_tool_action(ToolID::IMPORT_DXF, "import_dxf");
-
-    hamburger_menu->append("Export STEP", "win.export_step");
-    main_window->add_action("export_step", [this] { trigger_action(ActionID::STEP_EXPORT_WINDOW); });
-
-    hamburger_menu->append("Export Pick & place", "win.export_pnp");
-    main_window->add_action("export_pnp", [this] { trigger_action(ActionID::PNP_EXPORT_WINDOW); });
+    hamburger_menu->append("Part list", "win.part_list");
+    main_window->add_action("part_list", [this] { trigger_action(ActionID::PARTS_WINDOW); });
 
     hamburger_menu->append("Length tuning", "win.tuning");
     main_window->add_action("tuning", [this] { trigger_action(ActionID::TUNING); });
@@ -481,26 +513,11 @@ void ImpBoard::construct()
             this->send_json(j);
         });
 
-        connect_action(ActionID::SHOW_IN_POOL_MANAGER, [this](const auto &conn) {
-            for (const auto &it : canvas->get_selection()) {
-                auto board = core_board.get_board();
-                if (it.type == ObjectType::BOARD_PACKAGE) {
-                    const auto &pkg = board->packages.at(it.uuid);
-                    if (pkg.component->part) {
-                        json j;
-                        j["op"] = "show-in-pool-mgr";
-                        j["type"] = "part";
-                        UUID pool_uuid;
-                        pool->get_filename(ObjectType::PART, pkg.component->part->uuid, &pool_uuid);
-                        j["pool_uuid"] = (std::string)pool_uuid;
-                        j["uuid"] = (std::string)pkg.component->part->uuid;
-                        this->send_json(j);
-                        break;
-                    }
-                }
-            }
-        });
-        set_action_sensitive(make_action(ActionID::SHOW_IN_POOL_MANAGER), false);
+        connect_action(ActionID::SHOW_IN_POOL_MANAGER, sigc::mem_fun(*this, &ImpBoard::handle_show_in_pool_manager));
+        connect_action(ActionID::SHOW_IN_PROJECT_POOL_MANAGER,
+                       sigc::mem_fun(*this, &ImpBoard::handle_show_in_pool_manager));
+        set_action_sensitive(ActionID::SHOW_IN_POOL_MANAGER, false);
+        set_action_sensitive(ActionID::SHOW_IN_PROJECT_POOL_MANAGER, false);
 
         connect_action(ActionID::BACKANNOTATE_CONNECTION_LINES, [this](const auto &conn) {
             json j;
@@ -519,6 +536,8 @@ void ImpBoard::construct()
             allow_set_foreground_window(this->get_schematic_pid());
             this->send_json(j);
         });
+
+        connect_go_to_project_manager_action();
     }
 
     connect_action(ActionID::RELOAD_NETLIST, [this](const ActionConnection &c) {
@@ -530,13 +549,15 @@ void ImpBoard::construct()
 #endif
         core_board.reload_netlist();
         core_board.set_needs_save();
+        const auto sel = canvas->get_selection();
         canvas_update();
         airwire_filter_window->update_nets();
         parts_window->update();
+        canvas->set_selection(sel);
     });
 
     {
-        auto button = create_action_button(make_action(ActionID::RELOAD_NETLIST));
+        auto button = create_action_button(ActionID::RELOAD_NETLIST);
         button->show();
         main_window->header->pack_end(*button);
 
@@ -605,7 +626,8 @@ void ImpBoard::construct()
     tuning_window->set_transient_for(*main_window);
     imp_interface->signal_request_length_tuning_ref().connect([this] { return tuning_window->get_ref_length(); });
 
-    rules_window->signal_goto().connect([this](Coordi location, UUID sheet) { canvas->center_and_zoom(location); });
+    rules_window->signal_goto().connect(
+            [this](Coordi location, UUID sheet, UUIDVec instance_path) { canvas->center_and_zoom(location); });
 
     connect_action(ActionID::VIEW_3D, [this](const auto &a) {
         view_3d_window->update();
@@ -649,8 +671,8 @@ void ImpBoard::construct()
         highlights.clear();
         for (const auto &it : canvas->get_selection()) {
             if (auto uu = net_from_selectable(it)) {
-                const auto &net_sel = core_board.get_block()->nets.at(uu);
-                for (const auto &[net_uu, net] : core_board.get_block()->nets) {
+                const auto &net_sel = core_board.get_top_block()->nets.at(uu);
+                for (const auto &[net_uu, net] : core_board.get_top_block()->nets) {
                     if (net.net_class == net_sel.net_class)
                         highlights.emplace(ObjectType::NET, net_uu);
                 }
@@ -813,18 +835,18 @@ void ImpBoard::construct()
         airwire_filter_window->load_from_json(m_meta.at("nets"));
 
     {
-        auto &b = add_action_button(make_action(ToolID::ROUTE_TRACK_INTERACTIVE));
-        b.add_action(make_action(ToolID::ROUTE_DIFFPAIR_INTERACTIVE));
+        auto &b = add_action_button(ToolID::ROUTE_TRACK_INTERACTIVE);
+        b.add_action(ToolID::ROUTE_DIFFPAIR_INTERACTIVE);
     }
     add_action_button_polygon();
     {
-        auto &b = add_action_button(make_action(ToolID::PLACE_BOARD_HOLE));
-        b.add_action(make_action(ToolID::PLACE_VIA));
+        auto &b = add_action_button(ToolID::PLACE_BOARD_HOLE);
+        b.add_action(ToolID::PLACE_VIA);
     }
     add_action_button_line();
 
-    add_action_button(make_action(ToolID::PLACE_TEXT));
-    add_action_button(make_action(ToolID::DRAW_DIMENSION));
+    add_action_button(ToolID::PLACE_TEXT);
+    add_action_button(ToolID::DRAW_DIMENSION);
 
     update_monitor();
 
@@ -1051,7 +1073,7 @@ void ImpBoard::handle_measure_tracks(const ActionConnection &a)
             tracks.insert(it.uuid);
         }
     }
-    tuning_window->add_tracks(tracks, a.action_id == ActionID::TUNING_ADD_TRACKS_ALL);
+    tuning_window->add_tracks(tracks, a.id.action == ActionID::TUNING_ADD_TRACKS_ALL);
     tuning_window->present();
 }
 
@@ -1158,17 +1180,17 @@ void ImpBoard::handle_drag()
 ActionToolID ImpBoard::get_doubleclick_action(ObjectType type, const UUID &uu)
 {
     auto a = ImpBase::get_doubleclick_action(type, uu);
-    if (a.first != ActionID::NONE)
+    if (a.action != ActionID::NONE)
         return a;
     switch (type) {
     case ObjectType::BOARD_HOLE:
-        return make_action(ToolID::EDIT_BOARD_HOLE);
+        return ToolID::EDIT_BOARD_HOLE;
         break;
     case ObjectType::VIA:
-        return make_action(ToolID::EDIT_VIA);
+        return ToolID::EDIT_VIA;
         break;
     case ObjectType::TRACK:
-        return make_action(ActionID::SELECT_MORE);
+        return ActionID::SELECT_MORE;
         break;
     case ObjectType::POLYGON:
     case ObjectType::POLYGON_EDGE:
@@ -1177,10 +1199,10 @@ ActionToolID ImpBoard::get_doubleclick_action(ObjectType type, const UUID &uu)
         if (poly->usage) {
             switch (poly->usage->get_type()) {
             case PolygonUsage::Type::PLANE:
-                return make_action(ToolID::EDIT_PLANE);
+                return ToolID::EDIT_PLANE;
 
             case PolygonUsage::Type::KEEPOUT:
-                return make_action(ToolID::EDIT_KEEPOUT);
+                return ToolID::EDIT_KEEPOUT;
 
             default:
                 return {ActionID::NONE, ToolID::NONE};
@@ -1313,7 +1335,7 @@ void ImpBoard::update_net_colors()
 
 void ImpBoard::update_monitor()
 {
-    ItemSet mon_items = core_board.get_block()->get_pool_items_used();
+    ItemSet mon_items = core_board.get_top_block()->get_pool_items_used();
     {
         ItemSet items = core_board.get_board()->get_pool_items_used();
         mon_items.insert(items.begin(), items.end());
