@@ -2,10 +2,13 @@
 #include "pool/part.hpp"
 #include "widgets/layer_box.hpp"
 #include "util/util.hpp"
+#include "util/geom_util.hpp"
 #include "nlohmann/json.hpp"
 #include "util/gtk_util.hpp"
 #include "view_angle_window.hpp"
 #include "widgets/spin_button_angle.hpp"
+#include "actions.hpp"
+#include "grids_window.hpp"
 #include <iomanip>
 
 namespace horizon {
@@ -55,11 +58,6 @@ void ImpLayer::construct_layer_box(bool pack)
 
     bool layers_loaded = false;
     if (!m_meta.is_null()) {
-        if (m_meta.count("grid_settings")) {
-            grid_controller->load_from_json(m_meta.at("grid_settings"));
-        }
-        else
-            grid_controller->set_spacing_square(m_meta.value("grid_spacing", 1.25_mm));
         if (m_meta.count("layer_display")) {
             layer_box->load_from_json(m_meta.at("layer_display"));
             layers_loaded = true;
@@ -101,15 +99,12 @@ void ImpLayer::get_save_meta(json &j)
 {
     j["layer_display"] = layer_box->serialize();
     j["grid_spacing"] = grid_controller->get_spacing_square();
-    j["grid_settings"] = grid_controller->serialize();
 }
 
 void ImpLayer::apply_preferences()
 {
     const auto &canvas_prefs = get_canvas_preferences();
-    for (const auto &it : canvas_prefs.appearance.layer_colors) {
-        layer_box->set_layer_color(it.first, it.second);
-    }
+    layer_box->set_appearance(canvas_prefs.appearance);
     ImpBase::apply_preferences();
 }
 
@@ -168,10 +163,7 @@ void ImpLayer::add_view_angle_actions()
 static std::string view_angle_to_string(int x)
 {
     const bool pos_only = false;
-    while (x < 0) {
-        x += 65536;
-    }
-    x %= 65536;
+    x = wrap_angle(x);
     if (!pos_only && x > 32768)
         x -= 65536;
 
@@ -204,7 +196,7 @@ void ImpLayer::set_view_angle(int angle)
     while (view_angle < 0) {
         view_angle += 65536;
     }
-    canvas->set_view_angle(angle * M_PI / 32768);
+    canvas->set_view_angle(angle_to_rad(angle));
     view_angle_label->set_text(view_angle_to_string(view_angle));
     view_angle_window_conn.block();
     view_angle_window->get_spinbutton().set_value(view_angle);
