@@ -78,6 +78,8 @@ enum class InToolActionID {
 
     // draw arc
     ARC_MODE,
+    // restrict drawing to 45deg
+    TOGGLE_DEG45_RESTRICT,
 };
 
 } // namespace horizon
