@@ -65,6 +65,18 @@ MainWindow::MainWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
     GET_WIDGET(key_hint_revealer);
     key_hint_size_group = Gtk::SizeGroup::create(Gtk::SIZE_GROUP_HORIZONTAL);
 
+    GET_WIDGET(instance_path_revealer);
+    GET_WIDGET(parent_block_button);
+    GET_WIDGET(instance_path_box);
+    GET_WIDGET(block_symbol_button);
+    GET_WIDGET(ports_button);
+    GET_WIDGET(hierarchy_stack);
+    GET_WIDGET(out_of_hierarchy_label);
+
+    GET_WIDGET(undo_redo_hint_frame);
+    GET_WIDGET(undo_redo_hint_label);
+    undo_redo_hint_frame->set_visible(false);
+
     set_version_info("");
 
     grid_options_button->signal_clicked().connect([this] {
@@ -100,7 +112,6 @@ MainWindow::MainWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
     label_set_tnum(tool_bar_tip_label);
     label_set_tnum(grid_mul_label);
     label_set_tnum(search_status_label);
-    label_set_tnum(view_hints_label);
     {
         auto attributes_list = pango_attr_list_new();
         auto attribute_font_features = pango_attr_font_features_new("tnum 1");
@@ -299,6 +310,32 @@ void MainWindow::key_hint_set_visible(bool show)
     }
 }
 
+void MainWindow::set_undo_redo_hint(const std::string &s)
+{
+    if (!undo_redo_hint_frame->get_visible()) {
+        undo_redo_hint_connection.disconnect();
+        undo_redo_hint_label->set_text(s);
+        undo_redo_hint_frame->set_visible(true);
+        undo_redo_hint_connection = Glib::signal_timeout().connect(
+                [this] {
+                    undo_redo_hint_frame->set_visible(false);
+                    return false;
+                },
+                1000);
+    }
+    else if (undo_redo_hint_frame->get_visible()) {
+        undo_redo_hint_connection.disconnect();
+        undo_redo_hint_frame->set_visible(false);
+        const std::string s_copy = s;
+        undo_redo_hint_connection = Glib::signal_timeout().connect(
+                [this, s_copy] {
+                    set_undo_redo_hint(s_copy);
+                    return false;
+                },
+                200);
+    }
+}
+
 MainWindow *MainWindow::create()
 {
     MainWindow *w;
