@@ -2,6 +2,9 @@
 #include "rule_match_editor.hpp"
 #include "rules/rule_match.hpp"
 #include "widgets/spin_button_dim.hpp"
+#include "widgets/layer_combo_box.hpp"
+#include "document/idocument.hpp"
+#include "common/layer_provider.hpp"
 
 namespace horizon {
 RuleEditor::RuleEditor(Rule &r, class IDocument &c) : Gtk::Box(Gtk::ORIENTATION_VERTICAL, 20), rule(r), core(c)
@@ -44,4 +47,23 @@ RuleMatchEditor *RuleEditor::create_rule_match_editor(const char *into, RuleMatc
     box->pack_start(*r, true, true, 0);
     return r;
 }
+
+LayerComboBox *RuleEditor::create_layer_combo(int &layer, bool show_any)
+{
+    auto layer_combo = Gtk::manage(new LayerComboBox);
+    for (const auto &it : core.get_layer_provider().get_layers()) {
+        if (it.second.copper)
+            layer_combo->prepend(it.second);
+    }
+    if (show_any) {
+        Layer layer_any(10000, "Any layer");
+        layer_combo->prepend(layer_any);
+    }
+    layer_combo->set_active_layer(layer);
+    layer_combo->signal_changed().connect([this, layer_combo, &layer] {
+        layer = layer_combo->get_active_layer();
+        s_signal_updated.emit();
+    });
+    return layer_combo;
+}
 } // namespace horizon
