@@ -15,6 +15,7 @@
 #include "rule_editor_clearance_copper_keepout.hpp"
 #include "rule_editor_layer_pair.hpp"
 #include "rule_editor_clearance_same_net.hpp"
+#include "rule_editor_shorted_pads.hpp"
 #include "rules/cache.hpp"
 #include "rules/rule_descr.hpp"
 #include "rules/rules.hpp"
@@ -44,6 +45,8 @@ public:
         label_order->set_xalign(1);
         label_text = Gtk::manage(new Gtk::Label());
         label_text->set_markup(t);
+        label_text->set_line_wrap(true);
+        label_text->set_width_chars(22);
         label_text->set_xalign(0);
         pack_start(*label_order, false, false, 0);
         pack_start(*label_text, true, true, 0);
@@ -270,7 +273,8 @@ RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builde
                 if (it) {
                     Gtk::TreeModel::Row row = *it;
                     if (row[tree_columns.has_location]) {
-                        s_signal_goto.emit(row[tree_columns.location], row[tree_columns.sheet]);
+                        s_signal_goto.emit(row[tree_columns.location], row[tree_columns.sheet],
+                                           row[tree_columns.instance_path]);
                     }
                 }
             });
@@ -290,10 +294,14 @@ RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builde
                     row_err[tree_columns.has_location] = it_err.has_location;
                     row_err[tree_columns.location] = it_err.location;
                     row_err[tree_columns.sheet] = it_err.sheet;
+                    row_err[tree_columns.instance_path] = it_err.instance_path;
                     row_err[tree_columns.running] = false;
 
                     if (it_err.has_location) {
-                        dom.emplace_back(it_err.location, rules_check_error_level_to_color(it_err.level), it_err.sheet);
+                        UUIDVec sheet;
+                        if (it_err.sheet)
+                            sheet = uuid_vec_append(it_err.instance_path, it_err.sheet);
+                        dom.emplace_back(it_err.location, rules_check_error_level_to_color(it_err.level), sheet);
                     }
 
                     for (const auto &path : it_err.error_polygons) {
@@ -350,7 +358,7 @@ void RulesWindow::apply_rules()
     for (auto &rule : rules.get_rule_ids()) {
         rules_apply(rules, rule, core);
     }
-    core.rebuild();
+    core.rebuild("apply rules");
     s_signal_canvas_update.emit();
     s_signal_changed.emit();
 }
@@ -469,7 +477,7 @@ void RulesWindow::run_checks()
     check_result_store->clear();
     auto &dom = canvas.markers.get_domain(MarkerDomain::CHECK);
     dom.clear();
-    cache.reset(new RulesCheckCache(&core));
+    cache.reset(new RulesCheckCache(core));
     annotation->clear();
     run_store.clear();
     pulse_connection = Glib::signal_timeout().connect(sigc::track_obj(
@@ -515,17 +523,15 @@ void RulesWindow::rule_selected(RuleID id)
         update_warning();
     }
     else {
-        auto rule = rules.get_rule(id);
-        show_editor(create_editor(*rule));
+        auto &rule = rules.get_rule(id);
+        show_editor(create_editor(rule));
     }
 }
 
 void RulesWindow::rule_instance_selected(RuleID id, const UUID &uu)
 {
-    auto rule = rules.get_rule(id, uu);
-    if (rule == nullptr)
-        return;
-    show_editor(create_editor(*rule));
+    auto &rule = rules.get_rule(id, uu);
+    show_editor(create_editor(rule));
 }
 
 void RulesWindow::reload_editor()
@@ -557,7 +563,7 @@ void RulesWindow::show_editor(RuleEditor *e)
 RuleEditor *RulesWindow::create_editor(Rule &r)
 {
     RuleEditor *e = nullptr;
-    switch (r.id) {
+    switch (r.get_id()) {
     case RuleID::HOLE_SIZE:
         e = new RuleEditorHoleSize(r, core);
         break;
@@ -614,6 +620,10 @@ RuleEditor *RulesWindow::create_editor(Rule &r)
         e = new RuleEditorClearanceSameNet(r, core);
         break;
 
+    case RuleID::SHORTED_PADS:
+        e = new RuleEditorShortedPads(r, core);
+        break;
+
     default:
         e = new RuleEditor(r, core);
     }
@@ -621,9 +631,9 @@ RuleEditor *RulesWindow::create_editor(Rule &r)
     return e;
 }
 
-Block *RulesWindow::get_block()
+Block *RulesWindow::get_top_block()
 {
-    return core.get_block();
+    return core.get_top_block();
 }
 
 void RulesWindow::update_rule_instances(RuleID id)
@@ -638,8 +648,8 @@ void RulesWindow::update_rule_instances(RuleID id)
         delete it;
     }
     for (const auto &it : inst) {
-        auto la = Gtk::manage(
-                new RuleLabel(sg_order, it.second->get_brief(get_block()), id, it.first, it.second->get_order()));
+        auto la = Gtk::manage(new RuleLabel(sg_order, it.second->get_brief(get_top_block(), &core.get_pool()), id,
+                                            it.first, it.second->get_order()));
         la->set_sensitive(it.second->enabled);
         la->set_imported(it.second->imported);
         la->show();
@@ -659,11 +669,11 @@ void RulesWindow::update_rule_instance_labels()
 {
     for (auto ch : lb_multi->get_children()) {
         auto la = dynamic_cast<RuleLabel *>(dynamic_cast<Gtk::ListBoxRow *>(ch)->get_child());
-        auto rule = rules.get_rule(la->id, la->uuid);
-        la->set_text(rule->get_brief(get_block()));
-        la->set_sensitive(rule->enabled);
-        la->set_order(rule->get_order());
-        la->set_imported(rule->imported);
+        auto &rule = rules.get_rule(la->id, la->uuid);
+        la->set_text(rule.get_brief(get_top_block(), &core.get_pool()));
+        la->set_sensitive(rule.enabled);
+        la->set_order(rule.get_order());
+        la->set_imported(rule.imported);
     }
     lb_multi->invalidate_sort();
 }
@@ -674,9 +684,9 @@ void RulesWindow::update_rule_labels()
         auto la = dynamic_cast<RuleLabel *>(dynamic_cast<Gtk::ListBoxRow *>(ch)->get_child());
         auto is_multi = rule_descriptions.at(la->id).multi;
         if (!is_multi) {
-            auto r = rules.get_rule(la->id);
-            la->set_sensitive(r->enabled);
-            la->set_imported(r->imported);
+            auto &r = rules.get_rule(la->id);
+            la->set_sensitive(r.enabled);
+            la->set_imported(r.imported);
         }
         else {
             auto r = rules.get_rules(la->id);
