@@ -10,14 +10,14 @@ namespace horizon {
 
 class ParametricEditor : public Gtk::Grid, public Changeable {
 public:
-    ParametricEditor(PoolParametric *p, const std::string &t);
+    ParametricEditor(PoolParametric &p, const std::string &t);
 
     void update(const std::map<std::string, std::string> &params);
 
     std::map<std::string, std::string> get_values();
 
 private:
-    PoolParametric *pool;
+    PoolParametric &pool;
     const PoolParametric::Table &table;
     std::map<std::string, class ParametricParamEditor *> editors;
 };
