@@ -1,5 +1,6 @@
 #include "kicad_symbol_import_wizard.hpp"
 #include "util/util.hpp"
+#include "util/geom_util.hpp"
 #include "util/str_util.hpp"
 #include "util/gtk_util.hpp"
 #include "widgets/pool_browser_package.hpp"
@@ -15,7 +16,7 @@ namespace horizon {
 
 KiCadSymbolImportWizard::KiCadSymbolImportWizard(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x,
                                                  const UUID &pkg_uuid, class Pool &po,
-                                                 class PoolProjectManagerAppWindow *aw, const std::string &lib_filename)
+                                                 class PoolProjectManagerAppWindow &aw, const std::string &lib_filename)
     : Gtk::Window(cobject), pool(po), appwin(aw), state_store(this, "kicad-symbol-import-wizard")
 {
     GET_WIDGET(header);
@@ -146,7 +147,7 @@ KiCadSymbolImportWizard::KiCadSymbolImportWizard(BaseObjectType *cobject, const
 
         Gtk::MessageDialog md(*this, "Really close?", false /* use_markup */, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
         md.set_secondary_text(
-                "By closing the part wizard, all changes to the new part will "
+                "By closing the wizard, all changes to the new items will "
                 "be lost");
         md.add_button("Close and discard changes", 1);
         md.add_button("Keep open", Gtk::RESPONSE_CANCEL);
@@ -454,7 +455,7 @@ void KiCadSymbolImportWizard::select_symbol()
 void KiCadSymbolImportWizard::handle_edit_entity()
 {
     std::string entity_filename = pool.get_tmp_filename(ObjectType::ENTITY, entity_uuid);
-    auto proc = appwin->spawn(PoolProjectManagerProcess::Type::ENTITY, {entity_filename});
+    auto proc = appwin.spawn(PoolProjectManagerProcess::Type::ENTITY, {entity_filename});
     processes.emplace(entity_filename, proc);
     proc->signal_exited().connect([this, entity_filename](int status, bool modified) {
         processes.erase(entity_filename);
@@ -477,7 +478,7 @@ void KiCadSymbolImportWizard::handle_edit_entity()
 void KiCadSymbolImportWizard::handle_edit_part()
 {
     std::string part_filename = pool.get_tmp_filename(ObjectType::PART, part_uuid);
-    auto proc = appwin->spawn(PoolProjectManagerProcess::Type::PART, {part_filename});
+    auto proc = appwin.spawn(PoolProjectManagerProcess::Type::PART, {part_filename});
     processes.emplace(part_filename, proc);
     proc->signal_exited().connect([this, part_filename](int status, bool modified) {
         processes.erase(part_filename);
@@ -521,6 +522,7 @@ LocationEntry *KiCadSymbolImportWizard::pack_location_entry(const Glib::RefPtr<G
                                                             Gtk::Button **button_other)
 {
     auto en = Gtk::manage(new LocationEntry(pool.get_base_path()));
+    en->set_append_json(true);
     if (button_other) {
         *button_other = Gtk::manage(new Gtk::Button());
         en->pack_start(**button_other, false, false);
@@ -575,7 +577,7 @@ void KiCadSymbolImportWizard::update_can_finish()
 }
 
 KiCadSymbolImportWizard *KiCadSymbolImportWizard::create(const UUID &pkg_uuid, class Pool &po,
-                                                         class PoolProjectManagerAppWindow *aw,
+                                                         class PoolProjectManagerAppWindow &aw,
                                                          const std::string &lib_filename)
 {
     KiCadSymbolImportWizard *w;
