@@ -8,23 +8,21 @@
 #include "util/sqlite.hpp"
 #include <git2.h>
 #include "util/sort_controller.hpp"
-
-class git_repository;
-
+#include "rules/rules.hpp"
 
 namespace horizon {
 using json = nlohmann::json;
 
 class PoolGitBox : public Gtk::Box {
 public:
-    PoolGitBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, class PoolNotebook *nb);
-    static PoolGitBox *create(class PoolNotebook *nb);
+    PoolGitBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, class PoolNotebook &nb);
+    static PoolGitBox *create(class PoolNotebook &nb);
 
     void refresh();
     bool refreshed_once = false;
 
 private:
-    class PoolNotebook *notebook = nullptr;
+    class PoolNotebook &notebook;
 
     Gtk::Button *refresh_button = nullptr;
     Gtk::Label *info_label = nullptr;
@@ -51,6 +49,7 @@ private:
 
             Gtk::TreeModelColumnRecord::add(status);
             Gtk::TreeModelColumnRecord::add(path);
+            Gtk::TreeModelColumnRecord::add(check_result);
         }
         Gtk::TreeModelColumn<Glib::ustring> name;
         Gtk::TreeModelColumn<ObjectType> type;
@@ -58,6 +57,7 @@ private:
 
         Gtk::TreeModelColumn<unsigned int> status;
         Gtk::TreeModelColumn<std::string> path;
+        Gtk::TreeModelColumn<RulesCheckResult> check_result;
     };
     TreeColumns list_columns;
 
