@@ -259,6 +259,7 @@ PoolNotebook::PoolNotebook(const std::string &bp, class PoolProjectManagerAppWin
         settings_box = PoolSettingsBox::create(pool);
         settings_box->signal_open_pool().connect(
                 [this](auto pool_bp) { appwin.app.open_pool(Glib::build_filename(pool_bp, "pool.json")); });
+        settings_box->signal_saved().connect([this] { s_signal_saved.emit(); });
         pool_uuid = pool.get_pool_info().uuid;
 
         settings_box->show();
@@ -328,6 +329,8 @@ PoolNotebook::PoolNotebook(const std::string &bp, class PoolProjectManagerAppWin
                         return;
                     }
                 }
+                if (settings_box)
+                    settings_box->update_pools();
             },
             *this));
 }
@@ -372,8 +375,8 @@ void PoolNotebook::add_context_menu(PoolBrowser *br)
             });
     br->add_context_menu_item(
             "Move to other pool",
-            [this](const UUID &uu) {
-                auto win = MoveWindow::create(pool, ObjectType::PART, uu);
+            [this, ty](const UUID &uu) {
+                auto win = MoveWindow::create(pool, ty, uu);
                 auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
                 win->set_transient_for(*top);
                 win->present();
@@ -387,17 +390,9 @@ void PoolNotebook::add_context_menu(PoolBrowser *br)
             this_pool_lambda);
 }
 
-PoolNotebook::ItemPoolInfo PoolNotebook::get_pool_uuids(ObjectType ty, const UUID &uu)
+Pool::ItemPoolInfo PoolNotebook::get_pool_uuids(ObjectType ty, const UUID &uu)
 {
-    SQLite::Query q(pool.get_db(), "SELECT pool_uuid, last_pool_uuid FROM all_items_view WHERE type = ? AND uuid = ?");
-    q.bind(1, ty);
-    q.bind(2, uu);
-    if (q.step()) {
-        const UUID pool_uu(q.get<std::string>(0));
-        const UUID last_pool_uu(q.get<std::string>(1));
-        return {pool_uu, last_pool_uu};
-    }
-    throw std::runtime_error("item not found");
+    return pool.get_pool_uuids(ty, uu);
 }
 
 void PoolNotebook::handle_delete(ObjectType ty, const UUID &uu)
