@@ -34,6 +34,12 @@ public:
     ~PoolNotebook();
     PoolProjectManagerAppWindow &appwin;
 
+    typedef sigc::signal<void> type_signal_saved;
+    type_signal_saved signal_saved()
+    {
+        return s_signal_saved;
+    }
+
 private:
     const std::string base_path;
     Pool pool;
@@ -111,11 +117,8 @@ private:
     void handle_move_rename(ObjectType ty, const UUID &uu);
     void handle_copy_path(ObjectType ty, const UUID &uu);
     void add_context_menu(class PoolBrowser *br);
-    struct ItemPoolInfo {
-        UUID pool;
-        UUID last;
-    };
-    ItemPoolInfo get_pool_uuids(ObjectType ty, const UUID &uu);
+
+    Pool::ItemPoolInfo get_pool_uuids(ObjectType ty, const UUID &uu);
     void handle_edit_item(ObjectType ty, const UUID &uu);
     void handle_duplicate_item(ObjectType ty, const UUID &uu);
 
@@ -135,5 +138,7 @@ private:
 
     void pool_updated();
     bool pool_busy = false;
+
+    type_signal_saved s_signal_saved;
 };
 } // namespace horizon
