@@ -1,23 +1,15 @@
 #include "pool_notebook.hpp"
-#include "editors/editor_window.hpp"
-#include "duplicate/duplicate_unit.hpp"
 #include "util/util.hpp"
 #include "nlohmann/json.hpp"
 #include "pool-prj-mgr/pool-prj-mgr-app_win.hpp"
 #include "widgets/pool_browser_padstack.hpp"
 #include "widgets/padstack_preview.hpp"
 #include "widgets/where_used_box.hpp"
-#include "pool_remote_box.hpp"
 
 namespace horizon {
 void PoolNotebook::handle_edit_padstack(const UUID &uu)
 {
-    if (!uu)
-        return;
-    UUID item_pool_uuid;
-    auto path = pool.get_filename(ObjectType::PADSTACK, uu, &item_pool_uuid);
-    appwin->spawn(PoolProjectManagerProcess::Type::IMP_PADSTACK, {path}, {},
-                  pool_uuid && (item_pool_uuid != pool_uuid));
+    handle_edit_item(ObjectType::PADSTACK, uu);
 }
 
 void PoolNotebook::handle_create_padstack()
@@ -32,47 +24,24 @@ void PoolNotebook::handle_create_padstack()
     chooser->set_current_folder(Glib::build_filename(base_path, "padstacks"));
 
     if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-        std::string fn = EditorWindow::fix_filename(chooser->get_filename());
+        std::string fn = append_dot_json(chooser->get_filename());
         Padstack ps(horizon::UUID::random());
         save_json_to_file(fn, ps.serialize());
-        appwin->spawn(PoolProjectManagerProcess::Type::IMP_PADSTACK, {fn});
+        appwin.spawn(PoolProjectManagerProcess::Type::IMP_PADSTACK, {fn});
     }
 }
 
 void PoolNotebook::handle_duplicate_padstack(const UUID &uu)
 {
-    if (!uu)
-        return;
-    auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
-
-    GtkFileChooserNative *native =
-            gtk_file_chooser_native_new("Save Padstack", top->gobj(), GTK_FILE_CHOOSER_ACTION_SAVE, "_Save", "_Cancel");
-    auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
-    chooser->set_do_overwrite_confirmation(true);
-    auto ps_filename = pool.get_filename(ObjectType::PADSTACK, uu);
-    auto ps_basename = Glib::path_get_basename(ps_filename);
-    auto ps_dirname = Glib::path_get_dirname(ps_filename);
-    chooser->set_current_folder(ps_dirname);
-    chooser->set_current_name(DuplicateUnitWidget::insert_filename(ps_basename, "-copy"));
-
-    if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-        std::string fn = EditorWindow::fix_filename(chooser->get_filename());
-        Padstack ps(*pool.get_padstack(uu));
-        ps.name += " (Copy)";
-        ps.uuid = UUID::random();
-        save_json_to_file(fn, ps.serialize());
-        pool_update(nullptr, {fn});
-        appwin->spawn(PoolProjectManagerProcess::Type::IMP_PADSTACK, {fn});
-    }
+    handle_duplicate_item(ObjectType::PADSTACK, uu);
 }
 
 void PoolNotebook::construct_padstacks()
 {
     auto br = Gtk::manage(new PoolBrowserPadstack(pool));
     br->set_show_path(true);
-    br->set_include_padstack_type(Padstack::Type::VIA, true);
-    br->set_include_padstack_type(Padstack::Type::MECHANICAL, true);
-    br->set_include_padstack_type(Padstack::Type::HOLE, true);
+    using PT = Padstack::Type;
+    br->set_padstacks_included({PT::BOTTOM, PT::HOLE, PT::MECHANICAL, PT::THROUGH, PT::TOP, PT::VIA});
     br->signal_activated().connect([this, br] { handle_edit_padstack(br->get_selected()); });
 
     br->show();
@@ -90,9 +59,7 @@ void PoolNotebook::construct_padstacks()
     add_action_button("Create", bbox, sigc::mem_fun(*this, &PoolNotebook::handle_create_padstack));
     add_action_button("Edit", bbox, br, sigc::mem_fun(*this, &PoolNotebook::handle_edit_padstack));
     add_action_button("Duplicate", bbox, br, sigc::mem_fun(*this, &PoolNotebook::handle_duplicate_padstack));
-    if (remote_repo.size())
-        add_action_button("Merge", bbox, br,
-                          [this](const UUID &uu) { remote_box->merge_item(ObjectType::PADSTACK, uu); });
+    add_merge_button(bbox, br);
 
     auto stack = Gtk::manage(new Gtk::Stack);
     add_preview_stack_switcher(bbox, stack);
@@ -132,6 +99,7 @@ void PoolNotebook::construct_padstacks()
 
     append_page(*paned, "Padstacks");
     install_search_once(paned, br);
+    create_paned_state_store(paned, "padstacks");
 }
 
 } // namespace horizon
