@@ -528,7 +528,7 @@ PullRequestItemBox::PullRequestItemBox(BaseObjectType *cobject, const Glib::RefP
     name_label->set_text(j.at("title").get<std::string>());
     number_label->set_text("#" + std::to_string(pr_number));
     link_button->set_uri(j.at("_links").at("html").at("href").get<std::string>());
-    const std::string open_user = j.at("user").at("login");
+    const auto open_user = j.at("user").at("login").get<std::string>();
 
     const bool is_pool_mgr_branch = Glib::Regex::match_simple(
             "^pool_mgr_[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-"
@@ -536,7 +536,7 @@ PullRequestItemBox::PullRequestItemBox(BaseObjectType *cobject, const Glib::RefP
             branch_name);
 
     update_button->set_visible(!no_update && open_user == gh_user && is_pool_mgr_branch);
-    const std::string created_at = j.at("created_at");
+    const auto created_at = j.at("created_at").get<std::string>();
     descr_label->set_text("opened by " + open_user + " on " + created_at);
 }
 
@@ -574,7 +574,7 @@ void PoolRemoteBox::update_prs()
 
     for (const auto &it : pull_requests) {
         if (gh_username.size() && show_only_my_prs_cb->get_active()) {
-            std::string user = it.at("user").at("login");
+            const auto user = it.at("user").at("login").get<std::string>();
             if (user != gh_username)
                 continue;
         }
@@ -783,7 +783,6 @@ void PoolRemoteBox::refresh_prs_thread()
         pr_status_dispatcher.set_status(StatusDispatcher::Status::BUSY, "Refreshing");
         GitHubClient client;
         pull_requests = client.get_pull_requests(gh_owner, gh_repo);
-        std::cout << "get prs" << std::endl;
         /*pull_requests = json::array();
         {
                 json j;
@@ -854,12 +853,12 @@ void PoolRemoteBox::login_thread()
 
         auto j_auth = load_json_from_file(get_token_filename());
 
-        auto token = j_auth.at("token");
+        const auto token = j_auth.at("token").get<std::string>();
 
         GitHubClient client;
         auto user_info = client.login_token(token);
 
-        gh_username = user_info.at("login");
+        gh_username = user_info.at("login").get<std::string>();
         gh_token = token;
 
         {
@@ -941,7 +940,6 @@ git_oid PoolRemoteBox::items_to_tree(git_repository *repo)
     files_merge.insert(models_merge.begin(), models_merge.end());
 
     for (std::string filename : files_merge) {
-        std::cout << "merge " << filename << std::endl;
         std::string filename_src = Glib::build_filename(notebook.base_path, filename);
         std::string filename_dest = Glib::build_filename(notebook.remote_repo, filename);
         std::string dirname_dest = Glib::path_get_dirname(filename_dest);
@@ -1036,7 +1034,7 @@ autofree_ptr<git_remote> PoolRemoteBox::get_or_create_remote(GitHubClient &clien
             throw std::runtime_error("timeout waiting for fork");
         }
 
-        std::string fork_url = fork_info.at("clone_url");
+        const auto fork_url = fork_info.at("clone_url").get<std::string>();
         if (git_remote_create(&my_remote.ptr, repo, gh_username.c_str(), fork_url.c_str()) != 0) {
             throw std::runtime_error("error adding remote");
         }
