@@ -17,6 +17,7 @@ public:
     bool get_needs_save() const;
     void save();
     void pool_updated();
+    void update_pools();
 
     typedef sigc::signal<void, std::string> type_signal_open_pool;
     type_signal_open_pool signal_open_pool()
@@ -24,6 +25,12 @@ public:
         return s_signal_open_pool;
     }
 
+    typedef sigc::signal<void> type_signal_saved;
+    type_signal_saved signal_saved()
+    {
+        return s_signal_saved;
+    }
+
 private:
     IPool &pool;
     Gtk::Entry *entry_name = nullptr;
@@ -39,15 +46,16 @@ private:
 
     std::vector<UUID> pools_included;
 
-    void update_pools();
     void update_actual();
 
     void inc_excl_pool(bool inc);
     void pool_up_down(bool up);
+    void update_button_sensitivity();
 
     bool needs_save = false;
     void set_needs_save();
 
     type_signal_open_pool s_signal_open_pool;
+    type_signal_saved s_signal_saved;
 };
 } // namespace horizon
