@@ -3,12 +3,12 @@
 #include "view_create_pool.hpp"
 #include "nlohmann/json.hpp"
 #include "util/util.hpp"
+#include "pool/pool.hpp"
 #include <fstream>
 
 namespace horizon {
 
-PoolProjectManagerViewCreatePool::PoolProjectManagerViewCreatePool(const Glib::RefPtr<Gtk::Builder> &builder,
-                                                                   class PoolProjectManagerAppWindow *w)
+PoolProjectManagerViewCreatePool::PoolProjectManagerViewCreatePool(const Glib::RefPtr<Gtk::Builder> &builder)
 {
     builder->get_widget("create_pool_name_entry", pool_name_entry);
     builder->get_widget("create_pool_path_chooser", pool_path_chooser);
@@ -49,28 +49,27 @@ std::pair<bool, std::string> PoolProjectManagerViewCreatePool::create()
             }
         }
         auto pool_json = Glib::build_filename(base_path, "pool.json");
-        json j;
-        j["type"] = "pool";
-        j["name"] = static_cast<std::string>(pool_name_entry->get_text());
-        j["default_via"] = (std::string)UUID();
-        j["uuid"] = (std::string)UUID::random();
+        PoolInfo pool_info;
+        pool_info.name = pool_name_entry->get_text();
+        pool_info.default_via = UUID();
+        pool_info.uuid = UUID::random();
+        pool_info.base_path = base_path;
+        pool_info.save();
 
-        mkdir_pool(base_path, "units");
-        mkdir_pool(base_path, "symbols");
-        mkdir_pool(base_path, "entities");
-        mkdir_pool(base_path, "padstacks");
-        mkdir_pool(base_path, "packages");
-        mkdir_pool(base_path, "parts");
+        for (const auto &[type, name] : Pool::type_names) {
+            mkdir_pool(base_path, name);
+        }
         mkdir_pool(base_path, "3d_models");
 
-        save_json_to_file(pool_json, j);
-
         {
             auto ofs = make_ofstream(Glib::build_filename(base_path, ".gitignore"));
-            ofs << "*.db\n\
-tmp/*.json\n\
-3d_models/local\n\
-";
+            ofs << "*.db\n"
+                   "*.db-*\n"
+                   "tmp/*.json\n"
+                   "3d_models/local\n"
+                   "*.imp_meta\n"
+                   "*.autosave\n"
+                   "*.bak";
         }
 
         s = pool_json;
