@@ -51,6 +51,12 @@ public:
         return s_signal_pool_updated;
     }
 
+    struct CloseOrHomeWindow {
+        class PoolProjectManagerAppWindow &win;
+        bool close = true;
+    };
+    bool close_windows(std::vector<CloseOrHomeWindow> windows);
+
 protected:
     // Override default signal handlers:
     void on_activate() override;
@@ -78,8 +84,8 @@ private:
     type_signal_pool_items_edited s_signal_pool_items_edited;
     type_signal_pool_updated s_signal_pool_updated;
 
-
 public:
+    const UUID ipc_cookie;
     zmq::socket_t sock_broadcast;
 };
 } // namespace horizon
