@@ -7,7 +7,7 @@
 #include "pool/pool_manager.hpp"
 #include "pool/project_pool.hpp"
 #include "pool-mgr/pool_notebook.hpp"
-#include "forced_pool_update_dialog.hpp"
+#include "util/pool_check_schema_update.hpp"
 #include "util/gtk_util.hpp"
 #include "util/recent_util.hpp"
 #include "widgets/recent_item_box.hpp"
@@ -28,6 +28,8 @@
 #include "pool-update/pool-update.hpp"
 #include "pool_update_error_dialog.hpp"
 #include "project/project.hpp"
+#include "blocks/blocks_schematic.hpp"
+#include "project/project.hpp"
 #include <filesystem>
 
 #ifdef G_OS_WIN32
@@ -71,6 +73,7 @@ PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject
     builder->get_widget("info_bar_version", info_bar_version);
     builder->get_widget("version_label", version_label);
     builder->get_widget("info_bar_gitignore", info_bar_gitignore);
+    builder->get_widget("info_bar_installation_uuid_mismatch", info_bar_installation_uuid_mismatch);
 
     set_view_mode(ViewMode::OPEN);
 
@@ -139,6 +142,10 @@ PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject
         info_bar_hide(info_bar_gitignore);
     });
 
+    info_bar_hide(info_bar_installation_uuid_mismatch);
+    info_bar_installation_uuid_mismatch->signal_response().connect(
+            [this](int resp) { info_bar_hide(info_bar_installation_uuid_mismatch); });
+
     show_output_button->signal_clicked().connect([this] {
         output_window->present();
         info_bar_hide(info_bar);
@@ -158,7 +165,11 @@ PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject
                                               zmq::message_t msg;
                                               if (zmq_helper::recv(sock_mgr, msg)) {
                                                   char *data = (char *)msg.data();
-                                                  json jrx = json::parse(data);
+                                                  if (msg.size() < (UUID::size + 1))
+                                                      throw std::runtime_error("received short message");
+                                                  if (memcmp(data, app.ipc_cookie.get_bytes(), UUID::size) != 0)
+                                                      throw std::runtime_error("IPC cookie didn't match");
+                                                  json jrx = json::parse(data + UUID::size);
                                                   json jtx = handle_req(jrx);
 
                                                   std::string stx = jtx.dump();
@@ -387,7 +398,7 @@ PoolProjectManagerProcess *PoolProjectManagerAppWindow::find_top_schematic_proce
 {
     if (!project)
         return nullptr;
-    return find_process(project->get_top_block().schematic_filename);
+    return find_process(project->blocks_filename);
 }
 
 PoolProjectManagerProcess *PoolProjectManagerAppWindow::find_board_process()
@@ -412,7 +423,7 @@ bool PoolProjectManagerAppWindow::check_pools()
 
 json PoolProjectManagerAppWindow::handle_req(const json &j)
 {
-    std::string op = j.at("op");
+    const auto op = j.at("op").get<std::string>();
     guint32 timestamp = j.value("time", 0);
     if (op == "part-placed") {
         UUID part = j.at("part").get<std::string>();
@@ -514,14 +525,14 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
         }
     }
     else if (op == "needs-save") {
-        int pid = j.at("pid");
-        bool needs_save = j.at("needs_save");
+        const auto pid = j.at("pid").get<int>();
+        const auto needs_save = j.at("needs_save").get<bool>();
         std::cout << "needs save " << pid << " " << needs_save << std::endl;
         pids_need_save[pid] = needs_save;
         if (project && !needs_save)
             view_project.update_meta();
         if (!needs_save) {
-            s_signal_process_saved.emit(j.at("filename"));
+            s_signal_process_saved.emit(j.at("filename").get<std::string>());
             if (auto proc = find_top_schematic_process()) {
                 if (proc->proc->get_pid() == pid) { // schematic got saved
                     if (auto proc_board = find_board_process()) {
@@ -535,7 +546,7 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
         }
     }
     else if (op == "ready") {
-        int pid = j.at("pid");
+        const auto pid = j.at("pid").get<int>();
         for (auto &it : processes) {
             if (it.second.proc && it.second.proc->get_pid() == pid) {
                 it.second.signal_ready().emit();
@@ -575,7 +586,7 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
     else if (op == "preferences") {
         auto win = app.show_preferences_window(timestamp);
         if (j.count("page"))
-            win->show_page(j.at("page"));
+            win->show_page(j.at("page").get<std::string>());
     }
     else if (op == "show-in-pool-mgr") {
         UUID uu = j.at("uuid").get<std::string>();
@@ -616,6 +627,9 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
             }
         }
     }
+    else if (op == "focus") {
+        present(timestamp);
+    }
     return nullptr;
 }
 
@@ -780,12 +794,7 @@ static std::optional<std::string> peek_name(const std::string &path)
         }
         else {
             auto prj = Project::new_from_file(path);
-            auto block_filename = prj.get_top_block().block_filename;
-            auto meta = Block::peek_project_meta(block_filename);
-            if (meta.count("project_title"))
-                name = meta.at("project_title");
-            if (!name.size())
-                name = load_json_from_file(path).at("title");
+            return prj.peek_title();
         }
     }
     catch (...) {
@@ -830,61 +839,19 @@ void PoolProjectManagerAppWindow::prepare_close()
 
 bool PoolProjectManagerAppWindow::close_pool_or_project()
 {
-    auto pol = get_close_policy();
-
-    if (!pol.can_close) {
-        Gtk::MessageDialog md(*this, "Can't close right now", false /* use_markup */, Gtk::MESSAGE_ERROR,
-                              Gtk::BUTTONS_OK);
-        md.set_secondary_text(pol.reason);
-        md.run();
-        return false;
-    }
-    else if (pol.procs_need_save.size()) {
-        ConfirmCloseDialog dia(this);
-        std::map<std::string, std::map<UUID, std::string>> files;
-        auto &this_files = files[get_filename()];
-        for (const auto &it : pol.procs_need_save) {
-            this_files[it] = get_proc_filename(it);
-        }
-        dia.set_files(files);
-        auto r = dia.run();
-        if (r == ConfirmCloseDialog::RESPONSE_NO_SAVE || r == ConfirmCloseDialog::RESPONSE_SAVE) { // save
-            prepare_close();
-            auto files_from_dia = dia.get_files();
-            auto &this_files_from_dia = files_from_dia.at(get_filename());
-            if (r == ConfirmCloseDialog::RESPONSE_SAVE) {
-                for (auto &it : this_files_from_dia) { // files that need save
-                    process_save(it);
-                }
+    std::vector<PoolProjectManagerApplication::CloseOrHomeWindow> windows;
+    windows.push_back({*this, false});
+    if (project) {
+        auto pool_path = Glib::build_filename(project->pool_directory, "pool.json");
+        for (auto ws : app.get_windows()) {
+            auto wi = dynamic_cast<PoolProjectManagerAppWindow *>(ws);
+            if (wi && wi->get_filename() == pool_path) {
+                windows.push_back({*wi, true});
+                break;
             }
-            std::set<UUID> open_procs;
-            for (auto &it : processes) {
-                open_procs.emplace(it.first);
-            }
-            for (auto &it : open_procs) {
-                process_close(it);
-            }
-            wait_for_all_processes();
-            return really_close_pool_or_project();
-        }
-        else {
-            return false;
         }
     }
-    else {
-        prepare_close();
-        std::set<UUID> open_procs;
-        for (auto &it : processes) {
-            open_procs.emplace(it.first);
-        }
-        for (auto &it : open_procs) {
-            process_close(it);
-        }
-        wait_for_all_processes();
-        return really_close_pool_or_project();
-    }
-
-    return really_close_pool_or_project();
+    return app.close_windows(windows);
 }
 
 
@@ -959,6 +926,7 @@ void PoolProjectManagerAppWindow::set_view_mode(ViewMode mode)
         header->set_title("Horizon EDA");
         update_recent_items();
         set_version_info("");
+        info_bar_hide(info_bar_installation_uuid_mismatch);
         break;
 
     case ViewMode::POOL:
@@ -1035,32 +1003,22 @@ PoolProjectManagerAppWindow *PoolProjectManagerAppWindow::create(PoolProjectMana
     return window;
 }
 
-bool PoolProjectManagerAppWindow::check_schema_update(const std::string &base_path)
+void PoolProjectManagerAppWindow::check_schema_update(const std::string &base_path)
 {
-    bool update_required = false;
-    try {
-        Pool my_pool(base_path);
-        int user_version = my_pool.db.get_user_version();
-        int required_version = my_pool.get_required_schema_version();
-        update_required = user_version != required_version;
-    }
-    catch (...) {
-        update_required = true;
-    }
-    if (update_required) {
-        ForcedPoolUpdateDialog dia(base_path, this);
-        while (dia.run() != 1) {
-        }
-    }
-    return true;
+    const auto r = pool_check_schema_update(base_path, *this);
+    if (r == CheckSchemaUpdateResult::INSTALLATION_UUID_MISMATCH)
+        info_bar_show(info_bar_installation_uuid_mismatch);
+    if (r != CheckSchemaUpdateResult::NO_UPDATE)
+        app.signal_pool_updated().emit(base_path);
 }
 
+
 void PoolProjectManagerAppWindow::check_pool_update(const std::string &base_path)
 {
     Pool my_pool(base_path);
     for (auto &[bp, uu] : my_pool.get_actually_included_pools(false)) {
-        Pool inc_pool(bp);
-        if (inc_pool.db.get_user_version() == my_pool.db.get_user_version()) {
+        PoolInfo pool_info(bp);
+        if (pool_info.is_usable()) {
             {
                 SQLite::Query q(my_pool.db, "ATTACH ? as inc");
                 q.bind(1, (fs::u8path(bp) / "pool.db").u8string());
@@ -1109,6 +1067,7 @@ bool PoolProjectManagerAppWindow::migrate_project(const std::string &path)
             ProjectPool::create_directories(info.base_path);
             info.save();
             save_json_to_file(path, prj.serialize());
+            prj.create_blocks();
 
             // copy pool items
             for (const auto &file : fs::directory_iterator(cache_directory)) {
@@ -1158,6 +1117,26 @@ bool PoolProjectManagerAppWindow::migrate_project(const std::string &path)
     return true;
 }
 
+#if GTK_CHECK_VERSION(3, 24, 0)
+gboolean PoolProjectManagerAppWindow::part_browser_key_pressed(GtkEventControllerKey *controller, guint keyval,
+                                                               guint keycode, GdkModifierType state, gpointer user_data)
+{
+    if (keyval != GDK_KEY_Escape)
+        return FALSE;
+
+    auto self = reinterpret_cast<PoolProjectManagerAppWindow *>(user_data);
+    if (auto proc = self->find_top_schematic_process()) {
+        auto pid = proc->proc->get_pid();
+        json tx;
+        tx["op"] = "present";
+        tx["time"] = gtk_get_current_event_time();
+        self->app.send_json(pid, tx);
+    }
+
+    return TRUE;
+}
+#endif
+
 void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &file)
 {
     auto path = file->get_path();
@@ -1186,9 +1165,19 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
             if (!PoolManager::get().get_pools().count(pool_base_path) && !pool->get_pool_info().is_project_pool()) {
                 info_bar_show(info_bar_pool_not_added);
             }
+            pool_notebook->signal_saved().connect([this] { set_version_info(""); });
             pool_box->pack_start(*pool_notebook, true, true, 0);
             pool_notebook->show();
             header->set_subtitle(pool_base_path);
+            const auto &version = pool->get_pool_info().version;
+            const auto version_msg = version.get_message(ObjectType::POOL);
+            if (version_msg.size()) {
+                set_version_info(version_msg
+                                 + " This only applies to the pool.json file. Pool items have different versions.");
+            }
+            else {
+                set_version_info("");
+            }
 
             if (pool->get_pool_info().is_project_pool()) {
                 button_close->hide();
@@ -1220,6 +1209,8 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
         }
         project = std::make_unique<Project>(Project::new_from_file(path));
         project_filename = path;
+        project_needs_save = false;
+
         check_schema_update(project->pool_directory);
 
         check_pool_update(project->pool_directory);
@@ -1236,9 +1227,15 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
             else {
                 set_version_info("");
             }
+
             if (version.get_app() < version.get_file()) {
                 project_read_only = true;
             }
+            if (version.get_app() > version.get_file()) {
+                // need to create blocks.json
+                project->create_blocks();
+                project_needs_save = true;
+            }
         }
 
         if (Project::gitignore_needs_fixing(get_gitignore_filename())) {
@@ -1255,15 +1252,17 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
         part_browser_window->signal_assign_part().connect(
                 sigc::mem_fun(*this, &PoolProjectManagerAppWindow::handle_assign_part));
 
+#if GTK_CHECK_VERSION(3, 24, 0)
+        {
+            auto ctrl = gtk_event_controller_key_new(GTK_WIDGET(part_browser_window->gobj()));
+            gtk_event_controller_set_propagation_phase(ctrl, GTK_PHASE_CAPTURE);
+            g_signal_connect(ctrl, "key-pressed", G_CALLBACK(part_browser_key_pressed), this);
+        }
+#endif
+
         set_view_mode(ViewMode::PROJECT);
 
-        if (view_project.update_meta() == false) {
-            Gtk::MessageDialog md(*this, "Metadata update required", false /* use_markup */, Gtk::MESSAGE_ERROR,
-                                  Gtk::BUTTONS_OK);
-            md.set_secondary_text(
-                    "Open and save the schematic for the project title to appear in the project manager.");
-            md.run();
-        }
+        view_project.update_meta();
     }
 }
 
@@ -1326,7 +1325,8 @@ PoolProjectManagerProcess *PoolProjectManagerAppWindow::spawn(PoolProjectManager
 
         std::vector<std::string> env = {"HORIZON_POOL=" + pool_base_path,
                                         "HORIZON_EP_BROADCAST=" + app.get_ep_broadcast(),
-                                        "HORIZON_EP_MGR=" + sock_mgr_ep, "HORIZON_MGR_PID=" + std::to_string(getpid())};
+                                        "HORIZON_EP_MGR=" + sock_mgr_ep, "HORIZON_MGR_PID=" + std::to_string(getpid()),
+                                        "HORIZON_IPC_COOKIE=" + (std::string)app.ipc_cookie};
         std::string filename = args.at(0);
         if (filename.size()) {
             if (!Glib::file_test(filename, Glib::FILE_TEST_IS_REGULAR)) {
@@ -1401,11 +1401,11 @@ bool PoolProjectManagerAppWindow::check_autosave(PoolProjectManagerProcess::Type
         return true;
 
     std::vector<std::string> my_filenames = filenames;
+    my_filenames.resize(1);
     if (type == PoolProjectManagerProcess::Type::IMP_SCHEMATIC) {
-        my_filenames.resize(2);
-    }
-    else {
-        my_filenames.resize(1);
+        // is blocks file, get filename
+        const auto filenames_from_blocks = BlocksBase::peek_filenames(my_filenames.front());
+        my_filenames.insert(my_filenames.end(), filenames_from_blocks.begin(), filenames_from_blocks.end());
     }
 
     bool have_autosave = std::all_of(my_filenames.begin(), my_filenames.end(), [](auto &x) {
@@ -1564,19 +1564,20 @@ bool PoolProjectManagerAppWindow::cleanup_pool_cache(Gtk::Window *parent)
     ItemSet items_needed;
 
     ProjectPool project_pool(project->pool_directory, false);
-    auto block = Block::new_from_file(project->get_top_block().block_filename, project_pool);
-    auto sch = Schematic::new_from_file(project->get_top_block().schematic_filename, block, project_pool);
-    sch.expand();
-    auto board = Board::new_from_file(project->board_filename, block, project_pool);
+    auto blocks = BlocksSchematic::new_from_file(project->blocks_filename, project_pool);
+    auto flat_block = blocks.get_top_block_item().block.flatten();
+    auto board = Board::new_from_file(project->board_filename, flat_block, project_pool);
     board.expand();
-
-    {
-        auto items = block.get_pool_items_used();
-        items_needed.insert(items.begin(), items.end());
-    }
-    {
-        auto items = sch.get_pool_items_used();
-        items_needed.insert(items.begin(), items.end());
+    for (auto &[uu, block] : blocks.blocks) {
+        block.schematic.expand();
+        {
+            ItemSet items = block.block.get_pool_items_used();
+            items_needed.insert(items.begin(), items.end());
+        }
+        {
+            ItemSet items = block.schematic.get_pool_items_used();
+            items_needed.insert(items.begin(), items.end());
+        }
     }
     {
         auto items = board.get_pool_items_used();
