@@ -8,7 +8,7 @@ namespace horizon {
 
 PoolProjectManagerProcess::PoolProjectManagerProcess(const UUID &uu, PoolProjectManagerProcess::Type ty,
                                                      const std::vector<std::string> &args,
-                                                     const std::vector<std::string> &ienv, Pool *pool,
+                                                     const std::vector<std::string> &ienv, IPool *pool,
                                                      class PoolParametric *pool_parametric, bool read_only,
                                                      bool is_temp)
     : uuid(uu), type(ty)
@@ -22,7 +22,8 @@ PoolProjectManagerProcess::PoolProjectManagerProcess(const UUID &uu, PoolProject
     bool is_imp =
             any_of(type, {PoolProjectManagerProcess::Type::IMP_SYMBOL, PoolProjectManagerProcess::Type::IMP_PACKAGE,
                           PoolProjectManagerProcess::Type::IMP_PADSTACK, PoolProjectManagerProcess::Type::IMP_BOARD,
-                          PoolProjectManagerProcess::Type::IMP_SCHEMATIC, PoolProjectManagerProcess::Type::IMP_FRAME});
+                          PoolProjectManagerProcess::Type::IMP_SCHEMATIC, PoolProjectManagerProcess::Type::IMP_FRAME,
+                          PoolProjectManagerProcess::Type::IMP_DECAL});
     if (is_imp) { // imp
         std::vector<std::string> argv;
         std::vector<std::string> env = ienv;
@@ -58,6 +59,10 @@ PoolProjectManagerProcess::PoolProjectManagerProcess(const UUID &uu, PoolProject
             argv.push_back("-f");
             argv.insert(argv.end(), args.begin(), args.end());
             break;
+        case PoolProjectManagerProcess::Type::IMP_DECAL:
+            argv.push_back("-d");
+            argv.insert(argv.end(), args.begin(), args.end());
+            break;
         default:;
         }
         if (read_only)
