@@ -5,8 +5,7 @@
 #include "preferences_window_keys.hpp"
 #include "preferences_window_in_tool_keys.hpp"
 #include "preferences_window_canvas.hpp"
-#include "preferences_window_pool.hpp"
-#include "preferences_window_partinfo.hpp"
+#include "preferences_window_stock_info.hpp"
 #include "preferences_window_misc.hpp"
 #include "canvas/color_palette.hpp"
 #include "board/board_layers.hpp"
@@ -53,10 +52,9 @@ PreferencesWindow::PreferencesWindow(Preferences &prefs) : Gtk::Window(), prefer
         ed->show();
     }
     {
-        auto ed = PartinfoPreferencesEditor::create(preferences);
-        stack->add(*ed, "partinfo", "Partinfo");
+        auto ed = Gtk::manage(new StockInfoPreferencesEditor(preferences));
+        stack->add(*ed, "stockinfo", "Stock info");
         ed->show();
-        ed->unreference();
     }
     {
         auto ed = KeySequencesPreferencesEditor::create(preferences);
@@ -70,23 +68,11 @@ PreferencesWindow::PreferencesWindow(Preferences &prefs) : Gtk::Window(), prefer
         ed->show();
         ed->unreference();
     }
-    {
-        auto ed = PoolPreferencesEditor::create();
-        stack->add(*ed, "pools", "Pools");
-        ed->show();
-        pool_prefs_editor = ed;
-    }
 
     box->show();
     add(*box);
 }
 
-void PreferencesWindow::open_pool(const std::string &path)
-{
-    stack->set_visible_child("pools");
-    pool_prefs_editor->add_pool(path);
-}
-
 void PreferencesWindow::show_page(const std::string &pg)
 {
     stack->set_visible_child(pg);
