@@ -83,43 +83,36 @@ PreferencesRowBoolButton::PreferencesRowBoolButton(const std::string &title, con
     pack_start(*box, false, false, 0);
 }
 
-class PreferencesRowNumeric : public PreferencesRow {
+template <typename T> class PreferencesRowNumeric : public PreferencesRow {
 public:
-    PreferencesRowNumeric(const std::string &title, const std::string &subtitle, Preferences &prefs, float &v);
+    PreferencesRowNumeric(const std::string &title, const std::string &subtitle, Preferences &prefs, T &v)
+        : PreferencesRow(title, subtitle, prefs), value(v)
+    {
+        sp = Gtk::manage(new Gtk::SpinButton);
+        sp->set_valign(Gtk::ALIGN_CENTER);
+        sp->show();
+        pack_start(*sp, false, false, 0);
+    }
 
-    Gtk::SpinButton &get_spinbutton();
-    void bind();
+    Gtk::SpinButton &get_spinbutton()
+    {
+        return *sp;
+    }
+
+    void bind()
+    {
+        sp->set_value(value);
+        sp->signal_value_changed().connect([this] {
+            value = sp->get_value();
+            preferences.signal_changed().emit();
+        });
+    }
 
 private:
-    float &value;
+    T &value;
     Gtk::SpinButton *sp = nullptr;
 };
 
-PreferencesRowNumeric::PreferencesRowNumeric(const std::string &title, const std::string &subtitle, Preferences &prefs,
-                                             float &v)
-    : PreferencesRow(title, subtitle, prefs), value(v)
-{
-    sp = Gtk::manage(new Gtk::SpinButton);
-    sp->set_valign(Gtk::ALIGN_CENTER);
-    sp->show();
-    pack_start(*sp, false, false, 0);
-}
-
-void PreferencesRowNumeric::bind()
-{
-    sp->set_value(value);
-    sp->signal_value_changed().connect([this] {
-        value = sp->get_value();
-        preferences.signal_changed().emit();
-    });
-}
-
-Gtk::SpinButton &PreferencesRowNumeric::get_spinbutton()
-{
-    return *sp;
-}
-
-
 class PreferencesGroup : public Gtk::Box {
 public:
     PreferencesGroup(const std::string &title);
@@ -241,9 +234,9 @@ MiscPreferencesEditor::MiscPreferencesEditor(Preferences &prefs) : preferences(p
             gr->add_row(*r);
         }
         {
-            auto r =
-                    Gtk::manage(new PreferencesRowNumeric("Zoom factor", "How far to zoom in on each mouse wheel click",
-                                                          preferences, preferences.zoom.zoom_factor));
+            auto r = Gtk::manage(new PreferencesRowNumeric<float>("Zoom factor",
+                                                                  "How far to zoom in on each mouse wheel click",
+                                                                  preferences, preferences.zoom.zoom_factor));
             auto &sp = r->get_spinbutton();
             sp.set_range(10, 100);
             sp.set_increments(10, 10);
@@ -307,6 +300,28 @@ MiscPreferencesEditor::MiscPreferencesEditor(Preferences &prefs) : preferences(p
             gr->add_row(*r);
         }
     }
+    {
+        auto gr = Gtk::manage(new PreferencesGroup("Undo / Redo"));
+        box->pack_start(*gr, false, false, 0);
+        gr->show();
+        {
+            auto r = Gtk::manage(new PreferencesRowBool(
+                    "Show popups", "Briefly show what's been undone/redone when using keyboard shortcuts", preferences,
+                    preferences.undo_redo.show_hints));
+            gr->add_row(*r);
+        }
+        {
+            auto r = Gtk::manage(new PreferencesRowNumeric<unsigned int>("Undo/redo depth",
+                                                                         "How many undo/redo steps to store",
+                                                                         preferences, preferences.undo_redo.max_depth));
+            auto &sp = r->get_spinbutton();
+            sp.set_range(3, 100);
+            sp.set_increments(10, 10);
+            sp.set_width_chars(5);
+            r->bind();
+            gr->add_row(*r);
+        }
+    }
     {
         auto gr = Gtk::manage(new PreferencesGroup("Action Bar"));
         box->pack_start(*gr, false, false, 0);
@@ -329,6 +344,16 @@ MiscPreferencesEditor::MiscPreferencesEditor(Preferences &prefs) : preferences(p
             gr->add_row(*r);
         }
     }
+    {
+        auto gr = Gtk::manage(new PreferencesGroup("Appearance (also applies to Pool/Project Manager)"));
+        box->pack_start(*gr, false, false, 0);
+        gr->show();
+        {
+            auto r = Gtk::manage(new PreferencesRowBool("Dark theme", "Use dark theme variant if available",
+                                                        preferences, preferences.appearance.dark_theme));
+            gr->add_row(*r);
+        }
+    }
 }
 
 } // namespace horizon
