@@ -3,12 +3,12 @@
 #include "widgets/part_preview.hpp"
 #include "widgets/pool_browser_part.hpp"
 #include "widgets/pool_browser_parametric.hpp"
-#include "util/stock_info_provider_partinfo.hpp"
+#include "util/stock_info_provider.hpp"
 #include "preferences/preferences_provider.hpp"
 #include "util/win32_undef.hpp"
 #include "preferences/preferences.hpp"
 #include "util/gtk_util.hpp"
-#include "pool-prj-mgr/prj-mgr/pool_cache_status.hpp"
+#include "pool/pool_cache_status.hpp"
 
 namespace horizon {
 
@@ -74,13 +74,12 @@ PartBrowserWindow::PartBrowserWindow(BaseObjectType *cobject, const Glib::RefPtr
         la->show();
         add_search_menu->append(*la);
     }
-    notebook->signal_switch_page().connect(sigc::mem_fun(*this, &PartBrowserWindow::handle_switch_page));
     fav_toggled_conn =
             fav_button->signal_toggled().connect(sigc::mem_fun(*this, &PartBrowserWindow::handle_fav_toggled));
     place_part_button->signal_clicked().connect(sigc::mem_fun(*this, &PartBrowserWindow::handle_place_part));
     assign_part_button->signal_clicked().connect(sigc::mem_fun(*this, &PartBrowserWindow::handle_assign_part));
 
-    preview = Gtk::manage(new PartPreview(pool, false));
+    preview = Gtk::manage(new PartPreview(pool, false, "part_browser"));
     {
         Gtk::Box *box;
         x->get_widget("box", box);
@@ -96,7 +95,60 @@ PartBrowserWindow::PartBrowserWindow(BaseObjectType *cobject, const Glib::RefPtr
         add_search_parametric(it.first);
     }
     notebook->set_current_page(notebook->page_num(*ch_search));
-    signal_show().connect(sigc::track_obj([this, ch_search] { ch_search->search_once(); }, *ch_search));
+    signal_show().connect(sigc::track_obj([ch_search] { ch_search->search_once(); }, *ch_search));
+    signal_show().connect([this] {
+        if (needs_reload) {
+            needs_reload = false;
+            reload();
+        }
+    });
+
+    notebook->signal_switch_page().connect(sigc::mem_fun(*this, &PartBrowserWindow::handle_switch_page));
+
+    {
+        Gtk::Paned *paned;
+        GET_WIDGET(paned);
+        paned_state_store.emplace(paned, "part_browser");
+    }
+}
+
+void PartBrowserWindow::reload()
+{
+    pool.clear();
+    auto current_page = notebook->get_nth_page(notebook->get_current_page());
+    for (auto page : notebook->get_children()) {
+        if (auto ch = dynamic_cast<PoolBrowser *>(page)) {
+            ch->reload_pools();
+            if (current_page == ch)
+                ch->search();
+            else
+                ch->clear_search_once();
+        }
+    }
+}
+
+void PartBrowserWindow::pool_updated(const std::string &p)
+{
+    for (const auto &[it_path, it_uu] : pool.get_actually_included_pools(true)) {
+        if (it_path == p) {
+            if (!is_visible()) {
+                needs_reload = true;
+                return;
+            }
+
+            pool_updated_conn.disconnect();
+            // debounce since we might receive multiple updated signals in short succession
+            pool_updated_conn = Glib::signal_timeout().connect(sigc::track_obj(
+                                                                       [this] {
+                                                                           reload();
+                                                                           return false;
+                                                                       },
+                                                                       *this),
+                                                               500);
+
+            return;
+        }
+    }
 }
 
 void PartBrowserWindow::handle_favorites_selected(Gtk::ListBoxRow *row)
@@ -123,7 +175,6 @@ void PartBrowserWindow::handle_assign_part()
 
 void PartBrowserWindow::handle_fav_toggled()
 {
-    std::cout << "fav toggled" << std::endl;
     if (part_current) {
         if (fav_button->get_active()) {
             assert(std::count(favorites.begin(), favorites.end(), part_current) == 0);
@@ -138,9 +189,13 @@ void PartBrowserWindow::handle_fav_toggled()
     }
 }
 
-void PartBrowserWindow::handle_switch_page(Gtk::Widget *w, guint index)
+void PartBrowserWindow::handle_switch_page(Gtk::Widget *page, guint index)
 {
     update_part_current();
+    if (notebook->in_destruction())
+        return;
+    if (auto ch = dynamic_cast<PoolBrowser *>(page))
+        ch->search_once();
 }
 
 void PartBrowserWindow::placed_part(const UUID &uu)
@@ -312,7 +367,7 @@ void PartBrowserWindow::update_out_of_date_info_bar()
                     "INNER JOIN deps ON (symbols.unit = uuidx AND typex = 'unit')");
     q.bind(1, part_current);
     while (q.step()) {
-        ObjectType type = object_type_lut.lookup(q.get<std::string>(0));
+        auto type = q.get<ObjectType>(0);
         UUID uu(q.get<std::string>(1));
 
         auto r = std::find_if(pool_cache_status->items.begin(), pool_cache_status->items.end(),
@@ -336,9 +391,9 @@ void PartBrowserWindow::set_can_assign(bool v)
 
 PoolBrowserPart *PartBrowserWindow::add_search(const UUID &part)
 {
-    auto ch = Gtk::manage(new PoolBrowserPart(pool));
-    if (PreferencesProvider::get_prefs().partinfo.is_enabled()) {
-        auto prv = std::make_unique<StockInfoProviderPartinfo>(pool.get_base_path());
+    auto ch = Gtk::manage(new PoolBrowserPart(pool, UUID(), "part_browser"));
+    ch->set_include_base_parts(false);
+    if (auto prv = StockInfoProvider::create(pool.get_base_path())) {
         ch->add_stock_info_provider(std::move(prv));
     }
     ch->get_style_context()->add_class("background");
@@ -348,7 +403,7 @@ PoolBrowserPart *PartBrowserWindow::add_search(const UUID &part)
     auto close_button = Gtk::manage(new Gtk::Button());
     close_button->set_relief(Gtk::RELIEF_NONE);
     close_button->set_image_from_icon_name("window-close-symbolic");
-    close_button->signal_clicked().connect([this, ch] { delete ch; });
+    close_button->signal_clicked().connect([ch] { delete ch; });
     tab_label->pack_start(*close_button, false, false, 0);
     tab_label->pack_start(*la, true, true, 0);
     ch->show_all();
@@ -361,14 +416,14 @@ PoolBrowserPart *PartBrowserWindow::add_search(const UUID &part)
     ch->signal_activated().connect(sigc::mem_fun(*this, &PartBrowserWindow::handle_place_part));
     if (part)
         ch->go_to(part);
+    ch->focus_search();
     return ch;
 }
 
 PoolBrowserParametric *PartBrowserWindow::add_search_parametric(const std::string &table_name)
 {
-    auto ch = Gtk::manage(new PoolBrowserParametric(pool, pool_parametric, table_name));
-    if (PreferencesProvider::get_prefs().partinfo.is_enabled()) {
-        auto prv = std::make_unique<StockInfoProviderPartinfo>(pool.get_base_path());
+    auto ch = Gtk::manage(new PoolBrowserParametric(pool, pool_parametric, table_name, "part_browser"));
+    if (auto prv = StockInfoProvider::create(pool.get_base_path())) {
         ch->add_stock_info_provider(std::move(prv));
     }
     ch->add_copy_name_context_menu_item();
@@ -379,7 +434,7 @@ PoolBrowserParametric *PartBrowserWindow::add_search_parametric(const std::strin
     auto close_button = Gtk::manage(new Gtk::Button());
     close_button->set_relief(Gtk::RELIEF_NONE);
     close_button->set_image_from_icon_name("window-close-symbolic");
-    close_button->signal_clicked().connect([this, ch] { delete ch; });
+    close_button->signal_clicked().connect([ch] { delete ch; });
     tab_label->pack_start(*close_button, false, false, 0);
     tab_label->pack_start(*la, true, true, 0);
     ch->show_all();
@@ -391,17 +446,17 @@ PoolBrowserParametric *PartBrowserWindow::add_search_parametric(const std::strin
     ch->signal_selected().connect(sigc::mem_fun(*this, &PartBrowserWindow::update_part_current));
     ch->signal_activated().connect(sigc::mem_fun(*this, &PartBrowserWindow::handle_place_part));
 
-    notebook->signal_switch_page().connect(sigc::track_obj(
-            [this, ch](Gtk::Widget *page, guint pagenum) {
-                if (notebook->in_destruction())
-                    return;
-                if (page == ch)
-                    ch->search_once();
-            },
-            *ch));
     return ch;
 }
 
+void PartBrowserWindow::focus_search()
+{
+    auto page = notebook->get_nth_page(notebook->get_current_page());
+    if (auto br = dynamic_cast<PoolBrowserPart *>(page)) {
+        br->focus_search();
+    }
+}
+
 PartBrowserWindow *PartBrowserWindow::create(Gtk::Window *p, const std::string &pool_path, std::deque<UUID> &favs)
 {
     PartBrowserWindow *w;
