@@ -5,10 +5,12 @@
 #include "pool/pool_parametric.hpp"
 #include "util/uuid.hpp"
 #include "util/window_state_store.hpp"
+#include "util/paned_state_store.hpp"
 #include <array>
 #include <gtkmm.h>
 #include <set>
 #include "util/item_set.hpp"
+#include <optional>
 
 namespace horizon {
 
@@ -37,6 +39,8 @@ public:
     void go_to_part(const UUID &uu);
     void set_can_assign(bool v);
     void set_pool_cache_status(const class PoolCacheStatus &st);
+    void focus_search();
+    void pool_updated(const std::string &bp);
 
 private:
     Gtk::Menu *add_search_menu = nullptr;
@@ -76,5 +80,10 @@ private:
     const class PoolCacheStatus *pool_cache_status = nullptr;
 
     WindowStateStore state_store;
+    std::optional<PanedStateStore> paned_state_store;
+
+    sigc::connection pool_updated_conn;
+    bool needs_reload = false;
+    void reload();
 };
 } // namespace horizon
