@@ -7,12 +7,12 @@ namespace horizon {
 
 class WelcomeWindow : public Gtk::Window {
 public:
-    WelcomeWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, class PoolProjectManagerAppWindow *aw);
-    static WelcomeWindow *create(class PoolProjectManagerAppWindow *aw);
+    WelcomeWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, class PoolProjectManagerAppWindow &aw);
+    static WelcomeWindow *create(class PoolProjectManagerAppWindow &aw);
 
 
 private:
-    class PoolProjectManagerAppWindow *appwin;
+    class PoolProjectManagerAppWindow &appwin;
     Gtk::DrawingArea *banner_area = nullptr;
     Glib::RefPtr<Gdk::Pixbuf> pixbuf;
     Gtk::Button *button_back = nullptr;
