@@ -1,9 +1,11 @@
 #pragma once
 #include <string>
 #include <set>
+#include <map>
 
 namespace horizon {
 class UUID;
+enum class ObjectType;
 
 namespace SQLite {
 class Database;
@@ -28,6 +30,13 @@ public:
 
     virtual void clear() = 0;
 
+    virtual class PoolParametric *get_parametric() = 0;
+
+    virtual const class PoolInfo &get_pool_info() const = 0;
+
+    static const std::map<ObjectType, std::string> type_names;
+    virtual std::map<std::string, UUID> get_actually_included_pools(bool include_self) = 0;
+
     virtual ~IPool()
     {
     }
