@@ -93,7 +93,7 @@ const std::string &Pool::get_base_path() const
 
 int Pool::get_required_schema_version()
 { // keep in sync with schema definition
-    return 21;
+    return 24;
 }
 
 std::string Pool::get_tmp_filename(ObjectType type, const UUID &uu) const
@@ -344,5 +344,26 @@ std::map<std::string, UUID> Pool::get_actually_included_pools(bool include_self)
     return base_paths;
 }
 
+UUID Pool::get_installation_uuid()
+{
+    SQLite::Query q(db, "SELECT uuid FROM installation_uuid");
+    if (q.step())
+        return q.get<std::string>(0);
+    else
+        return UUID();
+}
+
+Pool::ItemPoolInfo Pool::get_pool_uuids(ObjectType ty, const UUID &uu)
+{
+    SQLite::Query q(db, "SELECT pool_uuid, last_pool_uuid FROM all_items_view WHERE type = ? AND uuid = ?");
+    q.bind(1, ty);
+    q.bind(2, uu);
+    if (q.step()) {
+        const UUID pool_uu(q.get<std::string>(0));
+        const UUID last_pool_uu(q.get<std::string>(1));
+        return {pool_uu, last_pool_uu};
+    }
+    throw std::runtime_error("item not found");
+}
 
 } // namespace horizon
