@@ -6,10 +6,8 @@
 #include "package.hpp"
 #include "package/pad.hpp"
 #include "decal.hpp"
-#include <fstream>
 #include <map>
 #include <set>
-#include <vector>
 #include "ipool.hpp"
 #include "pool_info.hpp"
 
@@ -78,6 +76,14 @@ public:
 
     std::map<std::string, UUID> get_actually_included_pools(bool include_self) override;
 
+    UUID get_installation_uuid();
+
+    struct ItemPoolInfo {
+        UUID pool;
+        UUID last;
+    };
+    ItemPoolInfo get_pool_uuids(ObjectType ty, const UUID &uu);
+
 protected:
     const std::string base_path;
     const PoolInfo pool_info;
