@@ -3,6 +3,7 @@
 #include "util/uuid.hpp"
 #include "nlohmann/json_fwd.hpp"
 #include <vector>
+#include "util/file_version.hpp"
 
 namespace horizon {
 using json = nlohmann::json;
@@ -12,6 +13,7 @@ public:
     static const UUID project_pool_uuid;
     PoolInfo(const std::string &bp);
     PoolInfo();
+    PoolInfo(const json &j);
     std::string base_path;
     UUID uuid;
     UUID default_via;
@@ -19,6 +21,11 @@ public:
     std::vector<UUID> pools_included;
     void save() const;
     bool is_project_pool() const;
-    void from_json(const json &j);
+    bool is_usable() const;
+    static unsigned int get_app_version();
+    FileVersion version;
+
+private:
+    PoolInfo(const json &j, const std::string &bp);
 };
 } // namespace horizon
