@@ -19,12 +19,6 @@ static const LutEnumStr<SymbolPin::Decoration::Driver> decoration_driver_lut = {
         {"tristate", SymbolPin::Decoration::Driver::TRISTATE},
 };
 
-static const LutEnumStr<SymbolPin::NameOrientation> name_orientation_lut = {
-        {"in_line", SymbolPin::NameOrientation::IN_LINE},
-        {"perpendicular", SymbolPin::NameOrientation::PERPENDICULAR},
-        {"horizontal", SymbolPin::NameOrientation::HORIZONTAL},
-};
-
 SymbolPin::Decoration::Decoration()
 {
 }
@@ -58,7 +52,7 @@ SymbolPin::SymbolPin(const UUID &uu, const json &j)
         }
     }
     else if (j.count("name_orientation")) {
-        name_orientation = name_orientation_lut.lookup(j.at("name_orientation"));
+        name_orientation = pin_name_orientation_lut.lookup(j.at("name_orientation"));
     }
 }
 
@@ -66,48 +60,9 @@ SymbolPin::SymbolPin(UUID uu) : uuid(uu)
 {
 }
 
-static const std::map<Orientation, Orientation> omap_90 = {
-        {Orientation::LEFT, Orientation::DOWN},
-        {Orientation::UP, Orientation::LEFT},
-        {Orientation::RIGHT, Orientation::UP},
-        {Orientation::DOWN, Orientation::RIGHT},
-};
-static const std::map<Orientation, Orientation> omap_180 = {
-        {Orientation::LEFT, Orientation::RIGHT},
-        {Orientation::UP, Orientation::DOWN},
-        {Orientation::RIGHT, Orientation::LEFT},
-        {Orientation::DOWN, Orientation::UP},
-};
-static const std::map<Orientation, Orientation> omap_270 = {
-        {Orientation::LEFT, Orientation::UP},
-        {Orientation::UP, Orientation::RIGHT},
-        {Orientation::RIGHT, Orientation::DOWN},
-        {Orientation::DOWN, Orientation::LEFT},
-};
-static const std::map<Orientation, Orientation> omap_mirror = {
-        {Orientation::LEFT, Orientation::RIGHT},
-        {Orientation::UP, Orientation::UP},
-        {Orientation::RIGHT, Orientation::LEFT},
-        {Orientation::DOWN, Orientation::DOWN},
-};
-
 Orientation SymbolPin::get_orientation_for_placement(const Placement &pl) const
 {
-    Orientation pin_orientation = orientation;
-    auto angle = pl.get_angle();
-    if (angle == 16384) {
-        pin_orientation = omap_90.at(pin_orientation);
-    }
-    if (angle == 32768) {
-        pin_orientation = omap_180.at(pin_orientation);
-    }
-    if (angle == 49152) {
-        pin_orientation = omap_270.at(pin_orientation);
-    }
-    if (pl.mirror) {
-        pin_orientation = omap_mirror.at(pin_orientation);
-    }
-    return pin_orientation;
+    return get_pin_orientation_for_placement(orientation, pl);
 }
 
 json SymbolPin::serialize() const
@@ -118,7 +73,7 @@ json SymbolPin::serialize() const
     j["orientation"] = orientation_lut.lookup_reverse(orientation);
     j["name_visible"] = name_visible;
     j["pad_visible"] = pad_visible;
-    j["name_orientation"] = name_orientation_lut.lookup_reverse(name_orientation);
+    j["name_orientation"] = pin_name_orientation_lut.lookup_reverse(name_orientation);
     j["decoration"] = decoration.serialize();
     return j;
 }
@@ -128,7 +83,7 @@ UUID SymbolPin::get_uuid() const
     return uuid;
 }
 
-static const unsigned int app_version = 0;
+static const unsigned int app_version = 1;
 
 unsigned int Symbol::get_app_version()
 {
@@ -195,6 +150,10 @@ Symbol::Symbol(const UUID &uu, const json &j, IPool &pool)
                 UUID u(it2.key());
                 if (texts.count(u)) {
                     Placement placement(it2.value());
+                    if (version.get_file() == 0) { // has broken text placement
+                        if (placement.mirror)
+                            placement.set_angle(-placement.get_angle());
+                    }
                     std::tuple<int, bool, UUID> key(angle, mirror, u);
                     text_placements[key] = placement;
                 }
@@ -356,15 +315,24 @@ void Symbol::apply_expand(const Symbol &ref, unsigned int n_expand)
     }
 }
 
+unsigned int Symbol::get_required_version() const
+{
+    if (text_placements.size())
+        return 1;
+    else
+        return 0;
+}
+
 json Symbol::serialize() const
 {
     json j;
-    version.serialize(j);
     j["type"] = "symbol";
     j["name"] = name;
     j["uuid"] = (std::string)uuid;
     j["unit"] = (std::string)unit->uuid;
     j["can_expand"] = can_expand;
+    if (const auto v = get_required_version())
+        j["version"] = v;
     j["junctions"] = json::object();
     for (const auto &it : junctions) {
         j["junctions"][(std::string)it.first] = it.second.serialize();
