@@ -635,6 +635,12 @@ int Reviewer::main(int c_argc, char *c_argv[])
                     ofs << " (inherited)";
                 ofs << "\n";
             }
+            if (part.override_prefix != Part::OverridePrefix::NO) {
+                ofs << "| Prefix | " << part.get_prefix();
+                if (part.override_prefix == Part::OverridePrefix::INHERIT)
+                    ofs << " (inherited)";
+                ofs << "\n";
+            }
             ofs << "\n\n";
 
             if (part.orderable_MPNs.size()) {
@@ -860,7 +866,7 @@ int Reviewer::main(int c_argc, char *c_argv[])
                 ofs << "\n";
             }
             {
-                auto &rule_cl = dynamic_cast<RuleClearancePackage &>(*pkg.rules.get_rule_nc(RuleID::CLEARANCE_PACKAGE));
+                auto &rule_cl = pkg.rules.get_rule_t<RuleClearancePackage>();
                 rule_cl.clearance_silkscreen_cu = 0.2_mm;
                 rule_cl.clearance_silkscreen_pkg = 0.2_mm;
                 auto r = pkg.rules.check(RuleID::CLEARANCE_PACKAGE, pkg);
@@ -1041,6 +1047,27 @@ int Reviewer::main(int c_argc, char *c_argv[])
             }
         }
     }
+
+    {
+        SQLite::Query q(pool->db, "SELECT DISTINCT uuid from git_files_view where type = 'decal'");
+        Once once;
+        while (q.step()) {
+            if (once()) {
+                ofs << "## Decals\n";
+            }
+            const auto uu = q.get<UUID>(0);
+            const auto &decal = *pool->get_decal(uu);
+
+            ofs << "### " << decal.name << "\n";
+            {
+                CanvasCairo2 ca;
+                ca.load(decal);
+                const std::string img_filename = "dec_" + static_cast<std::string>(decal.uuid) + ".png";
+                ca.get_image_surface(5)->write_to_png(Glib::build_filename(images_dir, img_filename));
+                ofs << "![Decal](" << images_prefix << img_filename << ")\n";
+            }
+        }
+    }
     return 0;
 }
 
@@ -1050,6 +1077,7 @@ int main(int c_argc, char *c_argv[])
     Gio::init();
     PoolManager::init();
     git_libgit2_init();
+    create_config_dir();
 
     Reviewer rev;
     return rev.main(c_argc, c_argv);
