@@ -71,6 +71,15 @@ public:
     json serialize() const;
 };
 
+class UndoRedoPreferences {
+public:
+    bool show_hints = true;
+    unsigned int max_depth = 50;
+
+    void load_from_json(const json &j);
+    json serialize() const;
+};
+
 class PartInfoPreferences {
 public:
     std::string url = "https://dev-partinfo.kitspace.org/graphql";
@@ -116,6 +125,14 @@ public:
     json serialize() const;
 };
 
+class AppearancePreferences {
+public:
+    bool dark_theme = false;
+
+    void load_from_json(const json &j);
+    json serialize() const;
+};
+
 class Preferences {
 public:
     Preferences();
@@ -143,6 +160,8 @@ public:
     ActionBarPreferences action_bar;
     InToolKeySequencesPreferences in_tool_key_sequences;
     MousePreferences mouse;
+    UndoRedoPreferences undo_redo;
+    AppearancePreferences appearance;
 
     bool show_pull_request_tools = false;
     bool hud_debug = false;
