@@ -6,6 +6,8 @@
 #include "widgets/spin_button_dim.hpp"
 #include "widgets/spin_button_angle.hpp"
 #include "util/util.hpp"
+#include "widgets/text_editor.hpp"
+#include "widgets/layer_combo_box.hpp"
 #include <algorithm>
 #include <iostream>
 #include <iomanip>
@@ -44,6 +46,7 @@ PropertyEditor::PropertyEditor(ObjectType t, ObjectProperty::ID prop, class Prop
 
     apply_all_button = Gtk::manage(new Gtk::ToggleButton());
     apply_all_button->set_label("All");
+    apply_all_button->get_style_context()->add_class("imp-property-all-button");
     apply_all_button->signal_toggled().connect([this] { s_signal_apply_all.emit(); });
     hbox->pack_start(*apply_all_button, false, false, 0);
 
@@ -117,7 +120,6 @@ void PropertyEditorString::activate()
         return;
     }
     modified = false;
-    std::string txt = en->get_text();
     s_signal_changed.emit();
     s_signal_activate.emit();
 }
@@ -255,7 +257,7 @@ PropertyValue &PropertyEditorNetClass::get_value()
 
 Gtk::Widget *PropertyEditorLayer::create_editor()
 {
-    combo = Gtk::manage(new Gtk::ComboBoxText());
+    combo = Gtk::manage(new LayerComboBox);
     connections.push_back(combo->signal_changed().connect(sigc::mem_fun(*this, &PropertyEditorLayer::changed)));
     return combo;
 }
@@ -266,9 +268,9 @@ void PropertyEditorLayer::reload()
     combo->remove_all();
     for (const auto &it : my_meta.layers) {
         if (!copper_only || it.second.copper)
-            combo->insert(0, std::to_string(it.first), it.second.name);
+            combo->prepend(it.second);
     }
-    combo->set_active_id(std::to_string(value.value));
+    combo->set_active_layer(value.value);
 }
 
 void PropertyEditorLayer::changed()
@@ -278,9 +280,9 @@ void PropertyEditorLayer::changed()
 
 PropertyValue &PropertyEditorLayer::get_value()
 {
-    auto active_id = combo->get_active_id();
-    if (active_id.size())
-        value.value = std::stoi(active_id);
+    const auto active_layer = combo->get_active_layer();
+    if (active_layer != 10000)
+        value.value = active_layer;
     return value;
 }
 
@@ -308,45 +310,37 @@ PropertyValue &PropertyEditorAngle::get_value()
 
 Gtk::Widget *PropertyEditorStringMultiline::create_editor()
 {
-    en = Gtk::manage(new Gtk::TextView());
-    en->set_left_margin(3);
-    en->set_right_margin(3);
-    en->set_top_margin(3);
-    en->set_bottom_margin(3);
+    editor = Gtk::manage(new TextEditor(TextEditor::Lines::MULTI));
     connections.push_back(
-            en->get_buffer()->signal_changed().connect(sigc::mem_fun(*this, &PropertyEditorStringMultiline::changed)));
-    en->signal_focus_out_event().connect(sigc::mem_fun(*this, &PropertyEditorStringMultiline::focus_out_event));
-    auto sc = Gtk::manage(new Gtk::ScrolledWindow());
-    sc->set_shadow_type(Gtk::SHADOW_IN);
-    sc->add(*en);
-    en->show();
-    return sc;
+            editor->signal_changed().connect(sigc::mem_fun(*this, &PropertyEditorStringMultiline::changed)));
+    connections.push_back(
+            editor->signal_activate().connect(sigc::mem_fun(*this, &PropertyEditorStringMultiline::activate)));
+    connections.push_back(
+            editor->signal_lost_focus().connect(sigc::mem_fun(*this, &PropertyEditorStringMultiline::activate)));
+    return editor;
 }
 
 void PropertyEditorStringMultiline::activate()
 {
-    if (!modified)
+    if (!modified) {
+        s_signal_activate.emit();
         return;
+    }
     modified = false;
     s_signal_changed.emit();
-}
-
-bool PropertyEditorStringMultiline::focus_out_event(GdkEventFocus *e)
-{
-    activate();
-    return false;
+    s_signal_activate.emit();
 }
 
 void PropertyEditorStringMultiline::reload()
 {
     ScopedBlock block(connections);
-    en->get_buffer()->set_text(value.value);
+    editor->set_text(value.value, TextEditor::Select::NO);
     modified = false;
 }
 
 PropertyValue &PropertyEditorStringMultiline::get_value()
 {
-    value.value = en->get_buffer()->get_text();
+    value.value = editor->get_text();
     return value;
 }
 
@@ -367,7 +361,8 @@ Gtk::Widget *PropertyEditorInt::create_editor()
     sp = Gtk::manage(new Gtk::SpinButton());
     sp->set_range(0, 65536);
     sp->set_width_chars(7);
-    connections.push_back(sp->signal_value_changed().connect(sigc::mem_fun(*this, &PropertyEditorInt::changed)));
+    connections.push_back(sp->signal_value_changed().connect([this] { s_signal_changed.emit(); }));
+    sp->signal_activate().connect([this] { s_signal_activate.emit(); });
     return sp;
 }
 
@@ -377,11 +372,6 @@ void PropertyEditorInt::reload()
     sp->set_value(value.value);
 }
 
-void PropertyEditorInt::changed()
-{
-    s_signal_changed.emit();
-}
-
 PropertyValue &PropertyEditorInt::get_value()
 {
     value.value = sp->get_value_as_int();
