@@ -3,7 +3,8 @@
 #include "common/common.hpp"
 #include "pool/entity.hpp"
 #include "util/uuid.hpp"
-#include <deque>
+#include "util/uuid_vec.hpp"
+#include <vector>
 #include <map>
 #include <memory>
 #include <mutex>
@@ -20,8 +21,8 @@ public:
 
 class RulesCheckCacheBoardImage : public RulesCheckCacheBase {
 public:
-    RulesCheckCacheBoardImage(class IDocument *c);
-    const CanvasPatch *get_canvas() const;
+    RulesCheckCacheBoardImage(class IDocument &c);
+    const CanvasPatch &get_canvas() const;
 
 private:
     CanvasPatch canvas;
@@ -29,26 +30,36 @@ private:
 
 class RulesCheckCacheNetPins : public RulesCheckCacheBase {
 public:
-    RulesCheckCacheNetPins(class IDocument *c);
-    const std::map<class Net *,
-                   std::deque<std::tuple<class Component *, const class Gate *, const class Pin *, UUID, Coordi>>> &
-    get_net_pins() const;
+    RulesCheckCacheNetPins(class IDocument &c);
+    struct NetPin {
+        UUID comp;
+        const class Gate &gate;
+        const class Pin &pin;
+        UUID sheet;
+        UUIDVec instance_path;
+        Coordi location;
+    };
+    struct NetInfo {
+        std::string name;
+        bool is_nc = false;
+        std::vector<NetPin> pins;
+    };
+    using NetPins = std::map<UUID, NetInfo>;
+    const NetPins &get_net_pins() const;
 
 private:
-    std::map<class Net *,
-             std::deque<std::tuple<class Component *, const class Gate *, const class Pin *, UUID, Coordi>>>
-            net_pins;
+    NetPins net_pins;
 };
 
 class RulesCheckCache {
 public:
-    RulesCheckCache(class IDocument *c);
-    RulesCheckCacheBase *get_cache(RulesCheckCacheID id);
+    RulesCheckCache(class IDocument &c);
+    RulesCheckCacheBase &get_cache(RulesCheckCacheID id);
     void clear();
 
 private:
     std::map<RulesCheckCacheID, std::unique_ptr<RulesCheckCacheBase>> cache;
-    class IDocument *core;
+    class IDocument &core;
     std::mutex mutex;
 };
 } // namespace horizon
