@@ -4,6 +4,7 @@
 #include "nlohmann/json_fwd.hpp"
 #include "rule.hpp"
 #include "util/uuid.hpp"
+#include "util/uuid_vec.hpp"
 #include <deque>
 #include <set>
 #include <functional>
@@ -23,6 +24,7 @@ public:
 
     RulesCheckErrorLevel level = RulesCheckErrorLevel::NOT_RUN;
     UUID sheet;
+    UUIDVec instance_path;
     Coordi location;
     std::string comment;
     bool has_location = false;
@@ -57,17 +59,36 @@ public:
 
 
     virtual json serialize() const = 0;
-    virtual std::set<RuleID> get_rule_ids() const = 0;
+    virtual std::vector<RuleID> get_rule_ids() const = 0;
 
-    virtual const Rule *get_rule(RuleID id) const = 0;
-    Rule *get_rule(RuleID id);
-    Rule *get_rule_nc(RuleID id)
+    virtual const Rule &get_rule(RuleID id) const = 0;
+    Rule &get_rule(RuleID id);
+    Rule &get_rule_nc(RuleID id)
     {
         return get_rule(id);
     }
 
-    virtual const Rule *get_rule(RuleID id, const UUID &uu) const = 0;
-    Rule *get_rule(RuleID id, const UUID &uu);
+    template <typename T> const T &get_rule_t() const
+    {
+        return dynamic_cast<const T &>(get_rule(T::id));
+    }
+
+    template <typename T> T &get_rule_t()
+    {
+        return dynamic_cast<T &>(get_rule(T::id));
+    }
+
+    virtual const Rule &get_rule(RuleID id, const UUID &uu) const = 0;
+    Rule &get_rule(RuleID id, const UUID &uu);
+
+    template <typename T> const T &get_rule_t(const UUID &uu) const
+    {
+        return dynamic_cast<const T &>(get_rule(T::id, uu));
+    }
+    template <typename T> T &get_rule_t(const UUID &uu)
+    {
+        return dynamic_cast<T &>(get_rule(T::id, uu));
+    }
 
     virtual std::map<UUID, const Rule *> get_rules(RuleID id) const = 0;
     std::map<UUID, Rule *> get_rules(RuleID id);
@@ -88,6 +109,11 @@ public:
         return rv;
     }
 
+    template <typename T> std::vector<const T *> get_rules_sorted() const
+    {
+        return get_rules_sorted<T>(T::id);
+    }
+
     template <typename T = Rule> std::vector<T *> get_rules_sorted(RuleID id)
     {
         std::vector<T *> r;
@@ -97,8 +123,17 @@ public:
         return r;
     }
 
+    template <typename T> std::vector<T *> get_rules_sorted()
+    {
+        return get_rules_sorted<T>(T::id);
+    }
+
     virtual void remove_rule(RuleID id, const UUID &uu) = 0;
-    virtual Rule *add_rule(RuleID id) = 0;
+    template <typename T> T &add_rule_t()
+    {
+        return dynamic_cast<T &>(add_rule(T::id));
+    }
+    virtual Rule &add_rule(RuleID id) = 0;
     void move_rule(RuleID id, const UUID &uu, int dir);
 
     virtual ~Rules();
