@@ -6,7 +6,7 @@
 namespace horizon {
 NetLabel::NetLabel(const UUID &uu, const json &j, Sheet *sheet)
     : uuid(uu), orientation(orientation_lut.lookup(j.at("orientation"))), size(j.value("size", 2500000)),
-      offsheet_refs(j.value("offsheet_refs", true))
+      offsheet_refs(j.value("offsheet_refs", true)), show_port(j.value("show_port", false))
 {
     if (sheet)
         junction = &sheet->junctions.at(j.at("junction").get<std::string>());
@@ -25,6 +25,7 @@ json NetLabel::serialize() const
     j["orientation"] = orientation_lut.lookup_reverse(orientation);
     j["size"] = size;
     j["offsheet_refs"] = offsheet_refs;
+    j["show_port"] = show_port;
     return j;
 }
 } // namespace horizon
