@@ -6,13 +6,11 @@
 namespace horizon {
 RuleSinglePinNet::RuleSinglePinNet() : Rule()
 {
-    id = RuleID::SINGLE_PIN_NET;
 }
 
 RuleSinglePinNet::RuleSinglePinNet(const json &j) : Rule(j)
 {
-    id = RuleID::SINGLE_PIN_NET;
-    include_unnamed = j.at("include_unnamed");
+    include_unnamed = j.at("include_unnamed").get<bool>();
 }
 
 json RuleSinglePinNet::serialize() const
@@ -22,7 +20,7 @@ json RuleSinglePinNet::serialize() const
     return j;
 }
 
-std::string RuleSinglePinNet::get_brief(const class Block *block) const
+std::string RuleSinglePinNet::get_brief(const class Block *block, class IPool *pool) const
 {
     return "";
 }
