@@ -11,6 +11,7 @@
 #include <map>
 #include <fstream>
 #include "util/file_version.hpp"
+#include "util/template_util.hpp"
 
 namespace horizon {
 using json = nlohmann::json;
@@ -31,8 +32,10 @@ private:
     unsigned int update_nets();
 
 public:
-    Schematic(const UUID &uu, const json &, Block &block, class IPool &pool);
-    static Schematic new_from_file(const std::string &filename, Block &block, IPool &pool);
+    Schematic(const UUID &uu, const json &, Block &block, class IPool &pool,
+              class IBlockSymbolAndSchematicProvider &prv);
+    static Schematic new_from_file(const std::string &filename, Block &block, IPool &pool,
+                                   class IBlockSymbolAndSchematicProvider &prv);
     Schematic(const UUID &uu, Block &block);
     static unsigned int get_app_version();
 
@@ -41,7 +44,8 @@ public:
      * \param carful when true, superfluous things will get cleaned up. Don't do
      * this when you may hold pointers to these.
      */
-    void expand(bool careful = false);
+    void expand(bool careful = false, const class IInstanceMappingProvider *inst_map = nullptr);
+    void expand_connectivity(bool carful = false);
 
     Schematic(const Schematic &sch);
     void operator=(const Schematic &sch) = delete;
@@ -66,6 +70,7 @@ public:
      * coincident with pins.
      */
     void autoconnect_symbol(Sheet *sheet, SchematicSymbol *sym);
+    void autoconnect_block_symbol(Sheet *sheet, SchematicBlockSymbol *sym);
 
     /**
      * Turns sym's texts to regular text objects.
@@ -83,8 +88,15 @@ public:
 
     void swap_gates(const UUID &comp, const UUID &g1, const UUID &g2);
 
+    void disconnect_block_symbol(Sheet *sheet, SchematicBlockSymbol *sym);
+
     std::map<UUIDPath<2>, std::string> get_unplaced_gates() const;
 
+    Sheet &add_sheet();
+    void delete_sheet(const UUID &uu);
+    Sheet &get_sheet_at_index(unsigned int index);
+    const Sheet &get_sheet_at_index(unsigned int index) const;
+
     static Glib::RefPtr<Glib::Regex> get_sheetref_regex();
 
     UUID uuid;
@@ -114,14 +126,47 @@ public:
     Annotation annotation;
     void annotate();
 
+    class SheetMapping {
+    public:
+        std::map<UUIDVec, unsigned int> sheet_numbers;
+        unsigned int sheet_total;
+        void update(const Schematic &sch);
+
+    private:
+        void update(const Schematic &sch, const UUIDVec &instance_path);
+        unsigned int index;
+    };
+    SheetMapping sheet_mapping;
+
+    void update_sheet_mapping();
+
+    template <bool c> struct SheetItem {
+        SheetItem(make_const_ref_t<c, Sheet> sh, unsigned int i, make_const_ref_t<c, Schematic> sch, const UUIDVec &p)
+            : sheet(sh), sheet_index(i), schematic(sch), instance_path(p)
+        {
+        }
+        make_const_ref_t<c, Sheet> sheet;
+        unsigned int sheet_index;
+        make_const_ref_t<c, Schematic> schematic;
+        UUIDVec instance_path;
+    };
+
+    std::vector<SheetItem<false>> get_all_sheets();
+    std::vector<SheetItem<true>> get_all_sheets() const;
+
     PDFExportSettings pdf_export_settings;
 
     FileVersion version;
 
     json serialize() const;
-    void save_pictures(const std::string &dir) const;
     void load_pictures(const std::string &dir);
 
     ItemSet get_pool_items_used() const;
+
+    std::vector<Sheet *> get_sheets_sorted();
+    std::vector<const Sheet *> get_sheets_sorted() const;
+
+private:
+    void expand_frames(const IInstanceMappingProvider *inst_map);
 };
 } // namespace horizon
