@@ -15,12 +15,12 @@ public:
     RulesCheckResult check(RuleID id, const class Schematic &sch, class RulesCheckCache &cache) const;
     void apply(RuleID id, class Schematic *sch);
     json serialize() const override;
-    std::set<RuleID> get_rule_ids() const override;
-    const Rule *get_rule(RuleID id) const override;
-    const Rule *get_rule(RuleID id, const UUID &uu) const override;
+    std::vector<RuleID> get_rule_ids() const override;
+    const Rule &get_rule(RuleID id) const override;
+    const Rule &get_rule(RuleID id, const UUID &uu) const override;
     std::map<UUID, const Rule *> get_rules(RuleID id) const override;
     void remove_rule(RuleID id, const UUID &uu) override;
-    Rule *add_rule(RuleID id) override;
+    Rule &add_rule(RuleID id) override;
 
 private:
     RuleSinglePinNet rule_single_pin_net;
