@@ -3,6 +3,12 @@
 #include "str_util.hpp"
 #include "common/common.hpp"
 
+#ifdef G_OS_WIN32
+#include <windows.h>
+#include <shellapi.h>
+#include <gdk/gdkwin32.h>
+#endif
+
 namespace horizon {
 void bind_widget(Gtk::Switch *sw, bool &v)
 {
@@ -369,4 +375,17 @@ void spinbutton_connect_activate(Gtk::SpinButton *sp, std::function<void()> cb)
             false);
 }
 
+void open_directory(Gtk::Window &win, const std::string &filename)
+{
+#ifdef G_OS_WIN32
+    auto wnd = reinterpret_cast<HWND>(GDK_WINDOW_HWND(win.get_window()->gobj()));
+    auto wfilename = reinterpret_cast<wchar_t *>(g_utf8_to_utf16(filename.c_str(), -1, NULL, NULL, NULL));
+    ShellExecuteW(wnd, NULL, wfilename, NULL, NULL, SW_SHOWNORMAL);
+    g_free(wfilename);
+#else
+    auto uri = Gio::File::create_for_path(Glib::path_get_dirname(filename))->get_uri();
+    Gio::AppInfo::launch_default_for_uri(uri);
+#endif
+}
+
 } // namespace horizon
