@@ -30,22 +30,17 @@ public:
     bool step();
     template <class T> T get(int idx) const
     {
-        return get(idx, T());
-    }
-
-    template <class T> struct convert {
-        using to_int = int;
-    };
-
-    template <class... Ts> std::tuple<Ts...> get_columns(typename convert<Ts>::to_int... idxs) const
-    {
-        return std::make_tuple(get(idxs, Ts())...);
+        T r;
+        get(idx, r);
+        return r;
     }
 
     void bind(int idx, const std::string &v, bool copy = true);
     void bind(const char *name, const std::string &v, bool copy = true);
     void bind(int idx, int v);
     void bind(const char *name, int v);
+    void bind_int64(int idx, sqlite3_int64 v);
+    void bind_int64(const char *name, sqlite3_int64 v);
     void bind(int idx, const horizon::UUID &v);
     void bind(const char *name, const horizon::UUID &v);
     void bind(int idx, ObjectType type);
@@ -56,9 +51,11 @@ private:
     class Database &db;
     sqlite3_stmt *stmt;
 
-    std::string get(int idx, std::string) const;
-    int get(int idx, int) const;
-    ObjectType get(int idx, ObjectType) const;
+    void get(int idx, std::string &r) const;
+    void get(int idx, UUID &r) const;
+    void get(int idx, int &r) const;
+    void get(int idx, sqlite3_int64 &r) const;
+    void get(int idx, ObjectType &r) const;
 };
 
 class Error : public std::runtime_error {
