@@ -145,7 +145,7 @@ static const std::vector<unsigned int> &get_font_data(TextData::Font font)
 
 static unsigned int codepoint_to_hershey(gunichar c, TextData::Font font)
 {
-    auto font_data = get_font_data(font);
+    const auto &font_data = get_font_data(font);
     unsigned int x = c - ' ';
     if (x >= 0 && x < font_data.size()) {
         return font_data.at(x);
@@ -178,8 +178,9 @@ static unsigned int codepoint_to_hershey(gunichar c, TextData::Font font)
 
 extern const char *hershey_glyphs[];
 
-TextData::TextData(const std::string &str, Font font)
+TextData::TextData(Buffer &buf, const std::string &str, Font font) : lines(buf)
 {
+    lines.clear();
     Glib::ustring ustr(str);
     int x0 = 0;
     lines.reserve(ustr.size());
