@@ -1,5 +1,7 @@
 #include "uuid.hpp"
 #include <string.h>
+#include <glibmm/checksum.h>
+#include <array>
 
 namespace horizon {
 
@@ -32,6 +34,25 @@ UUID::UUID(const std::string &str)
     }
 }
 
+UUID UUID::UUID5(const UUID &nsid, const unsigned char *name, size_t name_size)
+{
+    UUID uu;
+    Glib::Checksum chk(Glib::Checksum::CHECKSUM_SHA1);
+    chk.update(nsid.uu, sizeof(nsid.uu));
+    chk.update(name, name_size);
+    std::array<unsigned char, 20> buf;
+    gsize length = buf.size();
+    chk.get_digest(buf.data(), &length);
+    memcpy(uu.uu, buf.data(), sizeof(uu.uu));
+    // set version 5 (sha1)
+    uu.uu[6] &= ~0xf0;
+    uu.uu[6] |= 0x50;
+    // set variant to RFC 4122
+    uu.uu[8] &= ~0xc0;
+    uu.uu[8] |= 0x80;
+    return uu;
+}
+
 bool operator==(const UUID &self, const UUID &other)
 {
     return memcmp(self.uu, other.uu, sizeof(self.uu)) == 0;
