@@ -7,6 +7,8 @@ class LocationEntry : public Gtk::Box {
 public:
     LocationEntry(const std::string &rel = "");
     void set_filename(const std::string &s);
+    void set_rel_filename(const std::string &s);
+    void set_relative_to(const std::string &s);
     std::string get_filename();
     void set_warning(const std::string &t);
 
@@ -17,11 +19,13 @@ public:
     }
 
     bool check_ends_json(bool *v = nullptr);
+    void set_append_json(bool s);
 
 private:
-    const std::string relative_to;
+    std::string relative_to;
     std::string get_rel_filename(const std::string &s) const;
     Gtk::Entry *entry = nullptr;
+    bool autoappend_json = false;
     void handle_button();
 
     type_signal_changed s_signal_changed;
