@@ -2,7 +2,7 @@
 #include "block/block.hpp"
 
 namespace horizon {
-NetClassButton::NetClassButton(Block *c) : Gtk::ComboBoxText(), block(c)
+NetClassButton::NetClassButton(const Block &c) : Gtk::ComboBoxText(), block(c)
 {
     update();
     signal_changed().connect([this] {
@@ -19,7 +19,7 @@ void NetClassButton::set_net_class(const UUID &uu)
 
 void NetClassButton::update()
 {
-    for (const auto &it : block->net_classes) {
+    for (const auto &it : block.net_classes) {
         insert(0, (std::string)it.first, it.second.name);
     }
 }
