@@ -8,6 +8,7 @@
 #include "tag_entry.hpp"
 #include "util/sqlite.hpp"
 #include "pool_selector.hpp"
+#include "util/sort_controller.hpp"
 
 namespace horizon {
 PoolBrowser::PoolBrowser(IPool &p, const std::string &prefix)
@@ -124,11 +125,24 @@ void PoolBrowser::install_column_tooltip(Gtk::TreeViewColumn &tvc, const Gtk::Tr
                     Gtk::TreeIter iter(treeview->get_model()->get_iter(path));
                     if (!iter)
                         return false;
-                    Glib::ustring val;
-                    iter->get_value(col.index(), val);
-                    tooltip->set_text(val);
-                    treeview->set_tooltip_row(tooltip, path);
-                    return true;
+
+                    auto cells = column->get_cells();
+                    for (auto cell : cells) {
+                        int start, width;
+                        if (column->get_cell_position(*cell, start, width)) {
+                            if ((cell_x >= start) && (cell_x < (start + width))) {
+                                // only show tooltip when on text, avoids conflict with item source cell renderer
+                                if (dynamic_cast<Gtk::CellRendererText *>(cell)) {
+                                    Glib::ustring val;
+                                    iter->get_value(col.index(), val);
+                                    tooltip->set_text(val);
+                                    treeview->set_tooltip_row(tooltip, path);
+                                    return true;
+                                }
+                            }
+                        }
+                    }
+                    return false;
                 }
                 return false;
             });
@@ -248,8 +262,13 @@ void PoolBrowser::construct(Gtk::Widget *search_box)
     sort_controller->set_sort(0, SortController::Sort::ASC);
     sort_controller->signal_changed().connect(sigc::mem_fun(*this, &PoolBrowser::search));
 
-    // dynamic_cast<Gtk::CellRendererText*>(view->get_column_cell_renderer(3))->property_ellipsize()
-    // = Pango::ELLIPSIZE_END;
+    {
+        auto la = Gtk::manage(new Gtk::MenuItem("Sort most recently modified first"));
+        la->show();
+        header_context_menu.append(*la);
+        la->signal_activate().connect(sigc::mem_fun(*this, &PoolBrowser::sort_by_mtime));
+    }
+
     treeview->get_selection()->set_mode(Gtk::SelectionMode::SELECTION_BROWSE);
     treeview->signal_row_activated().connect(sigc::mem_fun(*this, &PoolBrowser::row_activated));
     treeview->get_selection()->signal_changed().connect(sigc::mem_fun(*this, &PoolBrowser::selection_changed));
@@ -257,22 +276,33 @@ void PoolBrowser::construct(Gtk::Widget *search_box)
         path_column->set_visible(false);
     }
     treeview->signal_button_press_event().connect_notify([this](GdkEventButton *ev) {
-        if (ev->button == 3 && context_menu.get_children().size()) {
-            Gtk::TreeModel::Path path;
-            if (treeview->get_bin_window()->gobj() == ev->window && treeview->get_path_at_pos(ev->x, ev->y, path)) {
-                if (auto it = store->get_iter(path)) {
-                    Gtk::TreeModel::Row row = *it;
-                    auto sel = uuid_from_row(row);
-                    for (auto &[widget, cb] : menu_item_sensitive_cbs) {
-                        widget->set_visible(!sel || cb(sel));
-                    }
+        if (gdk_event_triggers_context_menu((GdkEvent *)ev)) {
+            if (treeview->get_bin_window()->gobj() == ev->window) {
+                if (context_menu.get_children().size()) {
+                    Gtk::TreeModel::Path path;
+                    if (treeview->get_path_at_pos(ev->x, ev->y, path)) {
+                        if (auto it = store->get_iter(path)) {
+                            Gtk::TreeModel::Row row = *it;
+                            auto sel = uuid_from_row(row);
+                            for (auto &[widget, cb] : menu_item_sensitive_cbs) {
+                                widget->set_visible(!sel || cb(sel));
+                            }
 #if GTK_CHECK_VERSION(3, 22, 0)
-                    context_menu.popup_at_pointer((GdkEvent *)ev);
+                            context_menu.popup_at_pointer((GdkEvent *)ev);
 #else
-                    context_menu.popup(ev->button, gtk_get_current_event_time());
+                            context_menu.popup(ev->button, gtk_get_current_event_time());
 #endif
+                        }
+                    }
                 }
             }
+            else {
+#if GTK_CHECK_VERSION(3, 22, 0)
+                header_context_menu.popup_at_pointer((GdkEvent *)ev);
+#else
+                header_context_menu.popup(ev->button, gtk_get_current_event_time());
+#endif
+            }
         }
     });
 
@@ -561,4 +591,11 @@ void PoolBrowser::bind_tags_query(SQLite::Query &q, const std::set<std::string>
     }
 }
 
+PoolBrowser::~PoolBrowser() = default;
+
+void PoolBrowser::sort_by_mtime()
+{
+    sort_controller->set_sort(mtime_column, SortController::Sort::DESC);
+}
+
 } // namespace horizon
