@@ -3,6 +3,7 @@
 #include "pool/part.hpp"
 #include "util/util.hpp"
 #include "util/geom_util.hpp"
+#include "util/sort_controller.hpp"
 
 namespace horizon {
 
@@ -319,6 +320,7 @@ void PoolBrowserParametric::add_sort_controller_columns()
         auto &col = table.columns.at(i);
         sort_controller->add_column(3 + i, "p." + col.name);
     }
+    sort_controller->add_column(mtime_column, "parts.mtime");
 }
 
 static std::string get_in(const std::string &prefix, size_t n)
